/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse.node;

import com.abssqr.plat.common.facade.enums.AnalyseServiceEnum;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.core.service.util.AnalyseReportUtil;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;

/**
 * 早偿率计算
 * @author hanfei
 * @version $Id: AnalyseChartAna102Node.java, v 0.1 2019-10-09 11:26 PM hanfei Exp $
 */
@Component
public class AnalyseChartAna102Node extends AnalyseChartNodeBase {
    
    /**
     * 放款池、承接池早偿金额和 (WK042+WK043) / 放款池、承接吃资产剩余本金和 (WK017)
     * @param context
     * @return
     */
    @Override
    protected Void handle(AnalyseChartContext context) {
        List<ReportField> amtFields = Lists.newArrayList(
                WkAstReportFields.WK042,
                WkAstReportFields.WK043,
                WkAstReportFields.WK017
        );
        super.handleDayTrendChart(context, amtFields, true, (acctDate, itemMap) -> {
            List<ReportField> repayFields = Lists.newArrayList(
                    WkAstReportFields.WK042,
                    WkAstReportFields.WK043
            );
            // 计算早偿率 = 放款池、承接池早偿金额和 (WK042+WK043) / 放款池、承接吃资产剩余本金和 (WK017)
            BigDecimal denominator = AnalyseReportUtil.getItemBig(itemMap, WkAstReportFields.WK017);
            return CalcUtil.dividePercRate(AnalyseReportUtil.getItemBig(itemMap, repayFields), denominator);
        });
        return null;
    }

    @Override
    protected AnalyseServiceEnum getServiceEnum(){
        return AnalyseServiceEnum.DAY_PRE;
    }
}
