/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse.node;

import cn.hutool.core.date.DateUtil;
import com.abssqr.plat.common.facade.enums.AnalyseServiceEnum;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.result.analyse.AnalyseChartItemVO;
import com.abssqr.plat.core.service.report.AnalyseSpvReportManager;
import com.general.system.tool.util.ToolUtil;
import com.general.system.util.adapter.selector.AbstractMultipleActionSelectorNode;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 统计图节点基类
 * @author hanfei
 * @version $Id: AnalyseChartNodeBase.java, v 0.1 2019-09-26 12:03 AM hanfei Exp $ 
 */
public abstract class AnalyseChartNodeBase
        extends AbstractMultipleActionSelectorNode<AnalyseChartContext, AnalyseServiceEnum> {

    @Autowired
    protected AnalyseSpvReportManager analyseSpvReportManager;
    
    @Override
    public boolean judge(AnalyseChartContext context) {
        return context.getServices().contains(getServiceEnum());
    }
    
    /**
     * 按日期趋势统计基础方法
     * @param context
     * @param statFields
     */
    public void handleDayTrendChart(AnalyseChartContext context, List<ReportField> statFields,
                                    boolean havePlan, AnalyseValueHandle valueHandle){
        // 获取放款池计划 按日期与统计项的分组汇总数据
        Map<Date, List<PlanReportItem>> planItemMap = analyseSpvReportManager.querySpvStatDaySectionItem(
                context.getStartDate(), context.getEndDate(), context.getPlanBase().getPlanNo(), statFields, havePlan);
        if (ToolUtil.isEmpty(planItemMap)) {
            return;
        }
        List<AnalyseChartItemVO> itemVOS = new ArrayList<>();
        for (Map.Entry<Date, List<PlanReportItem>> entry : planItemMap.entrySet()) {
            Map<String, PlanReportItem> itemMap = entry.getValue().stream().collect(
                    Collectors.toMap(PlanReportItem::getRptItemCode, item -> item));
            AnalyseChartItemVO itemVO = new AnalyseChartItemVO();
            // X 轴按日期
            itemVO.setEntry(DateUtil.formatDate(entry.getKey()));
            itemVO.setItem(getServiceEnum().getDesc());
            // 处理统计值
            itemVO.setValue(valueHandle.handle(entry.getKey(), itemMap));
            itemVOS.add(itemVO);
        }
        // 按X轴 日期 进行排序
        itemVOS.sort(Comparator.comparing(item -> DateUtil.parse(item.getEntry())));
        context.addItems(itemVOS);
    }
    
    /**
     * 执行统计业务项目
     * @return
     */
    protected abstract AnalyseServiceEnum getServiceEnum();
    
    
    protected interface AnalyseValueHandle{
        /**
         * 对应每日统计值处理
         * @param itemMap
         * @return
         */
        BigDecimal handle(Date acctDate, Map<String, PlanReportItem> itemMap);
    }
}
