/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.prod;

import com.abssqr.plat.biz.support.OprTemplate;
import com.abssqr.plat.biz.support.QryTemplate;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.ProductDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.ProductListPage;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.param.prod.ProdEditParam;
import com.abssqr.plat.common.facade.param.prod.ProdPageQryParam;
import com.abssqr.plat.common.facade.param.prod.ProdQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.prod.ProdListVO;
import com.abssqr.plat.common.facade.service.ProdService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.repo.org.OrgRepository;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.core.service.prod.ProdManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.BaseResult;
import com.general.system.facade.current.IdentityUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 项目计划对应产品管理
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.api.prod: ProdServiceImpl.java, v 0.1 2019-05-28 14:19 bangis.wangdf Exp $
 */
@Service
public class ProdServiceImpl extends ServiceSupport implements ProdService {
    @Autowired
    private OrgRepository      orgRepository;
    @Autowired
    private ProdManager         prodManager;

    /**
     * 产品分页查询
     *
     * @param param
     * @return
     */
    @Override
    public BasePage<ProdListVO> getProdList(@Valid ProdPageQryParam param) {
        return this.doQry(param, (QryTemplate<BasePage<ProdListVO>>)() -> {
            ProductListPage page = prodManager.getProdList(param, IdentityUtil.getUserCode());
            BasePage<ProdListVO> result = new BasePage<ProdListVO>();
            result.setCurrPageNo(page.getCurrPageNo());
            result.setTotal(page.getTotal());
            result.setLimit(page.getLimit());
            List<ProdListVO> datas = Lists.newArrayList();

            if (CollectionUtils.isNotEmpty(page.getDatas())) {
                Set<String> orgCodes = page.getDatas().stream().map(ProductDO::getManagerOrgCode).collect(
                    Collectors.toSet());
                Map<String, String> orgNameMap = orgRepository.getOrgNames(orgCodes);
                page.getDatas().forEach(DO -> {
                    ProdListVO data = new ProdListVO();
                    //prodCode 外部产品码
                    data.setProdCode(DO.getProdCode());
                    //prodName 产品名
                    data.setProdName(DO.getProdName());
                    //approveStatus 审批状态
                    data.setApproveStatus(ApproveStatusEnum.getEnum(DO.getApproveStatus()));
                    //abssqrProdCode 内部产品码
                    data.setAbssqrProdCode(DO.getAbssqrProdCode());
                    //managerOrgCode 贷款的管理机构
                    data.setManagerOrgCode(DO.getManagerOrgCode());
                    data.setManagerOrgName(orgNameMap.get(DO.getManagerOrgCode()));
                    //gmtModified
                    data.setGmtModified(DO.getGmtModified());
                    //可编辑判断
                    data.setOperate(ApproveChangeUtil.allowEdit(data.getApproveStatus()));
                    datas.add(data);
                });
            }
            result.setDatas(datas);
            return result;
        });
    }

    /**
     * 新增OrUpdate
     *
     * @param param
     * @return
     */
    @Override
    public BaseResult<String> addOrUpdate(@Valid ProdEditParam param) {
        return BaseResult.success(this.doOpr(param,
                (OprTemplate<String>)() -> prodManager.addOrUpdate(param, IdentityUtil.getUserCode(), IdentityUtil.getUserOrgCode())));
    }

    /**
     * 获取产品
     *
     * @param param
     * @return
     */
    @Override
    public LoanProduct getProd(@Valid ProdQryParam param) {
        if (param.getWork()){
            return this.doQry(param, () -> prodManager.getDataChangeProductBy(param.getAbssqrProdCode()));
        }else{
            return this.doQry(param, () -> prodManager.getProductBy(param.getAbssqrProdCode()));
        }
    }

}
