/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.biz.api.workOrder;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.dao.ActGeBytearrayDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.ActGeBytearrayDO;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderPageParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQprParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderBtn;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderVo;
import com.abssqr.plat.common.facade.service.WorkOrderService;
import com.abssqr.plat.common.model.convertor.WorkOrderConvert;
import com.abssqr.plat.common.model.enums.WorkOrderBtnEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.general.system.bpm.domain.param.ProcessInstanceRequest;
import com.general.system.bpm.domain.param.ProcessStartDefaultRequest;
import com.general.system.bpm.domain.param.ProcessTaskRequest;
import com.general.system.bpm.domain.result.ProcessSelStartResult;
import com.general.system.bpm.entity.ProcessHistoricEntity;
import com.general.system.bpm.entity.ProcessHistoricTaskEntity;
import com.general.system.bpm.entity.ProcessTaskEntity;
import com.general.system.bpm.exception.ProcessException;
import com.general.system.bpm.service.ProcessPageService;
import com.general.system.bpm.service.ProcessQueryService;
import com.general.system.bpm.service.ProcessWorkService;
import com.general.system.common.util.LogUtil;
import com.general.system.facade.current.IdentityUtil;
import com.general.system.tool.util.ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

/**
 * 工单列表及办理
 * @author yangcheng
 * @version WorkOrderServiceImpl.java, v0.12019-06-1010:02 yangchengExp$
 */
@Component
public class WorkOrderServiceImpl extends ServiceSupport implements WorkOrderService {
    
    @Autowired
    private ProcessPageService processPageService;
    @Autowired
    private ProcessQueryService processQueryService;
    @Autowired
    private ChangeManager changeManager;
    
    /**
     * 查询待办列表
     * @param param
     * @return
     */
    @Override
    public BasePage<WorkOrderVo> myWorkOrder(WorkOrderPageParam param) {
        return super.doQry(param, () -> {
            logger.info("myWorkOrder [{}] {}-{}", param.getEventCode(), param.getWorkOrdCode(), param.getWorkOrdName());
            BasePage<WorkOrderVo> result = new BasePage<>();
            result.setCurrPageNo(param.getPageNum());
            result.setLimit(param.getPageSize());

            // 查询待办列表
            ProcessTaskRequest request = new ProcessTaskRequest();
            request.setInvolvedUserId(IdentityUtil.getUserCode());
            request.setCurrPageNo(param.getPageNum());
            request.setLimit(param.getPageSize());
            processPageService.currentTaskPage(request);

            result.setTotal(request.getTotal());
            result.setDatas(WorkOrderConvert.convertTask(request.getDatas()));
            return result;
        });
    }
    
    /**
     * 查询我的申请
     * @param param
     * @return
     */
    @Override
    public BasePage<WorkOrderVo> applyWorkOrder(WorkOrderPageParam param) {
        return super.doQry(param, () -> {
            logger.info("applyWorkOrder [{}] {}-{}", param.getEventCode(), param.getWorkOrdCode(), param.getWorkOrdName());
            BasePage<WorkOrderVo> result = new BasePage<>();
            result.setCurrPageNo(param.getPageNum());
            result.setLimit(param.getPageSize());

            // 查询待办列表
            ProcessInstanceRequest request = new ProcessInstanceRequest();
            request.setInvolvedUserId(IdentityUtil.getUserCode());
            request.setCurrPageNo(param.getPageNum());
            request.setLimit(param.getPageSize());
            processPageService.processHistoricStartPage(request);

            result.setTotal(request.getTotal());
            result.setDatas(WorkOrderConvert.convertHis(request.getDatas()));
            return result;
        });
    }
    
    /**
     * 查询已办事项
     * @param param
     * @return
     */
    @Override
    public BasePage<WorkOrderVo> hisWorkOrder(WorkOrderPageParam param) {
        return super.doQry(param, () -> {
            logger.info("applyWorkOrder [{}] {}-{}", param.getEventCode(), param.getWorkOrdCode(), param.getWorkOrdName());
            BasePage<WorkOrderVo> result = new BasePage<>();
            result.setCurrPageNo(param.getPageNum());
            result.setLimit(param.getPageSize());

            // 查询待办列表
            ProcessInstanceRequest request = new ProcessInstanceRequest();
            request.setInvolvedUserId(IdentityUtil.getUserCode());
            request.setCurrPageNo(param.getPageNum());
            request.setLimit(param.getPageSize());
            processPageService.processHistoricPage(request , false);

            result.setTotal(request.getTotal());
            result.setDatas(WorkOrderConvert.convertHis(request.getDatas()));
            return result;
        });
    }
    
    /**
     * 获取按钮
     * @param param
     * @return
     */
    @Override
    public List<WorkOrderBtn> getBtns(WorkOrderQryParam param){
        return super.doQry(param, () -> {
            String businessKey = param.getBusinessKey();
            String businessType = param.getBusinessType();
            logger.info("getBtns {}-{}", businessType, businessKey);

            // 流程数据
            ProcessHistoricEntity entity = null;
            // 任务
            ProcessTaskEntity task = null;
            try{
                // 流程数据
                entity = processQueryService.getHistoricByBusinessKey(businessKey);
                // 获取对应的最近一项任务
                task = processQueryService.getTaskByProcessInstanceId(
                        entity.getProcessInstanceId(), IdentityUtil.getUserCode());
            }catch (ProcessException e){
                LogUtil.error(logger, e.getMessage());
            }
            return WorkOrderConvert.haveBtn(
                    entity, task, businessKey, businessType, IdentityUtil.getUserCode());
        });
    }
    
    /**
     * 任务办理详情
     * @param param
     * @return
     */
    @Override
    public List<WorkOrderVo> getWorkOrderTasks(WorkOrderQryParam param) {
        return super.doQry(param, () -> {
            String businessKey = param.getBusinessKey();
            logger.info("getWorkOrderTasks businessKey[{}] ", businessKey);
            // 对应流程示例
            ProcessHistoricEntity entity = processQueryService.getHistoricByBusinessKey(businessKey);
            // 任务情况
            List<ProcessHistoricTaskEntity> taskList =
                    processQueryService.findHistoricTaskByInstanceId(entity.getProcessInstanceId());

            List<WorkOrderVo> orders = new ArrayList<>();
            if (CollectionUtil.isEmpty(taskList)){
                return orders;
            }

            taskList.forEach(task -> {
                WorkOrderVo workOrderVo=new  WorkOrderVo();
                workOrderVo.setAssigneeName(task.getAssigneeName());
                workOrderVo.setWorkOrdCode(entity.getProcessInstanceId());
                workOrderVo.setWorkOrdName(entity.getProcessDefinitionName());
                workOrderVo.setBusinessKey(businessKey);
                workOrderVo.setBusinessType(ProcessStartDefaultRequest.getType(entity.getProcessDefinitionKey()));
                workOrderVo.setGmtCreate(task.getStartTime());
                workOrderVo.setCurrentApprovalNode(task.getName());
                workOrderVo.setCurrentApprovalStatus(task.getTaskStatus());
                workOrderVo.setGmtEnd(task.getEndTime());

                orders.add(workOrderVo);
            });
            return orders;
        });
    }
    
    @Autowired
    private ProcessWorkService processWorkService;
    /**
     * 统一操作入口
     * @param param
     * @return
     */
    @Override
    public void operate(WorkOrderQprParam param) {
        super.doOpr(param, () -> {
            logger.info("flow operate[{}] {}-{}",
                    param.getOperate(), param.getBusinessType(), param.getBusinessKey());
            String userCode = IdentityUtil.getUserCode();
            WorkOrderBtnEnum btnEnum = WorkOrderBtnEnum.getByCode(param.getOperate());
            // 流程数据
            ProcessHistoricEntity entity = processQueryService.getHistoricByBusinessKey(param.getBusinessKey());
            String taskId = "";
            switch (btnEnum){
                case refuse:
                    // 拒绝 获取对应的最近一项任务
                    ProcessTaskEntity refuse = processQueryService.getTaskByProcessInstanceId(
                            entity.getProcessInstanceId(), IdentityUtil.getUserCode());
                    taskId = refuse.getId();
                    processWorkService.refuseInstance(taskId, userCode, null);
                    break;
                case complete:
                    // 通过 获取对应的最近一项任务
                    ProcessTaskEntity complete = processQueryService.getTaskByProcessInstanceId(
                            entity.getProcessInstanceId(), IdentityUtil.getUserCode());
                    taskId = complete.getId();
                    processWorkService.completeTask(taskId, userCode, null);
                    break;
                case rollback:
                    // 退回 -- 暂时不支持
                    //processWorkService.rollBackTask(taskId, userCode, null);
                    break;
                case revoke:
                    // 撤销
                    List<ProcessTaskEntity> taskEntities =
                            processQueryService.findTasksByBusinessKey(param.getBusinessKey());
                    if (ToolUtil.isNotEmpty(taskEntities)){
                        taskId = taskEntities.get(0).getId();
                        processWorkService.revokeInstance(taskId, userCode, null);
                    }
                    break;
                default:
                    logger.info("不支持的操作类型");
                    throw new AbssqrBizException("不支持的操作类型");
            }

            return  null;
        });
    }
    
    /**
     * 查看已申请信息 包含 实例信息、历史记录及追踪图
     */
    @Override
    public ProcessSelStartResult detail(WorkOrderQryParam param) {
        return super.doQry(param, () -> {
            String businessKey = param.getBusinessKey();
            // 流程数据
            ProcessHistoricEntity entity = processQueryService.getHistoricByBusinessKey(businessKey);
            return processQueryService.selStartInfo(entity.getProcessInstanceId());
        });
    }

    /**
     * 获取工单业务的业务类型
     * @param param
     * @return
     */
    @Override
    public DataChangeBizTypeEnum getBizType(WorkOrderQryParam param) {
        return super.doQry(param, () -> {
            String businessKey = param.getBusinessKey();
            return changeManager.getDataChange(businessKey).getBizType();
        });
    }


    @Autowired
    private ActGeBytearrayDAO actGeBytearrayDAO;
    /**
     * 根据文件配置的流程数据进行初始化
     * 首先将数据导出insert语句放到tmpl/flow/flow_blob 文件上
     * 通过地址触发 读取代码下 tmpl/flow/flow_blob 的文件数据进行初始化
     */
    @Override
    public int init() {
        return super.doOpr(new WorkOrderQprParam(), () -> {
            List<String> list = new ArrayList<>();
            try {
                // 读取文件
                InputStream inputStream = Thread.currentThread().getContextClassLoader()
                                                  .getResourceAsStream("tmpl/flow/flow_blob");
                InputStreamReader inputStreamReader = new InputStreamReader(inputStream, CharsetUtil.UTF_8);
                BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
                String str;
                while ((str = bufferedReader.readLine()) != null) {
                    list.add(str);
                }
            } catch (Exception e) {
                e.printStackTrace();
                throw new AbssqrBizException("读取初始化流程文件异常");
            }
            int cnt = 0;
            if (ToolUtil.isEmpty(list)){
                return cnt;
            }
            logger.info("init size {}", list.size());
            for(String one : list){
                if (StrUtil.isNotBlank(one) && StrUtil.contains(one, ',')){
                    one = StrUtil.subBetween(one,"VALUES (",", TO_DATE");
                    // 列数据
                    String[] strs = one.split(",");
                    ActGeBytearrayDO bytearrayDO = new ActGeBytearrayDO();
                    bytearrayDO.setId(trimSign(strs[0]));
                    String rev = trimSign(strs[1]);
                    bytearrayDO.setRev(rev == null ? null : Long.valueOf(rev));
                    bytearrayDO.setName(trimSign(strs[2]));
                    bytearrayDO.setDeploymentId(trimSign(strs[3]));
                    // 设置流程文件数据
                    bytearrayDO.setBytes(hex2Byte(trimSign(strs[4])));
                    String gen = trimSign(strs[5]);
                    bytearrayDO.setGenerated(gen == null ? null : Long.valueOf(gen));
                    actGeBytearrayDAO.insert(bytearrayDO);
                }
            }
            return cnt;
        });
    }

    private String trimSign(String str){
        String tStr = StrUtil.trim(str);
        if (StrUtil.equals( "null", tStr)){
            return null;
        }
        return StrUtil.isEmpty(tStr) ? "" : tStr.replaceAll("'", "");
    }
    
    /**
     * 16进制文件字符串转byte[]
     * @param hex
     * @return
     */
    private byte[] hex2Byte(String hex){
        if (StrUtil.isEmpty(hex)){
            return null;
        }
        ByteBuffer bf = ByteBuffer.allocate(hex.length() / 2);
        for (int i = 0; i < hex.length(); i++) {
            String hexStr = hex.charAt(i) + "";
            i++;
            hexStr += hex.charAt(i);
            byte b = (byte) Integer.parseInt(hexStr, 16);
            bf.put(b);
        }
        return bf.array();
    }
    
}
