/**
 * abssqr.com Inc.
 * Copyright (c) 2017-${YEAR} All Rights Reserved.
 */
package com.abssqr.plat.biz.validate;

import java.util.ArrayList;
import java.util.List;

/**
 * 参数校验错误上下文
 * 
 * @author zhenxuan.luo
 * @version $Id: ValidatorError.java, v 0.1 2015年5月18日 下午7:35:16 zhenxuan.luo Exp $
 */
public class ValidatorError {
    /** 异常信息 */
    private List<FieldError> errList = null;

    /**
     * 存在异常
     * @return true:存在异常;反之返回false
     */
    public boolean hasErrors() {
        return errList != null && errList.size() > 0;
    }

    /**
     * 设置异常
     * @param fieldError
     */
    public void addFieldError(FieldError fieldError) {
        if (errList == null) {
            errList = new ArrayList<FieldError>();
        }
        errList.add(fieldError);
    }

    /**
     *  获取异常
     * @return 字段错误清单
     */
    public List<FieldError> getFieldErrors() {
        return errList;
    }

    /**
     * 错误信息描述
     * @return 错误信息
     */
    public StringBuilder getErrorMsg() {
        return getErrorMsg(null, null);
    }

    /**
     * 错误信息描述
     * @param prefix 前缀
     * @param suffix 后缀
     * @return 错误信息
     */
    public StringBuilder getErrorMsg(String prefix, String suffix) {
        StringBuilder sb = new StringBuilder();
        if (prefix != null) {
            sb.append(prefix);
        }
        if (errList == null) {
            return sb;
        }

        // 添加校验不通过的所有参数名、值和不通过信息
        for (FieldError error : errList) {
            sb.append(error.getObjectName()).append("#").append(error.getFieldName()).append("=[")
                .append(error.getFieldValue()).append("]:[").append(error.getErrorMsg())
                .append("];");
        }
        if (suffix != null) {
            sb.append(suffix);
        }
        return sb;
    }
}
