/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.match;

import java.util.Date;
import java.util.List;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.biz.shared.handler.PlanAstMatchHandler;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.ast.Pack;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.dispatcher.match: PlanAstMatchDispatcherStarter.java, v 0.1 2019-11-12 11:03
 * bangis.wangdf Exp $
 */

@Component
@Configuration
public class PlanAstMatchDispatcherExecutor extends DispatcherExecutorTemp<CommonTask> {

    private final static Logger LOGGER = CommonLoggers.AST_MATCH;

    @Autowired
    PlanAstMatchHandler  planAstMatchHandler;
    @Autowired
    CommonTaskRepository commonTaskRepository;

    @Override
    public CommonTask doExecute(CommonTask node) {
        LogUtil.info(LOGGER, "执行资产匹配任务[{0}]-start", node.getContext());
        planAstMatchHandler.execute(node.getContext(), (List<Pack>)node.getExtObj(), status -> {
            if (TaskStatusEnum.SUCCESS == status) {
                node.setTaskStatus(status);
                markSuccess(node);
            } else {
                markFail(node, "执行资产匹配任务失败");
            }
            LogUtil.info(LOGGER, "执行资产匹配任务[{0}]-end,status=[{1}]", node.getContext(), status);
        });
        
        return node;
    }

    @Override
    protected CommonTask markSuccess(CommonTask node) {
        //为保证数据一直性,讲成功设置
        commonTaskRepository.updateTaskStatus(node.getId(), TaskStatusEnum.SUCCESS, TaskStatusEnum.EXECUTING, "");
        return node;
    }

    @Override
    protected CommonTask markFail(CommonTask node, String failMessage) {
        // 因异常引起的重试
        Date nextTime = DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 60);
        String msg = failMessage.length() > 2000 ? failMessage.substring(0, 2000) : failMessage;
        commonTaskRepository.updateTaskExep(nextTime, node.getId(), msg);
        return node;
    }

    @Override
    protected CommonTask markException(CommonTask node, String failMessage) {
        return markFail(node, failMessage);
    }

    @Override
    protected String getName() {
        return "PlanAstMatchDispatcherExecutor";
    }
}