/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler;

import com.general.system.common.util.LogUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.quartz.JobDetail;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;

/**
 * 注册job至Scheduler
 * @author zhenxuan.luo
 * @version $Id: JobScheduler.java, v 0.1 2017-06-29 下午18:24 zhenxuan.luo Exp $
 */
@Component
public class AbssqrJobCollector {
    
    private static final Logger LOGGER = LoggerFactory.getLogger(AbssqrJobCollector.class);
    @Value("${sync.quartz.open:true}")
    private boolean isOpen;
    @Autowired
    private SchedulerFactoryBean schedulerFactoryBean;
    
    private List<BaseQuartzJob> jobs = Lists.newArrayList();
    
    public List<BaseQuartzJob> getJobs() {
        return jobs;
    }
    
    public void registerJob(BaseQuartzJob job) throws SchedulerException {
        if(!isOpen) {
            LogUtil.info(LOGGER, "定时任务[{0}]未开启：", job.getJobName());
            return;
        }
        try {
            LogUtil.info(LOGGER, "定时任务[{0}]启动开始：", job.getJobName());
            JobDetail jobDetail = job.buildJobDetail();
            Trigger jobTrigger = job.buildJobTrigger();
            
            Set<Trigger> triggerSet = Sets.newHashSet();
            triggerSet.add(jobTrigger);
            
            schedulerFactoryBean.getScheduler().scheduleJob(jobDetail, triggerSet, true);
            this.jobs.add(job);
            LogUtil.info(LOGGER, "定时任务[{0}]启动成功：", job.getJobName());
            
        } catch (Throwable e) {
            LogUtil.error(LOGGER, e,"定时任务[{0}]启动失败：", job.getJobName());
            throw e;
        }
    }
}