package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.CommonTaskDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.mapper.CommonTaskDOMapper;

/**
* The Table ABS_COMMON_TASK.
* ABS_COMMON_TASK
*/
@Repository
public class CommonTaskDAO{

    @Autowired
    private CommonTaskDOMapper commonTaskDOMapper;

    /**
     * desc:插入表:ABS_COMMON_TASK.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(CommonTaskDO entity){
        return commonTaskDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:ABS_COMMON_TASK.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<CommonTaskDO> list){
        commonTaskDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:查询待执行的任务.<br/>
     * @param id id
     * @return CommonTaskDO
     */
    
    public CommonTaskDO lockById(Long id){
        return commonTaskDOMapper.lockById(id);
    }
    /**
     * desc:查询待执行的任务.<br/>
     * @param curTime curTime
     * @param limit limit
     * @param typeList typeList
     * @return List<CommonTaskDO>
     */
    
    public List<CommonTaskDO> selectInitTask(Date curTime,Integer limit,List<String> typeList){
    if(typeList!=null && typeList.size()>201){throw new RuntimeException("typeList 超出201长度限制");}
        return commonTaskDOMapper.selectInitTask(curTime, limit, typeList);
    }
    /**
     * desc:查询待执行的任务.<br/>
     * @param taskType taskType
     * @return List<CommonTaskDO>
     */
    
    public List<CommonTaskDO> selectUnifinishTaskByType(String taskType){
        return commonTaskDOMapper.selectUnifinishTaskByType(taskType);
    }
    /**
     * desc:更新执行的任务状态.<br/>
     * @param id id
     * @param memo memo
     * @param newStatus newStatus
     * @param oldStatus oldStatus
     * @return int
     */
    
    public int updateTaskStatus(Long id,String memo,String newStatus,String oldStatus){
        return commonTaskDOMapper.updateTaskStatus(id, memo, newStatus, oldStatus);
    }
    /**
     * desc:更新任务为执行中.<br/>
     * @param lastExecTime lastExecTime
     * @param lastHost lastHost
     * @param tasks tasks
     * @return int
     */
    
    public int updateTaskExec(Date lastExecTime,String lastHost,List<CommonTaskDO> tasks){
    if(tasks!=null && tasks.size()>1000){throw new RuntimeException("tasks 超出1000长度限制");}
        return commonTaskDOMapper.updateTaskExec(lastExecTime, lastHost, tasks);
    }
    /**
     * desc:重置状态，更新下次执行时间.<br/>
     * @param nextExecTime nextExecTime
     * @param id id
     * @param memo memo
     * @return int
     */
    
    public int updateTaskExep(Date nextExecTime,Long id,String memo){
        return commonTaskDOMapper.updateTaskExep(nextExecTime, id, memo);
    }
    /**
     * desc:把任务为Ignore状态的任务改为Init.<br/>
     * @param id id
     * @return int
     */
    
    public int updateIgnoreTaskStatus2Init(Long id){
        return commonTaskDOMapper.updateIgnoreTaskStatus2Init(id);
    }
    /**
     * desc:根据ip重置任务.<br/>
     * @param lastHost lastHost
     * @return int
     */
    
    public int updateHangTasksByIp(String lastHost){
        return commonTaskDOMapper.updateHangTasksByIp(lastHost);
    }
    /**
     * desc:查询当前计划是否有正在执行中的的解绑，同步匹配，解绑匹配任务.<br/>
     * @param planNo planNo
     * @param statuses statuses
     * @param taskTypes taskTypes
     * @return List<CommonTaskDO>
     */
    
    public List<CommonTaskDO> queryUnMatchOrMatchTaskByContextAndStatus(String planNo,List<String> statuses,List<String> taskTypes){
    if(statuses!=null && statuses.size()>201){throw new RuntimeException("statuses 超出201长度限制");}
    if(taskTypes!=null && taskTypes.size()>201){throw new RuntimeException("taskTypes 超出201长度限制");}
        return commonTaskDOMapper.queryUnMatchOrMatchTaskByContextAndStatus(planNo, statuses, taskTypes);
    }
    /**
     * desc:通过BizNo查询CommonTask.<br/>
     * @param bizNo bizNo
     * @return List<CommonTaskDO>
     */
    
    public List<CommonTaskDO> queryByBizNo(String bizNo){
        return commonTaskDOMapper.queryByBizNo(bizNo);
    }
    /**
     * desc:根据任务类型和任务状态查询.<br/>
     * @param typeList typeList
     * @param statusList statusList
     * @return List<CommonTaskDO>
     */
    
    public List<CommonTaskDO> queryByTypesAndStatus(List<String> typeList,List<String> statusList){
    if(typeList!=null && typeList.size()>201){throw new RuntimeException("typeList 超出201长度限制");}
    if(statusList!=null && statusList.size()>201){throw new RuntimeException("statusList 超出201长度限制");}
        return commonTaskDOMapper.queryByTypesAndStatus(typeList, statusList);
    }
    /**
     * desc:根据任务类型、任务状态和执行时间范围计数.<br/>
     * @param second second
     * @param typeList typeList
     * @param statusList statusList
     * @return Long
     */
    
    public Long countByTypesAndStatusAndRange(Long second,List<String> typeList,List<String> statusList){
    if(typeList!=null && typeList.size()>201){throw new RuntimeException("typeList 超出201长度限制");}
    if(statusList!=null && statusList.size()>201){throw new RuntimeException("statusList 超出201长度限制");}
        return commonTaskDOMapper.countByTypesAndStatusAndRange(second, typeList, statusList);
    }
    /**
     * desc:查询未完成的执行次数大于1的任务数量.<br/>
     * @param typeList typeList
     * @return Long
     */
    
    public Long countUnFinishAndMultipleExecByTypes(List<String> typeList){
    if(typeList!=null && typeList.size()>201){throw new RuntimeException("typeList 超出201长度限制");}
        return commonTaskDOMapper.countUnFinishAndMultipleExecByTypes(typeList);
    }
}
