package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocItemDO;
import java.util.List;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAllocItemResultMap;
import java.util.Map;
import java.util.Set;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.HashSet;
import com.abssqr.plat.common.dal.mysql.auto.mapper.PlanAllocItemDOMapper;

/**
* The Table abs_plan_alloc_item.
* 分配事项
*/
@Repository
public class PlanAllocItemDAO{

    @Autowired
    private PlanAllocItemDOMapper planAllocItemDOMapper;

    /**
     * desc:插入表:abs_plan_alloc_item.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(PlanAllocItemDO entity){
        return planAllocItemDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_plan_alloc_item.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<PlanAllocItemDO> list){
        planAllocItemDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据主键删除数据:abs_plan_alloc_item.<br/>
     * @param id id
     * @return int
     */
    
    public int deleteById(Long id){
        return planAllocItemDOMapper.deleteById(id);
    }
    /**
     * desc:根据主键获取数据:abs_plan_alloc_item.<br/>
     * @param id id
     * @return PlanAllocItemDO
     */
    
    public PlanAllocItemDO getById(Long id){
        return planAllocItemDOMapper.getById(id);
    }
    /**
     * desc:根据唯一约束ItemNo更新表:abs_plan_alloc_item.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateByItemNo(PlanAllocItemDO entity){
        return planAllocItemDOMapper.updateByItemNo(entity);
    }
    /**
     * desc:根据唯一约束ItemNo删除数据:abs_plan_alloc_item.<br/>
     * @param itemNo itemNo
     * @return int
     */
    
    public int deleteByItemNo(String itemNo){
        return planAllocItemDOMapper.deleteByItemNo(itemNo);
    }
    /**
     * desc:根据唯一约束ItemNo获取数据:abs_plan_alloc_item.<br/>
     * @param itemNo itemNo
     * @return PlanAllocItemDO
     */
    
    public PlanAllocItemDO getByItemNo(String itemNo){
        return planAllocItemDOMapper.getByItemNo(itemNo);
    }
     /**
     * desc:根据计划编号获取分配事项.<br/>
     * @param planNo planNo
     * @return Map<String,List<PlanAllocItemResultMap>>
     */
    
    public List<PlanAllocItemResultMap> getByPlanNoList(String planNo){
        return planAllocItemDOMapper.getByPlanNo(planNo);
    }
    /**
     * desc:根据计划编号获取分配事项.<br/>
     * @param planNo planNo
     * @return Map<String,List<PlanAllocItemResultMap>>
     */
    
    public Map<String,List<PlanAllocItemResultMap>> getByPlanNo(String planNo){
        Map<String,List<PlanAllocItemResultMap>> result = new LinkedHashMap();
        List<PlanAllocItemResultMap> resultList = planAllocItemDOMapper.getByPlanNo(planNo);
        if(resultList!=null && !resultList.isEmpty()){
            for(PlanAllocItemResultMap entity:resultList){
                List<PlanAllocItemResultMap> vList =  result.get(entity.getAllocNo());
                if(vList == null){
                    vList = new ArrayList();
                    result.put(entity.getAllocNo(),vList);
                }
                vList.add(entity);
            }
        }
        return result;
    }
     /**
     * desc:根据分配编号获取分配事项.<br/>
     * @param allocNo allocNo
     * @return Map<String,Long>
     */
    
    public List<PlanAllocItemResultMap> getByAllocNoList(String allocNo){
        return planAllocItemDOMapper.getByAllocNo(allocNo);
    }
    /**
     * desc:根据分配编号获取分配事项.<br/>
     * @param allocNo allocNo
     * @return Map<String,Long>
     */
    
    public Map<String,Long> getByAllocNo(String allocNo){
        Map<String,Long> result = new LinkedHashMap();
        List<PlanAllocItemResultMap> resultList = planAllocItemDOMapper.getByAllocNo(allocNo);
        if(resultList!=null && !resultList.isEmpty()){
            for(PlanAllocItemResultMap entity:resultList){
                result.put(entity.getTitleNo(),entity.getSettleAmt());
            }
        }
        return result;
    }
    /**
     * desc:根据allocNo删除数据:abs_plan_alloc_item.<br/>
     * @param allocNo allocNo
     * @return int
     */
    
    public int deleteByAllocNo(String allocNo){
        return planAllocItemDOMapper.deleteByAllocNo(allocNo);
    }
    /**
     * desc:根据planNo删除数据.<br/>
     * @param planNo planNo
     * @return int
     */
    
    public int deleteByPlanNo(String planNo){
        return planAllocItemDOMapper.deleteByPlanNo(planNo);
    }
}
