package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportItemDO;
import java.util.List;
import java.util.Date;
import com.abssqr.plat.common.dal.mysql.auto.mapper.PlanReportItemDOMapper;

/**
* The Table abs_plan_report_item.
* ABS_PLAN_REPORT_ITEM
*/
@Repository
public class PlanReportItemDAO{

    @Autowired
    private PlanReportItemDOMapper planReportItemDOMapper;

    /**
     * desc:批量插入表:abs_plan_report_item.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<PlanReportItemDO> list){
        planReportItemDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据唯一约束AbsPlanReportItem获取数据:abs_plan_report_item.<br/>
     * @param rptNo rptNo
     * @param planNo planNo
     * @return List<PlanReportItemDO>
     */
    
    public List<PlanReportItemDO> getByRpt(String rptNo,String planNo){
        return planReportItemDOMapper.getByRpt(rptNo, planNo);
    }
    /**
     * desc:根据主键获取数据:abs_plan_report.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @return List<PlanReportItemDO>
     */
    
    public List<PlanReportItemDO> getByDate(Date endDate,Date startDate,String planNo){
        return planReportItemDOMapper.getByDate(endDate, startDate, planNo);
    }
    /**
     * desc:根据计划和日期获取单项统计值.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param rptItem rptItem
     * @return PlanReportItemDO
     */
    
    public PlanReportItemDO getOneRptItemByPlanNoAndAccDate(Date accountDate,String planNo,String rptItem){
        return planReportItemDOMapper.getOneRptItemByPlanNoAndAccDate(accountDate, planNo, rptItem);
    }
    /**
     * desc:根据计划和日期获取多个统计项的汇总值.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param list list
     * @return List<PlanReportItemDO>
     */
    
    public List<PlanReportItemDO> queryRptItemsByPlanNoAndAccDate(Date accountDate,String planNo,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return planReportItemDOMapper.queryRptItemsByPlanNoAndAccDate(accountDate, planNo, list);
    }
    /**
     * desc:根据计划和日期区间获取多个统计项的汇总值.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @param list list
     * @return List<PlanReportItemDO>
     */
    
    public List<PlanReportItemDO> queryPlanRptItemsByPlanNoSection(Date endDate,Date startDate,String planNo,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return planReportItemDOMapper.queryPlanRptItemsByPlanNoSection(endDate, startDate, planNo, list);
    }
    /**
     * desc:根据计划和日期区间获取多个统计项的每日数据.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param listItem listItem
     * @param listPlanNo listPlanNo
     * @return List<PlanReportItemDO>
     */
    
    public List<PlanReportItemDO> queryDayRptItemsByPlanNoSection(Date endDate,Date startDate,List<String> listItem,List<String> listPlanNo){
    if(listItem!=null && listItem.size()>201){throw new RuntimeException("listItem 超出201长度限制");}
    if(listPlanNo!=null && listPlanNo.size()>201){throw new RuntimeException("listPlanNo 超出201长度限制");}
        return planReportItemDOMapper.queryDayRptItemsByPlanNoSection(endDate, startDate, listItem, listPlanNo);
    }
    /**
     * desc:根据计划和日期区间获取最小日期.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @param list list
     * @return Date
     */
    
    public Date queryPlanReportMinDateByPlanNoSection(Date endDate,Date startDate,String planNo,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return planReportItemDOMapper.queryPlanReportMinDateByPlanNoSection(endDate, startDate, planNo, list);
    }
}
