package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.ProductDO;
import java.util.List;
import com.abssqr.plat.common.dal.mysql.auto.paging.ProductListPage;
import com.abssqr.plat.common.dal.mysql.auto.mapper.ProductDOMapper;

/**
* The Table ABS_PRODUCT.
* ABS_PRODUCT
*/
@Repository
public class ProductDAO{

    @Autowired
    private ProductDOMapper productDOMapper;

    /**
     * desc:插入表:ABS_PRODUCT.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(ProductDO entity){
        return productDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:ABS_PRODUCT.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<ProductDO> list){
        productDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据唯一约束OrgCodeProdCode获取数据:ABS_PRODUCT.<br/>
     * @param prodCode prodCode
     * @param managerOrgCode managerOrgCode
     * @return ProductDO
     */
    
    public ProductDO getByOrgCodeProdCode(String prodCode,String managerOrgCode){
        return productDOMapper.getByOrgCodeProdCode(prodCode, managerOrgCode);
    }
    /**
     * desc:根据唯一约束AbssqrProdCode删除数据:ABS_PRODUCT.<br/>
     * @param abssqrProdCode abssqrProdCode
     * @return int
     */
    
    public int deleteByAbssqrProdCode(String abssqrProdCode){
        return productDOMapper.deleteByAbssqrProdCode(abssqrProdCode);
    }
    /**
     * desc:根据唯一约束AbssqrProdCode获取数据:ABS_PRODUCT.<br/>
     * @param abssqrProdCode abssqrProdCode
     * @return ProductDO
     */
    
    public ProductDO getByAbssqrProdCode(String abssqrProdCode){
        return productDOMapper.getByAbssqrProdCode(abssqrProdCode);
    }
    /**
     * desc:根据唯一约束AbssqrProdCode获取数据:ABS_PRODUCT.<br/>
     * @param abssqrProdCodes abssqrProdCodes
     * @return List<ProductDO>
     */
    
    public List<ProductDO> getByAbssqrProdCodes(List<String> abssqrProdCodes){
    if(abssqrProdCodes!=null && abssqrProdCodes.size()>201){throw new RuntimeException("abssqrProdCodes 超出201长度限制");}
        return productDOMapper.getByAbssqrProdCodes(abssqrProdCodes);
    }
    /**
     * desc:更新产品.<br/>
     * @param entity entity
     * @return int
     */
    
    public int update(ProductDO entity){
        return productDOMapper.update(entity);
    }
    /**
     * desc:获取产品.<br/>
     * @param prodCode prodCode
     * @return ProductDO
     */
    
    public ProductDO getByProdCode(String prodCode){
        return productDOMapper.getByProdCode(prodCode);
    }
    /**
     * desc:获取同名产品数量，防止重名存在.<br/>
     * @param entity entity
     * @return Integer
     */
    
    public Integer getSameProdCnt(ProductDO entity){
        return productDOMapper.getSameProdCnt(entity);
    }
    /**
     * desc:查询产品列表.<br/>
     * @param productList productList
     * @return ProductListPage
     */
    
    public ProductListPage getProductList(ProductListPage productList){
        int total = productDOMapper.getProductListCount(productList);
        if(total>0){
            productList.setDatas(productDOMapper.getProductListResult(productList));
        }else{
            productList.setDatas(new ArrayList());
        }
        productList.setTotal(total);
        return productList;
    }
}
