package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import java.util.List;
import java.util.Date;
import com.abssqr.plat.common.dal.mysql.auto.paging.ActivePackPage;
import java.lang.Long;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table ABS_AST_PACK.
 * 资产包
 */
public interface AstPackDOMapper{

    /**
     * desc:插入表:ABS_AST_PACK.<br/>
     * @param entity entity
     * @return int
     */
    int insert(AstPackDO entity);
    /**
     * desc:批量插入表:abs_account.<br/>
     * @param list list
     * @return int
     */
    int insertBatch(List<AstPackDO> list);
    /**
     * desc:根据状态获取包.<br/>
     * @param scenDate scenDate
     * @param packType packType
     * @param packStatus packStatus
     * @return List<AstPackDO>
     */
    List<AstPackDO> selectPackByStatus(@Param("scenDate")Date scenDate,@Param("packType")String packType,@Param("packStatus")String packStatus);
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param packCode packCode
     * @return AstPackDO
     */
    AstPackDO getByPackCode(@Param("packCode")String packCode);
    /**
     * desc:根据普通索引AbsTrCtrTask0获取数据:abs_tr_ctr_task.<br/>
     * @param scenDate scenDate
     * @param status status
     * @param packType packType
     * @param managerOrgCode managerOrgCode
     * @return List<AstPackDO>
     */
    List<AstPackDO> getByManagerOrg(@Param("scenDate")Date scenDate,@Param("status")String status,@Param("packType")String packType,@Param("managerOrgCode")String managerOrgCode);
    /**
     * desc:更新表:ABS_AST_PACK.<br/>
     * @param packStatus packStatus
     * @param oldPackStatus oldPackStatus
     * @param packCodes packCodes
     * @return int
     */
    int updatePackStatus(@Param("packStatus")String packStatus,@Param("oldPackStatus")String oldPackStatus,@Param("packCodes")List<String> packCodes);
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param planNo planNo
     * @param packType packType
     * @return List<AstPackDO>
     */
    List<AstPackDO> getByPlanNo(@Param("planNo")String planNo,@Param("packType")String packType);
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param scenDate scenDate
     * @param planNo planNo
     * @param packType packType
     * @return AstPackDO
     */
    AstPackDO getLastDayPackByPlanNo(@Param("scenDate")Date scenDate,@Param("planNo")String planNo,@Param("packType")String packType);
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param activePack activePack
     * @return int
     */
    int queryByActivePacksCount(ActivePackPage activePack);
    /**
     * desc:根据packCode获取数据:ABS_AST_PACK.<br/>
     * @param activePack activePack
     * @return List<AstPackDO>
     */
    List<AstPackDO> queryByActivePacksResult(ActivePackPage activePack);
    /**
     * desc:获取指定状态的资产包数量.<br/>
     * @param time time
     * @param status status
     * @param packType packType
     * @return Long
     */
    Long countByStatusAndTime(@Param("time")Date time,@Param("status")String status,@Param("packType")String packType);
    /**
     * desc:锁资产包.<br/>
     * @param packCode packCode
     * @return AstPackDO
     */
    AstPackDO lockByPackCode(@Param("packCode")String packCode);
}
