package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.CommonTaskDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table ABS_COMMON_TASK.
 * ABS_COMMON_TASK
 */
public interface CommonTaskDOMapper{

    /**
     * desc:插入表:ABS_COMMON_TASK.<br/>
     * @param entity entity
     * @return int
     */
    int insert(CommonTaskDO entity);
    /**
     * desc:批量插入表:ABS_COMMON_TASK.<br/>
     * @param list list
     * @return int
     */
    int insertBatch(List<CommonTaskDO> list);
    /**
     * desc:查询待执行的任务.<br/>
     * @param id id
     * @return CommonTaskDO
     */
    CommonTaskDO lockById(Long id);
    /**
     * desc:查询待执行的任务.<br/>
     * @param curTime curTime
     * @param limit limit
     * @param typeList typeList
     * @return List<CommonTaskDO>
     */
    List<CommonTaskDO> selectInitTask(@Param("curTime")Date curTime,@Param("limit")Integer limit,@Param("typeList")List<String> typeList);
    /**
     * desc:查询待执行的任务.<br/>
     * @param taskType taskType
     * @return List<CommonTaskDO>
     */
    List<CommonTaskDO> selectUnifinishTaskByType(@Param("taskType")String taskType);
    /**
     * desc:更新执行的任务状态.<br/>
     * @param id id
     * @param memo memo
     * @param newStatus newStatus
     * @param oldStatus oldStatus
     * @return int
     */
    int updateTaskStatus(@Param("id")Long id,@Param("memo")String memo,@Param("newStatus")String newStatus,@Param("oldStatus")String oldStatus);
    /**
     * desc:更新任务为执行中.<br/>
     * @param lastExecTime lastExecTime
     * @param lastHost lastHost
     * @param tasks tasks
     * @return int
     */
    int updateTaskExec(@Param("lastExecTime")Date lastExecTime,@Param("lastHost")String lastHost,@Param("tasks")List<CommonTaskDO> tasks);
    /**
     * desc:重置状态，更新下次执行时间.<br/>
     * @param nextExecTime nextExecTime
     * @param id id
     * @param memo memo
     * @return int
     */
    int updateTaskExep(@Param("nextExecTime")Date nextExecTime,@Param("id")Long id,@Param("memo")String memo);
    /**
     * desc:把任务为Ignore状态的任务改为Init.<br/>
     * @param id id
     * @return int
     */
    int updateIgnoreTaskStatus2Init(Long id);
    /**
     * desc:根据ip重置任务.<br/>
     * @param lastHost lastHost
     * @return int
     */
    int updateHangTasksByIp(@Param("lastHost")String lastHost);
    /**
     * desc:查询当前计划是否有正在执行中的的解绑，同步匹配，解绑匹配任务.<br/>
     * @param planNo planNo
     * @param statuses statuses
     * @param taskTypes taskTypes
     * @return List<CommonTaskDO>
     */
    List<CommonTaskDO> queryUnMatchOrMatchTaskByContextAndStatus(@Param("planNo")String planNo,@Param("statuses")List<String> statuses,@Param("taskTypes")List<String> taskTypes);
    /**
     * desc:通过BizNo查询CommonTask.<br/>
     * @param bizNo bizNo
     * @return List<CommonTaskDO>
     */
    List<CommonTaskDO> queryByBizNo(@Param("bizNo")String bizNo);
    /**
     * desc:根据任务类型和任务状态查询.<br/>
     * @param typeList typeList
     * @param statusList statusList
     * @return List<CommonTaskDO>
     */
    List<CommonTaskDO> queryByTypesAndStatus(@Param("typeList")List<String> typeList,@Param("statusList")List<String> statusList);
    /**
     * desc:根据任务类型、任务状态和执行时间范围计数.<br/>
     * @param second second
     * @param typeList typeList
     * @param statusList statusList
     * @return Long
     */
    Long countByTypesAndStatusAndRange(@Param("second")Long second,@Param("typeList")List<String> typeList,@Param("statusList")List<String> statusList);
    /**
     * desc:查询未完成的执行次数大于1的任务数量.<br/>
     * @param typeList typeList
     * @return Long
     */
    Long countUnFinishAndMultipleExecByTypes(@Param("typeList")List<String> typeList);
}
