package com.abssqr.plat.common.dal.mysql.auto.mapper;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import org.apache.ibatis.annotations.Param;

/**
 * 由于需要对分页支持,请直接使用对应的DAO类
 * The Table abs_plan_alloc.
 * 计划-分配计划
 */
public interface PlanAllocDOMapper{

    /**
     * desc:插入表:abs_plan_alloc.<br/>
     * @param entity entity
     * @return int
     */
    int insert(PlanAllocDO entity);
    /**
     * desc:批量插入表:abs_plan_alloc.<br/>
     * @param list list
     * @return int
     */
    int insertBatch(List<PlanAllocDO> list);
    /**
     * desc:更新表:abs_plan_alloc.<br/>
     * @param entity entity
     * @return int
     */
    int updateByAllocNo(PlanAllocDO entity);
    /**
     * desc:根据AllocNo删除数据:abs_plan_alloc.<br/>
     * @param allocNo allocNo
     * @return int
     */
    int deleteByAllocNo(@Param("allocNo")String allocNo);
    /**
     * desc:根据主键获取数据:abs_plan_alloc.<br/>
     * @param allocNo allocNo
     * @return PlanAllocDO
     */
    PlanAllocDO getByAllocNo(@Param("allocNo")String allocNo);
    /**
     * desc:更新状态.<br/>
     * @param status status
     * @param allocNo allocNo
     * @param oldStatus oldStatus
     * @return int
     */
    int updateStatus(@Param("status")String status,@Param("allocNo")String allocNo,@Param("oldStatus")String oldStatus);
    /**
     * desc:根据主键获取数据:abs_plan_alloc.<br/>
     * @param planNo planNo
     * @return List<PlanAllocDO>
     */
    List<PlanAllocDO> queryByPlanNo(@Param("planNo")String planNo);
    /**
     * desc:根据计划编号获取时间段分配金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @param status status
     * @return List<PlanAllocDO>
     */
    List<PlanAllocDO> queryAllocByPlanNo(@Param("endDate")Date endDate,@Param("startDate")Date startDate,@Param("planNo")String planNo,@Param("status")String status);
    /**
     * desc:根据兑付日获取查询最近计划:abs_plan_alloc.<br/>
     * @param planType planType
     * @return PlanAllocDO
     */
    PlanAllocDO queryRecentlyPlans(@Param("planType")String planType);
    /**
     * desc:查询相同结算日分配计划数量.<br/>
     * @param clearDate clearDate
     * @param planNo planNo
     * @param allocNo allocNo
     * @return int
     */
    int querySameClearDate(@Param("clearDate")Date clearDate,@Param("planNo")String planNo,@Param("allocNo")String allocNo);
    /**
     * desc:根据主键获取数据:abs_plan_alloc.<br/>
     * @param allocNo allocNo
     * @return PlanAllocDO
     */
    PlanAllocDO lockByAllocNo(@Param("allocNo")String allocNo);
    /**
     * desc:获取所有已审批通过未解绑的兑付计划.<br/>
     * @param planNo planNo
     * @return List<PlanAllocDO>
     */
    List<PlanAllocDO> queryUnMatchedApprovedAlloc(@Param("planNo")String planNo);
    /**
     * desc:根据计划编号获取时间段分配金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param planNo planNo
     * @param status status
     * @param approveStatus approveStatus
     * @return Long
     */
    Long queryStatAllocAmtByPlanNoAndPayDate(@Param("endDate")Date endDate,@Param("startDate")Date startDate,@Param("planNo")String planNo,@Param("status")String status,@Param("approveStatus")String approveStatus);
    /**
     * desc:根据计划编号获取累计分配金额.<br/>
     * @param endDate endDate
     * @param startDate startDate
     * @param status status
     * @param approveStatus approveStatus
     * @param listPlanNos listPlanNos
     * @return Long
     */
    Long queryStatTotalAllocAmtByPlanNos(@Param("endDate")Date endDate,@Param("startDate")Date startDate,@Param("status")String status,@Param("approveStatus")String approveStatus,@Param("listPlanNos")List<String> listPlanNos);
    /**
     * desc:根据计划编号删除分配方案.<br/>
     * @param planNo planNo
     * @return int
     */
    int deleteByPlanNo(@Param("planNo")String planNo);
    /**
     * desc:查询已审批未兑付的分配计划.<br/>
     * @param planNo planNo
     * @return List<PlanAllocDO>
     */
    List<PlanAllocDO> queryApprovedNotPayAlloc(@Param("planNo")String planNo);
}
