/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.enums;

import cn.hutool.json.JSONUtil;
import com.abssqr.plat.common.facade.model.trans.CapitalTrader;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.MessageUtil;

import java.util.HashMap;
import java.util.Map;

/**
 * 计划资金交易类型
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.facade.model.enums: PlanIndicatorEnum.java, v 0.1 2019-05-28 15:07 zhenxuan.luo Exp $
 */
public enum PlanCapitalTransTypeEnum implements IEnum {

    /**
     * 已转让使用
     */
    AST_REPO("PCT02", "资产回收兑付", true, TransCodeEnum.REPO, AstFlowTypeEnum.FLOW_IN,
            PlanCapitalTransAcct.NORMAL_BANK, PlanCapitalTransAcct.RECV_PLAT),
    
    BUY_AST("PCT03", "资产交易兑付", true, TransCodeEnum.BUY_AST, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.NORMAL_PLAT, PlanCapitalTransAcct.RECV_BANK),
    
    DIST("PCT04", "承接池分配", true, TransCodeEnum.ALLOC, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.NORMAL_PLAT, PlanCapitalTransAcct.HOSTED_BANK),
    
    GRANT_REVERSE("PCT05", "放款冲正", true, TransCodeEnum.GRANT_REVERSE, AstFlowTypeEnum.FLOW_IN,
            PlanCapitalTransAcct.NORMAL_BANK, PlanCapitalTransAcct.PAY_PLAT),
    // --- 招联暂无
    REPO_REVERSE("PCT06", "还款冲正", true, TransCodeEnum.REPO_REVERSE, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.NORMAL_PLAT, PlanCapitalTransAcct.RECV_BANK),
    
    PAY_HANG_INTR("PCT07", "支付应付利息", true, TransCodeEnum.PAY_HANG_INTR, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.NORMAL_PLAT, PlanCapitalTransAcct.RECV_BANK),
    
    /**
     * 未转让 回款户转放款户
     */
    NT_AST_REPO("PCT11", "未转资产回收划账", false, TransCodeEnum.REPO, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.RECV_PLAT, PlanCapitalTransAcct.PAY_BANK),
    
    NT_BUY_AST("PCT12", "资产交易兑付划账", false, TransCodeEnum.BUY_AST, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.RECV_PLAT, PlanCapitalTransAcct.PAY_BANK),
    // --- 招联暂无
    NT_REPO_REVERSE("PCT13", "还款冲正", false, TransCodeEnum.REPO_REVERSE, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.RECV_PLAT, PlanCapitalTransAcct.PAY_BANK),
    
    NT_PAY_HANG_INTR("PCT14", "应付利息兑付回收划账", false, TransCodeEnum.PAY_HANG_INTR, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.RECV_PLAT, PlanCapitalTransAcct.PAY_BANK),
    
//    没有交易
//    NT_GRANT_REVERSE("PCT15", "放款冲正", false, TransCodeEnum.GRANT_REVERSE, AstFlowTypeEnum.FLOW_OUT,
//            PlanCapitalTransAcct.PAY_PLAT, PlanCapitalTransAcct.PAY_BANK),
    
    NT_DIST("PCT16", "放款池分配", false, TransCodeEnum.ALLOC, AstFlowTypeEnum.FLOW_OUT,
            PlanCapitalTransAcct.RECV_PLAT, PlanCapitalTransAcct.HOSTED_BANK),
    
    /**
     * 对于放款池计划 超过放款截止日 次日进行 放款户向 回款户资金划账
     */
    SPV_REMIT_REPO("PCT20", "向回款户资金划账", false, TransCodeEnum.CAP_ADJ_ADD, AstFlowTypeEnum.FLOW_IN,
            PlanCapitalTransAcct.RECV_BANK, PlanCapitalTransAcct.PAY_PLAT),
    
    /**
     * 回购交易
     */
    // 承接池A 回购交易卖出资产 承接池B
    BUY_BACK("PCT21", "资产回购兑付", true, TransCodeEnum.BUY_BACK_OUT, AstFlowTypeEnum.FLOW_IN,
            PlanCapitalTransAcct.NORMAL_BANK, PlanCapitalTransAcct.NORMAL_PLAT),
    
    // 承接池A 回购交易卖出资产 放款池A 从放款户扣钱
    NT_BUY_BACK("PCT22", "资产回购兑付", false, TransCodeEnum.BUY_BACK_OUT, AstFlowTypeEnum.FLOW_IN,
            PlanCapitalTransAcct.NORMAL_BANK, PlanCapitalTransAcct.PAY_PLAT),
    
    ;
    
    private static final Map<String, PlanCapitalTransTypeEnum> codeMap = new HashMap<>();
    
    static {
        for (PlanCapitalTransTypeEnum item : PlanCapitalTransTypeEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }
    
    private String               code;
    private String               desc;
    private boolean              isTransfer;
    /**
     * 交易编码
     */
    private TransCodeEnum        transCode;
    /**
     * 水位资金流向
     */
    private AstFlowTypeEnum      flowType;
    /**
     * 发起机构源账户
     */
    private PlanCapitalTransAcct srcAcct;
    /**
     * 对手机构目的账户
     */
    private PlanCapitalTransAcct desAcct;
    
    /**
     * @param code                      编码
     * @param desc                      描述
     * @param isTransfer                是否已转让
     * @param transCode                 对应交易码
     * @param astFlowType               资金流向
     * @param srcAcct  源账户
     * @param desAcct 目的账户
     */
    PlanCapitalTransTypeEnum(String code, String desc, boolean isTransfer, TransCodeEnum transCode,
                             AstFlowTypeEnum astFlowType,
                             PlanCapitalTransAcct srcAcct,
                             PlanCapitalTransAcct desAcct) {
        this.code = code;
        this.desc = desc;
        this.isTransfer = isTransfer;
        this.transCode = transCode;
        this.flowType = astFlowType;
        this.srcAcct = srcAcct;
        this.desAcct = desAcct;
    }
    
    public static PlanCapitalTransTypeEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    /**
     * 获取交易类型编码
     * @param capitalTrader
     * @return
     */
    public static PlanCapitalTransTypeEnum getCapitalTransType(CapitalTrader capitalTrader) {
        for (PlanCapitalTransTypeEnum type : PlanCapitalTransTypeEnum.values()) {
            if (type.getTransCode() == capitalTrader.getTransCode()
                && type.isTransfer() == capitalTrader.isTransfer()) {
                // 如果为回购 并且交易对手为放款池 时使用 NT_BUY_BACK
                if (type == BUY_BACK && capitalTrader.getCntPtyType() == PlanTypeEnum.SPV){
                    return PlanCapitalTransTypeEnum.NT_BUY_BACK;
                }else{
                    return type;
                }
            }
        }
        throw new IllegalArgumentException(MessageUtil.formatMsg(
                "不支持的计划资金事件：{0}", JSONUtil.toJsonStr(capitalTrader)));
        
    }
    
    public static PlanCapitalTransTypeEnum getCapitalTransTypeByTransfer(TransCodeEnum transCode, boolean isTransfer) {
        for (PlanCapitalTransTypeEnum type : PlanCapitalTransTypeEnum.values()) {
            if (type.getTransCode() == transCode && type.isTransfer() == isTransfer) {
                return type;
            }
        }
        throw new IllegalArgumentException("不支持的计划资金事件：" + transCode + "-" + isTransfer);
    }
    
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
    public String getDesc() {
        return desc;
    }
    
    public AstFlowTypeEnum getFlowType() {
        return flowType;
    }
    
    public PlanCapitalTransAcct getSrcAcct() {
        return srcAcct;
    }
    
    public PlanCapitalTransAcct getDesAcct() {
        return desAcct;
    }
    
    public TransCodeEnum getTransCode() {
        return transCode;
    }
    
    public boolean isTransfer() {
        return isTransfer;
    }
}