/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.model.plan;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;

import java.util.Date;

/**
 * @author hanfei
 * @version $Id: PlanAst.java, v 0.1 2018-01-04 下午8:07 hanfei Exp $
 */
public class PlanAst extends BaseEntity {
    
    /**
     * 计划类型
     */
    private PlanTypeEnum planType;
    /**
     * gmtModified 修改时间.
     */
    private Date         gmtModified;
    /**
     * lastUpdateDate 最近变更会计日.
     */
    private Date         lastUpdateDate;
    /**
     * cash 资金余额.
     */
    private Money        cash            = new Money();
    /**
     * frozenCash 冻结资金余额.
     */
    private Money        frozenCash      = new Money();
    /**
     * accTransPrin 累计转让本金.
     */
    private Money        accGrant        = new Money();
    /**
     * 支付在途金额
     */
    private Money        payOnWay        = new Money();
    /**
     * 还款在途金额
     */
    private Money repoOnWay       = new Money();
    /**
     * 当前银行账户余额
     */
    private Money bankBal         = new Money();
    /**
     * 变更前银行账户余额
     */
    private Money lastBankBal     = new Money();
    /**
     * 支付待结算金额
     */
    private Money payClear        = new Money();
    /**
     * 还款待结算金额
     */
    private Money repoClear       = new Money();
    /**
     * 校验水位金额
     */
    private Money checkCash    = new Money();
    /**
     * 水位状态
     */
    private TaskStatusEnum status;
    /**
     * planNo 计划编号.
     */
    private String         planNo;
    
    /**
     * 当前实际可用余额
     * 可用余额 = 余额 Cash - 冻结金额Frozen - 待支出金额（在途 + 待结算）
     * @return
     */
    public Money containClearAbleBal() {
        Money frozen = this.getFrozenCash().isGreaterThanZero() ? this.getFrozenCash() : new Money();
        Money ableCash = this.getCash().subtract(frozen).subtract(payAmt());
        if (!ableCash.isGreaterThanZero()) {
            return new Money();
        } else {
            return ableCash;
        }
    }
    
    /**
     * 当前可用支付余额
     * 可用支付余额 = 余额 Cash - 冻结金额Frozen - 支出在途金额
     * @return
     */
    public Money ablePayCash() {
        Money frozen = this.getFrozenCash().isGreaterThanZero() ? this.getFrozenCash() : new Money();
        Money onWay = this.getPayOnWay().isGreaterThanZero() ? this.getPayOnWay() : new Money();
        Money ableCash = this.getCash().subtract(frozen).subtract(onWay);
        if (!ableCash.isGreaterThanZero()) {
            return new Money();
        } else {
            return ableCash;
        }
    }
    
    /**
     * 支出金额 在途+待结算
     * @return
     */
    public Money payAmt() {
        Money onWay = this.getPayOnWay().isGreaterThanZero() ? this.getPayOnWay() : new Money();
        Money clear = this.getPayClear().isGreaterThanZero() ? this.getPayClear() : new Money();
        return onWay.add(clear);
    }
    
    /**
     * 收入金额 在途+待结算
     * @return
     */
    public Money repoAmt() {
        Money onWay = this.getRepoOnWay().isGreaterThanZero() ? this.getRepoOnWay() : new Money();
        Money clear = this.getRepoClear().isGreaterThanZero() ? this.getRepoClear() : new Money();
        return onWay.add(clear);
    }

    
    public Date getGmtModified() {
        return gmtModified;
    }
    
    public void setGmtModified(Date gmtModified) {
        this.gmtModified = gmtModified;
    }
    
    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }
    
    public void setLastUpdateDate(Date lastUpdateDate) {
        this.lastUpdateDate = lastUpdateDate;
    }
    
    public Money getCash() {
        return cash;
    }
    
    public void setCash(Money cash) {
        this.cash = cash;
    }
    
    public Money getFrozenCash() {
        return frozenCash;
    }
    
    public void setFrozenCash(Money frozenCash) {
        this.frozenCash = frozenCash;
    }
    
    public Money getAccGrant() {
        return accGrant;
    }
    
    public void setAccGrant(Money accGrant) {
        this.accGrant = accGrant;
    }
    
    public String getPlanNo() {
        return planNo;
    }
    
    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }
    
    public TaskStatusEnum getStatus() {
        return status;
    }
    
    public void setStatus(TaskStatusEnum status) {
        this.status = status;
    }
    
    public Money getPayOnWay() {
        return payOnWay;
    }
    
    public void setPayOnWay(Money payOnWay) {
        this.payOnWay = payOnWay;
    }
    
    public Money getRepoOnWay() {
        return repoOnWay;
    }
    
    public void setRepoOnWay(Money repoOnWay) {
        this.repoOnWay = repoOnWay;
    }
    
    public Money getBankBal() {
        return bankBal;
    }
    
    public void setBankBal(Money bankBal) {
        this.bankBal = bankBal;
    }
    
    public Money getLastBankBal() {
        return lastBankBal;
    }
    
    public void setLastBankBal(Money lastBankBal) {
        this.lastBankBal = lastBankBal;
    }
    
    public Money getPayClear() {
        return payClear;
    }
    
    public void setPayClear(Money payClear) {
        this.payClear = payClear;
    }
    
    public Money getRepoClear() {
        return repoClear;
    }
    
    public void setRepoClear(Money repoClear) {
        this.repoClear = repoClear;
    }
    
    public PlanTypeEnum getPlanType() {
        return planType;
    }
    
    public void setPlanType(PlanTypeEnum planType) {
        this.planType = planType;
    }
    
    public Money getCheckCash() {
        return checkCash;
    }
    
    public void setCheckCash(Money checkCash) {
        this.checkCash = checkCash;
    }
}
