/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.model.rule;

import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.general.system.common.model.BaseEntity;

import java.math.BigDecimal;

/**
 * 资产回购定价规则
 * @author hanfei
 * @version $Id: AstPriceRule.java, v 0.1 2020-02-07 4:53 PM hanfei Exp $ 
 */
public class AstPriceRule extends BaseEntity implements TradePriceRule<String>{
    
    /**
     * riskCl .
     */
    private String     riskCl;
    /**
     * abssqrProdCode .
     */
    private String     abssqrProdCode;
    /**
     * intrRate .
     */
    private BigDecimal intrRate;
    /**
     * prinRate .
     */
    private BigDecimal prinRate;
    /**
     * penalFeeRate .
     */
    private BigDecimal penalFeeRate;
    
    
    @Override
    public boolean getTradePriceRule(String condition) {
        return StrUtil.equals(this.riskCl, condition);
    }
    
    @Override
    public BigDecimal getTradeRate(DebtMoneyFieldEnum debt) {
        switch (debt){
            // 本金
            case PRIN:
            case OVD_PRIN:
                return prinRate;
            //利息
            case INTR:
            case OVD_INTR:
                return intrRate;
            // 其他罚与费
            case PRIN_ODUE_PNY:
            case INTR_ODUE_PNY:
            case FEE:
            case OVD_FEE:
            case FEE_PNY:
            case OTHER_FEE:
            case OTHER_FEE_PNY:
                return penalFeeRate;
            default:
                throw new RuntimeException("不支持的定价类型");
        }
    }
    
    @Override
    public String getSort() {
        return riskCl;
    }
    
    public String getRiskCl() {
        return riskCl;
    }
    
    public void setRiskCl(String riskCl) {
        this.riskCl = riskCl;
    }
    
    public String getAbssqrProdCode() {
        return abssqrProdCode;
    }
    
    public void setAbssqrProdCode(String abssqrProdCode) {
        this.abssqrProdCode = abssqrProdCode;
    }
    
    public BigDecimal getIntrRate() {
        return intrRate;
    }
    
    public void setIntrRate(BigDecimal intrRate) {
        this.intrRate = intrRate;
    }
    
    public BigDecimal getPrinRate() {
        return prinRate;
    }
    
    public void setPrinRate(BigDecimal prinRate) {
        this.prinRate = prinRate;
    }
    
    public BigDecimal getPenalFeeRate() {
        return penalFeeRate;
    }
    
    public void setPenalFeeRate(BigDecimal penalFeeRate) {
        this.penalFeeRate = penalFeeRate;
    }
    
    
}
