/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.facade.validation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.Map;

/**
 * 校验{@link Collection}和{@link Map}的size范围<br/>
 * {@code @{@link VldCollection }(sizeRange=@{@link VldRange }(min=1,max=2)) 等价于 集合的size为(1,2)<br/>
 * 本注解可校验的类型是:{@link Collection},{@link Map}<br/>
 * @see VldRange
 * @see Collection
 * @see Map
 * 
 * @author zhenxuan.luo
 * @version $Id: VldCollection.java, v 0.1 2015年5月18日 下午3:50:04 zhenxuan.luo Exp $
 */
@Documented
@Target({ ElementType.METHOD, ElementType.FIELD })
@Retention(RetentionPolicy.RUNTIME)
@Bind(verifiableClass = { Collection.class })
public @interface VldCollection {
    /**
     * 
     * @return 校验错误后信息
     */
    String message() default "{invalid check result for collection}";

    /**
     * 最小size
     * 
     * @return
     */
    int minSize() default 0;

    /**
     * 最大size
     * 
     * @return
     */
    int maxSize() default Integer.MAX_VALUE;

    /**
     * 是否嵌套校验,默认不嵌套
     * @return true:嵌套校验集合中的元素;false:不嵌套校验
     */
    boolean nestable() default true;

    /**
     * 是否允许为空(默认不允许为空,为空是要报错的)
     * @return true:允许为空;反之返回false
     */
    boolean nullable() default false;
}
