/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.common.integration.notice;

import cn.hutool.core.map.MapUtil;
import com.abssqr.plat.common.integration.notice.enums.RobotTypeEnum;
import com.abssqr.plat.common.integration.notice.model.RobotRequest;
import com.abssqr.plat.common.integration.notice.utils.RobotFactory;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author xiachenxiang
 * @version com.abssqr.plat.common.integration.notice: RobotConfigration.java, v 0.1 2019-09-30 6:15 PM xiachenxiang Exp $
 */
@Component
@ConfigurationProperties(prefix = "robot")
public class RobotConfig {
    /**
     * 通知机器人配置列表
     */
    private List<Map<String, String>> robotList = new ArrayList<>();
    
    private Map<String, List<RobotRequest>> robotMap = new HashMap<>();
    
    /**
     * 初始化外部通知接口配置信息
     */
    @PostConstruct
    public void init() {
        robotMap = Maps.newHashMap();
        if (CollectionUtils.isNotEmpty(robotList)) {
            robotList.forEach(map -> {
                RobotRequest robotRequest;
                String token = MapUtil.getStr(map, "token");
                String type = MapUtil.getStr(map, "type");
                String robotType = MapUtil.getStr(map, "robotType");
                // 代理配置
                Boolean proxy = MapUtil.getBool(map, "proxy");
                String server = MapUtil.getStr(map, "server");
                Integer port = MapUtil.getInt(map, "port");
                
                if (!ToolUtil.isOneEmpty(token, type, robotType)) {
                    robotRequest = RobotFactory.newInstance(RobotTypeEnum.getByCode(robotType));
                    // 配置代理
                    if (!ToolUtil.isOneEmpty(proxy, server, port) && proxy){
                        robotRequest.setProxy(new Proxy(Proxy.Type.HTTP, new InetSocketAddress(server, port)));
                    }
                    
                    robotRequest.setUrl(token);
                    robotRequest.setType(type);
                    List<RobotRequest> robotList = robotMap.get(type);
                    if (robotList == null) {
                        robotList = Lists.newArrayList();
                        robotMap.put(type, robotList);
                    }
                    robotList.add(robotRequest);
                }
            });
        }
    }

    public List<Map<String, String>> getRobotList() {
        return robotList;
    }

    public void setRobotList(List<Map<String, String>> robotList) {
        this.robotList = robotList;
    }

    public List<RobotRequest> getRobotRequestListByType(String type){
        return robotMap.getOrDefault(type,robotMap.get("default"));
    }

    public void setRobotMap(Map<String, List<RobotRequest>> robotMap) {
        this.robotMap = robotMap;
    }
}
