/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.convertor;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstScreenRuleDO;
import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.enums.ScreenTypeEnum;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.ast.AstScreenRuleEditParam;
import com.abssqr.plat.common.facade.param.ast.astScreenRuleArea.RuleArea;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.model.utils.TextUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author yaphet
 * @version com.abssqr.plat.common.model.convertor: AstScreenRuleConvert.java, v 0.1 2018-03-29 上午11:52 yaphet Exp $
 */
public class AstScreenRuleConvert {

    public static AstScreenRule convert2Domain(AstScreenRuleDO param) {
        if (param == null) {
            return null;
        }
        AstScreenRule result = new AstScreenRule();

        //singleMaxAmount 单笔最大金额
        if (param.getSingleMaxAmount() != null) {
            result.setSingleMaxAmount(Money.createWithCent(param.getSingleMaxAmount()));
        }
        //singleMinAmount 单笔最小金额
        if (param.getSingleMinAmount() != null) {
            result.setSingleMinAmount(Money.createWithCent(param.getSingleMinAmount()));
        }

        //region 地区
        if (StringUtils.isNotBlank(param.getRegion())) {
            JSONArray array = JSON.parseArray(param.getRegion());
            result.setRegionList(array.toJavaList(RuleArea.class));
        }
        //orgCode 原始权益机构
        result.setLenderOrgCode(param.getLenderOrgCode());
        //AbssqrProdCode 内部准入产品码
        result.setAbssqrProdCode(param.getAbssqrProdCode());
        //prodName 准入产品名称
        result.setProdName(param.getProdName());
        //ProdCode 外部产品码
        result.setProdCode(param.getProdCode());
        //assetStatus 资产状态,|分割，对应五级分类
        if (StringUtils.isNotBlank(param.getAssetStatus())) {
            List<String> assetStatusList = TextUtil.convertTxt2List(param.getAssetStatus());
            List<AstRiskClassEnum> astStatusEnumList =
                    assetStatusList.stream().map(code -> AstRiskClassEnum.getByCode(code)).collect(Collectors.toList());
            result.setAssetStatusList(astStatusEnumList);
        }
        //assetScreeningRuleNo 资产筛选规则编号
        result.setScreenRuleNo(param.getScreenRuleNo());
        //assetScreeningRuleName 资产筛选规则编号
        result.setScreenRuleName(param.getScreenRuleName());
        //maxAge 最大年龄
        result.setMaxAge(param.getMaxAge());
        //minAge 最小年龄
        result.setMinAge(param.getMinAge());
        //maxTerm 最大期次
        result.setMaxTerm(param.getMaxTerm());
        //minTerm 最小期次
        result.setMinTerm(param.getMinTerm());
        //maxRemainDays 最大剩余天数
        result.setMaxRemainDays(param.getMaxRemainDays());
        //minRemainDays 最小剩余天数
        result.setMinRemainDays(param.getMinRemainDays());
        //latestEndDate 最晚到期日
        result.setLatestEndDate(param.getLatestEndDate());
        //earliestEndDate 最早到期日
        result.setEarliestEndDate(param.getEarliestEndDate());
        //singleMaxInterRate 单笔最高利率
        result.setSingleMaxInterRate(param.getSingleMaxInterRate());
        //singleMinInterRate 单笔最单利率
        result.setSingleMinInterRate(param.getSingleMinInterRate());
        //ApproveStatus 审批状态
        result.setApproveStatus(ApproveStatusEnum.getEnum(param.getApproveStatus()));
        //GmtCreator  创建者
        if (param.getGmtCreator() != null) {
            result.setGmtCreator(param.getGmtCreator());
        }
        //GmtModifier  修改人
        if (param.getGmtModifier() != null) {
            result.setGmtModifier(param.getGmtModifier());
        }
        result.setManagerOrgCode(param.getManagerOrgCode());
        result.setScreenType(ScreenTypeEnum.getByCode(param.getScreenType()));
        return result;
    }

    /**
     * manager使用
     *
     * @param param
     * @return
     */
    public static AstScreenRule convertParam2Domain(AstScreenRuleEditParam param) {
        if (param == null) {
            return null;
        }
        AstScreenRule result = new AstScreenRule();
        //assetScreeningRuleNo 资产筛选规则编号
        result.setScreenRuleNo(param.getScreenRuleNo());
        if (StringUtils.isBlank(result.getScreenRuleNo())) {
            result.setScreenRuleNo(SequenceUtil.genId(IdTypeEnum.ASTSCREENINGRULE));
        }
        //assetScreeningRuleName 资产筛选规则编号  需要去空格效验
        result.setScreenRuleName(param.getScreenRuleName().trim());
        //singleMaxAmount 单笔最大金额
        if (param.getSingleMaxAmount() != null) {
            result.setSingleMaxAmount(new Money(param.getSingleMaxAmount()));
        }
        //singleMinAmount 单笔最小金额
        if (param.getSingleMinAmount() != null) {
            result.setSingleMinAmount(new Money(param.getSingleMinAmount()));
        }
        //region 地区
        if (CollectionUtils.isNotEmpty(param.getRegion())) {
            result.setRegionList(param.getRegion());
        }
        //orgCode 原始权益机构
        result.setLenderOrgCode(param.getLenderOrgCode());
        //AbssqrProdCode 内部准入产品码
        result.setAbssqrProdCode(param.getAbssqrProdCode());
        //prodCode 准入产品码
        result.setAbssqrProdCode(param.getAbssqrProdCode());
        //assetStatus 资产状态,|分割，对应五级分类
        if (StringUtils.isNotBlank(param.getAssetStatus())) {
            List<String> assetStatusList = TextUtil.convertTxt2List(param.getAssetStatus());
            List<AstRiskClassEnum> astStatusEnumList =
                    assetStatusList.stream().map(code -> AstRiskClassEnum.getByCode(code)).collect(Collectors.toList());
            result.setAssetStatusList(astStatusEnumList);
        }
        //maxAge 最大年龄
        result.setMaxAge(param.getMaxAge());
        //minAge 最小年龄
        result.setMinAge(param.getMinAge());
        //maxTerm 最大期次
        result.setMaxTerm(param.getMaxTerm());
        //minTerm 最小期次
        result.setMinTerm(param.getMinTerm());
        //maxRemainDays 最大剩余天数
        result.setMaxRemainDays(param.getMaxRemainDays());
        //minRemainDays 最小剩余天数
        result.setMinRemainDays(param.getMinRemainDays());
        //latestEndDate 最晚到期日
        result.setLatestEndDate(param.getLatestEndDate());
        //earliestEndDate 最早到期日
        result.setEarliestEndDate(param.getEarliestEndDate());

        //singleMaxInterRate 单笔最高利率
        if (param.getSingleMaxInterRate() != null) {
            result.setSingleMaxInterRate(CalcUtil.calcRealRate(param.getSingleMaxInterRate()));
        }
        //singleMinInterRate 单笔最单利率
        if (param.getSingleMinInterRate() != null) {
            result.setSingleMinInterRate(CalcUtil.calcRealRate(param.getSingleMinInterRate()));
        }
        result.setScreenType(ScreenTypeEnum.getByCode(param.getScreenType()));
        return result;
    }

    public static AstScreenRuleDO convertRule2Do(AstScreenRule param) {
        if (param == null) {
            return null;
        }
        AstScreenRuleDO result = new AstScreenRuleDO();
        //assetScreeningRuleNo 资产筛选规则编号
        result.setScreenRuleNo(param.getScreenRuleNo());
        if (StringUtils.isBlank(result.getScreenRuleNo())) {
            result.setScreenRuleNo(SequenceUtil.genId(IdTypeEnum.ASTSCREENINGRULE));
        }
        //assetScreeningRuleName 资产筛选规则编号
        result.setScreenRuleName(param.getScreenRuleName());
        //singleMaxAmount 单笔最大金额
        if (param.getSingleMaxAmount() != null) {
            result.setSingleMaxAmount(param.getSingleMaxAmount().getCent());
        }
        //singleMinAmount 单笔最小金额
        if (param.getSingleMinAmount() != null) {
            result.setSingleMinAmount(param.getSingleMinAmount().getCent());
        }
        //region 地区
        if (CollectionUtils.isNotEmpty(param.getRegionList())) {
            result.setRegion(JSON.toJSONString(param.getRegionList()));
        }
        //orgCode 原始权益机构
        result.setLenderOrgCode(param.getLenderOrgCode());
        //prodCode 准入产品码
        result.setAbssqrProdCode(param.getAbssqrProdCode());
        //prodName 准入产品名称
        result.setProdName(param.getProdName());
        //prodCode 准入产品码
        result.setProdCode(param.getProdCode());
        //assetStatus 资产状态,|分割，对应五级分类
        if (CollectionUtils.isNotEmpty(param.getAssetStatusList())) {
            result.setAssetStatus(CollectionUtil.join(
                    param.getAssetStatusList().stream().map(AstRiskClassEnum::getCode).collect(Collectors.toList()), ","));
        }
        //maxAge 最大年龄
        result.setMaxAge(param.getMaxAge());
        //minAge 最小年龄
        result.setMinAge(param.getMinAge());
        //maxTerm 最大期次
        result.setMaxTerm(param.getMaxTerm());
        //minTerm 最小期次
        result.setMinTerm(param.getMinTerm());
        //maxRemainDays 最大剩余天数
        result.setMaxRemainDays(param.getMaxRemainDays());
        //minRemainDays 最小剩余天数
        result.setMinRemainDays(param.getMinRemainDays());
        //latestEndDate 最晚到期日
        result.setLatestEndDate(param.getLatestEndDate());
        //earliestEndDate 最早到期日
        result.setEarliestEndDate(param.getEarliestEndDate());
        //singleMaxInterRate 单笔最高利率
        result.setSingleMaxInterRate(param.getSingleMaxInterRate());
        //singleMinInterRate 单笔最单利率
        result.setSingleMinInterRate(param.getSingleMinInterRate());
        //ApproveStatus 审批状态
        result.setApproveStatus(param.getApproveStatus().getCode());
        //GmtCreator  创建者
        if (param.getGmtCreator() != null) {
            result.setGmtCreator(param.getGmtCreator());
        }
        //GmtModifier  修改人
        if (param.getGmtModifier() != null) {
            result.setGmtModifier(param.getGmtModifier());
        }
        result.setManagerOrgCode(param.getManagerOrgCode());
        result.setScreenType(Optional.ofNullable(param.getScreenType()).map(ScreenTypeEnum::getCode).orElse(null));
        return result;
    }
}
