/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.facade.enums.CalcTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeScaleTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeTimeTypeEnum;
import com.abssqr.plat.common.facade.enums.FinSerFeeTypeEnum;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.facade.param.plan.sub.FeeStructureParam;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.general.system.common.model.Money;
import org.apache.commons.collections.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author yangcheng
 * @version FeeStructureConvert.java, v0.12019-08-2716:52 yangchengExp$
 */
public class FeeStructureConvert {
    
    /**
     * 费用模型转换
     * @param list
     * @return
     */
    public static List<FeeStructure> convert2FeeStructureList(List<FeeStructureParam> list) {
        List<FeeStructure> feeStructureList = new ArrayList();
        if (CollectionUtils.isEmpty(list)) {
            return feeStructureList;
        }
        for (FeeStructureParam feeStructureParam : list) {
            FeeStructure feeStructure = new FeeStructure();
            //fixedFeeAmt 固定费用
            if (feeStructureParam.getFixedFeeAmt() != null) {
                feeStructure.setFixedFeeAmt(new Money(feeStructureParam.getFixedFeeAmt()).multiply(10000));
            }
            //payLimitAmt 支付上限
            if (feeStructureParam.getPayLimitAmt() != null) {
                feeStructure.setPayLimitAmt(new Money(feeStructureParam.getPayLimitAmt()).multiply(10000));
            }
            //feeNo 费用编号
            feeStructure.setFeeNo(feeStructureParam.getFeeNo());
            //feeName 费用名称
            feeStructure.setFeeName(feeStructureParam.getFeeName());
            //feeCalcType 计算方式
            feeStructure.setFeeCalcType(CalcTypeEnum.getByCode(feeStructureParam.getFeeCalcType()));
            //feeTimeType 计费时间类型
            feeStructure.setFeeTimeType(FeeTimeTypeEnum.getByCode(feeStructureParam.getFeeTimeType()));
            //feeScaleType 费率基准
            feeStructure.setFeeScaleType(FeeScaleTypeEnum.getByCode(feeStructureParam.getFeeScaleType()));
            //calcByDay 按日计费
            feeStructure.setCalcByDay(feeStructureParam.isCalcByDay());
            //feeRate 费率
            feeStructure.setFeeRate(CalcUtil.calcRealRate(feeStructureParam.getFeeRate()));
            feeStructure.setFinSerFeeType(FinSerFeeTypeEnum.getByCode(feeStructureParam.getFinSerFeeType()));
            feeStructureList.add(feeStructure);
        }
        
        return feeStructureList;
    }
}
