/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanAstProfitDO;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

/**
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.convertor: LoanConvert.java, v 0.1 2018-03-07 下午4:14 guozhenhua Exp $ 
 */
public class LoanAstProfitConvert {
    public static List<LoanAstProfit> convert2Domains(List<LoanAstProfitDO> var) {
        List<LoanAstProfit> ret = Lists.newArrayList();
        
        if (CollectionUtils.isNotEmpty(var)) {
            for (LoanAstProfitDO loanAstProfitDO : var) {
                LoanAstProfit loanAstProfit = convert2Domain(loanAstProfitDO);
                if (null != loanAstProfit) {
                    ret.add(loanAstProfit);
                }
            }
        }
        return ret;
    }
    
    public static LoanAstProfit convert2Domain(LoanAstProfitDO var) {
        if (var == null) {
            return null;
        }
        LoanAstProfit instance = new LoanAstProfit(false);
        instance.setAstProfitNo(var.getAstProfitNo());
        instance.setLoanAstNo(var.getLoanAstNo());
        instance.setProfitType(ProfitTypeEnum.getByCode(var.getProfitType()));
        instance.setAstNo(var.getAstNo());
        instance.setOutAstNo(var.getOutAstNo());
        instance.setTermNo(var.getTermNo());
        instance.setSrcTermNo(var.getSrcTermNo());
        instance.setOwnOrg(var.getOwnOrg());
        instance.setSplitOrgCode(var.getSplitOrgCode());
        instance.setSplitRate(var.getSplitRate());
        instance.setExtInfo(var.getExtInfo());
        instance.setLastUpdateDate(var.getLastUpdateDate());
        instance.setIntrEntryDec(var.getIntrEntryDec());
        instance.setIntrPenalEntryDec(var.getIntrPenalEntryDec());
        instance.setPrinPenalEntryDec(var.getPrinPenalEntryDec());
        instance.setFeeEntryDec(var.getFeeEntryDec());
        instance.setFeePenalEntryDec(var.getFeePenalEntryDec());
        instance.setOtherFeeEntryDec(Optional.ofNullable(var.getOtherFeeEntryDec()).orElse(BigDecimal.ZERO));
        // 收益成本
        instance.setCostAmt(var.getCostAmt());
        
        DebtMoney bal = new DebtMoney();
        bal.setByFieldType(DebtMoneyFieldEnum.INTR, var.getIntrBal());
        bal.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, var.getPrinPenalBal());
        bal.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, var.getIntrPenalBal());
        bal.setByFieldType(DebtMoneyFieldEnum.FEE, var.getFeeBal());
        bal.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, var.getFeePenalBal());
        bal.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,var.getOtherFeeBal());
        instance.setBal(bal);
        
        DebtMoney rmtAmt = new DebtMoney();
        rmtAmt.setByFieldType(DebtMoneyFieldEnum.INTR, var.getRmtIntrAmt());
        rmtAmt.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, var.getRmtPrinPenalAmt());
        rmtAmt.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, var.getRmtIntrPenalAmt());
        rmtAmt.setByFieldType(DebtMoneyFieldEnum.FEE, var.getRmtFeeAmt());
        rmtAmt.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, var.getRmtFeePenalAmt());
        instance.setRmtAmt(rmtAmt);
        
        DebtMoney fundAmt = new DebtMoney();
        fundAmt.setByFieldType(DebtMoneyFieldEnum.INTR, var.getFundIntrAmt());
        fundAmt.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, var.getFundPrinPenalAmt());
        fundAmt.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, var.getFundIntrPenalAmt());
        fundAmt.setByFieldType(DebtMoneyFieldEnum.FEE, var.getFundFeeAmt());
        fundAmt.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, var.getFundFeePenalAmt());
        fundAmt.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,var.getFundOtherFeeAmt());
        instance.setFundAmt(fundAmt);
        
        DebtMoney paidAmt = new DebtMoney();
        paidAmt.setByFieldType(DebtMoneyFieldEnum.INTR, var.getPaidIntrAmt());
        paidAmt.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, var.getPaidPrinPenalAmt());
        paidAmt.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, var.getPaidIntrPenalAmt());
        paidAmt.setByFieldType(DebtMoneyFieldEnum.FEE, var.getPaidFeeAmt());
        paidAmt.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, var.getPaidFeePenalAmt());
        paidAmt.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,var.getPaidOtherFeeAmt());
        instance.setPaidAmt(paidAmt);
        
        DebtMoney entryAmt = new DebtMoney();
        entryAmt.setByFieldType(DebtMoneyFieldEnum.INTR, var.getIntrEntryAmt());
        entryAmt.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, var.getPrinPenalEntryAmt());
        entryAmt.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, var.getIntrPenalEntryAmt());
        entryAmt.setByFieldType(DebtMoneyFieldEnum.FEE, var.getFeeEntryAmt());
        entryAmt.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, var.getFeePenalEntryAmt());
        entryAmt.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,var.getOtherFeeEntryAmt());
        instance.setEntryAmt(entryAmt);
        
        return instance;
    }
    
    public static LoanAstProfitDO convert2DO(LoanAstProfit var) {
        if (var == null) {
            return null;
        }
        
        LoanAstProfitDO instance = new LoanAstProfitDO();
        instance.setAstProfitNo(var.getAstProfitNo());
        instance.setLoanAstNo(var.getLoanAstNo());
        instance.setProfitType(var.getProfitType().getCode());
        instance.setAstNo(var.getAstNo());
        instance.setOutAstNo(var.getOutAstNo());
        instance.setTermNo(var.getTermNo());
        instance.setSrcTermNo(var.getSrcTermNo());
        instance.setOwnOrg(var.getOwnOrg());
        instance.setManagerOrgFee(var.getPaidAmt().getSum().subtract(var.getFundAmt().getSum()));
        instance.setSplitOrgCode(var.getSplitOrgCode());
        instance.setSplitRate(var.getSplitRate());
        instance.setExtInfo(var.getExtInfo());
        instance.setLastUpdateDate(var.getLastUpdateDate());
        instance.setIntrEntryDec(var.getIntrEntryDec());
        instance.setIntrPenalEntryDec(var.getIntrPenalEntryDec());
        instance.setPrinPenalEntryDec(var.getPrinPenalEntryDec());
        instance.setFeeEntryDec(var.getFeeEntryDec());
        instance.setFeePenalEntryDec(var.getFeePenalEntryDec());
        instance.setOtherFeeEntryDec(var.getOtherFeeEntryDec());
        // 收益成本
        instance.setCostAmt(var.getCostAmt());
        
        instance.setIntrBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPrinPenalBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setIntrPenalBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setFeeBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setFeePenalBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setOtherFeeBal(var.getBal().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        
        instance.setRmtIntrAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setRmtPrinPenalAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setRmtIntrPenalAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setRmtFeeAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setRmtFeePenalAmt(var.getRmtAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        
        instance.setFundIntrAmt(var.getFundAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setFundPrinPenalAmt(var.getFundAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setFundIntrPenalAmt(var.getFundAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setFundFeeAmt(var.getFundAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setFundFeePenalAmt(var.getFundAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setFundOtherFeeAmt(var.getFundAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        
        instance.setPaidIntrAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPaidPrinPenalAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setPaidIntrPenalAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setPaidFeeAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setPaidFeePenalAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setPaidOtherFeeAmt(var.getPaidAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        
        instance.setIntrEntryAmt(var.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPrinPenalEntryAmt(var.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setIntrPenalEntryAmt(var.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setFeeEntryAmt(var.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setFeePenalEntryAmt(var.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setOtherFeeEntryAmt(var.getEntryAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));

        return instance;
    }
    
}
