/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanLogDO;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.loanTrans.LoanLog;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;

/**
 * 账单分期明细与测算模块的还款计划模型互相转换
 *
 * @author yaphet
 * @version com.abssqr.plat.common.model.convertor: LoanInstalConvert.java, v 0.1 2018-06-20 上午10:53 yaphet Exp $
 */
public class LoanLogConvert {
    
    public static List<LoanLog> convert2Domains(List<LoanLogDO> vars) {
        
        List<LoanLog> loanInstals = Lists.newArrayList();
        
        if (CollectionUtils.isEmpty(vars)) {
            return loanInstals;
        }
        
        for (LoanLogDO loanLogDO : vars) {
            loanInstals.add(convert2Domain(loanLogDO));
        }
        
        return loanInstals;
    }
    
    public static LoanLog convert2Domain(LoanLogDO var) {
        
        if (null == var) {
            return null;
        }
        
        LoanLog instance = new LoanLog();
        instance.setUserId(var.getUserId());
        instance.setExecTms(var.getExecTms());
        instance.setAstNo(var.getAstNo());
        instance.setOutAstNo(var.getOutAstNo());
        instance.setOutBizNo(var.getOutBizNo());
        instance.setBizNo(var.getBizNo());
        instance.setSeqNo(var.getSeqNo());
        instance.setLenderOrgCode(var.getLenderOrgCode());
        instance.setManagerOrgCode(var.getManagerOrgCode());
        instance.setSponserOrgCode(var.getSponserOrgCode());
        instance.setOwnOrg(var.getOwnOrg());
        instance.setOwnPack(var.getOwnPack());
        instance.setProdNo(var.getProdNo());
        instance.setOutProdNo(var.getOutProdNo());
        instance.setContractNo(var.getContractNo());
        instance.setBfRiskCl(AstRiskClassEnum.getByCode(var.getBfRiskCl()));
        instance.setAfRiskCl(AstRiskClassEnum.getByCode(var.getAfRiskCl()));
        instance.setBfStatus(LoanStatusEnum.getByCode(var.getBfStatus()));
        instance.setAfStatus(LoanStatusEnum.getByCode(var.getAfStatus()));
        instance.setEventCode(EventCodeEnum.getByCode(var.getEventCode()));
        instance.setBfOvdDays(var.getBfOvdDays());
        instance.setAfOvdDays(var.getAfOvdDays());
        instance.setAfTermCnt(var.getAfTermCnt());
        instance.setBfTermCnt(var.getBfTermCnt());
        instance.setBfEndDate(var.getBfEndDate());
        instance.setAfEndDate(var.getAfEndDate());
        instance.setTransTime(var.getTransTime());
        instance.setAccountDate(var.getAccountDate());
        instance.setClrgAmt(var.getClrgAmt());
        instance.setAmt(createAmt(var));
        instance.setAfBal(createBal(var));
        instance.setAfOwnStatus(OwnStatusEnum.getByCode(var.getAfOwnStatus()));
        instance.setBfOwnStatus(OwnStatusEnum.getByCode(var.getBfOwnStatus()));
        instance.setAfWtfStatus(YNEnum.getByCode(var.getAfWtfStatus()));
        instance.setBfWtfStatus(YNEnum.getByCode(var.getBfWtfStatus()));
        instance.setBfCapitalNo(var.getBfCapitalNo());
        instance.setAfCapitalNo(var.getAfCapitalNo());
        instance.setMemo(var.getMemo());
        return instance;
        
    }
    
    public static LoanLogDO convert2Do(LoanLog var) {
        
        if (null == var) {
            return null;
        }
        
        LoanLogDO instance = new LoanLogDO();
    
        instance.setUserId(var.getUserId());
        instance.setExecTms(var.getExecTms());
        instance.setAstNo(var.getAstNo());
        instance.setOutAstNo(var.getOutAstNo());
        instance.setOutBizNo(var.getOutBizNo());
        instance.setBizNo(var.getBizNo());
        instance.setSeqNo(var.getSeqNo());
        instance.setLenderOrgCode(var.getLenderOrgCode());
        instance.setManagerOrgCode(var.getManagerOrgCode());
        instance.setSponserOrgCode(var.getSponserOrgCode());
        instance.setOwnOrg(var.getOwnOrg());
        instance.setOwnPack(var.getOwnPack());
        instance.setProdNo(var.getProdNo());
        instance.setOutProdNo(var.getOutProdNo());
        instance.setContractNo(var.getContractNo());
        instance.setBfRiskCl(var.getBfRiskCl().getCode());
        instance.setAfRiskCl(var.getAfRiskCl().getCode());
        instance.setBfStatus(var.getBfStatus().getCode());
        instance.setAfStatus(var.getAfStatus().getCode());
        instance.setEventCode(var.getEventCode().getCode());
        instance.setTransCode(var.getEventCode().getSceneCode().getCode());
        instance.setBfOvdDays(var.getBfOvdDays());
        instance.setAfOvdDays(var.getAfOvdDays());
        instance.setAfTermCnt(var.getAfTermCnt());
        instance.setBfTermCnt(var.getBfTermCnt());
        instance.setBfEndDate(var.getBfEndDate());
        instance.setAfEndDate(var.getAfEndDate());
        instance.setTransTime(var.getTransTime());
        instance.setAccountDate(var.getAccountDate());
        instance.setClrgAmt(var.getClrgAmt());
        
        instance.setPrinAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        instance.setOvdPrinAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OVD_PRIN));
        instance.setOvdIntrAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OVD_INTR));
        instance.setOvdFeeAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OVD_FEE));
        instance.setIntrAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setPrinPenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setIntrPenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setFeeAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setFeePenalAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setOtherFeeAmt(var.getAmt().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        
        instance.setAfPrinBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.PRIN));
        instance.setAfOvdPrinBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.OVD_PRIN));
        instance.setAfOvdIntrBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.OVD_INTR));
        instance.setAfOvdFeeBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.OVD_FEE));
        instance.setAfIntrBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.INTR));
        instance.setAfPrinPenalBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY));
        instance.setAfIntrPenalBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY));
        instance.setAfFeeBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.FEE));
        instance.setAfFeePenalBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.FEE_PNY));
        instance.setAfOtherFeeBal(var.getAfBal().getByFieldType(DebtMoneyFieldEnum.OTHER_FEE));
        instance.setAfOwnStatus(var.getAfOwnStatus().getCode());
        instance.setBfOwnStatus(var.getBfOwnStatus().getCode());
    
        instance.setAfWtfStatus(var.getAfWtfStatus().getCode());
        instance.setBfWtfStatus(var.getBfWtfStatus().getCode());
        instance.setBfCapitalNo(var.getBfCapitalNo());
        instance.setAfCapitalNo(var.getAfCapitalNo());
        instance.setMemo(var.getMemo());
        return instance;
        
    }
    
    private static DebtMoney createBal(LoanLogDO loanLogDO) {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN, new Money().addTo(loanLogDO.getAfPrinBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OVD_PRIN, new Money().addTo(loanLogDO.getAfOvdPrinBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OVD_INTR, new Money().addTo(loanLogDO.getAfOvdIntrBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OVD_FEE, new Money().addTo(loanLogDO.getAfOvdFeeBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,new Money().add(loanLogDO.getAfOtherFeeBal()));

        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, new Money().addTo(loanLogDO.getAfPrinPenalBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, new Money().addTo(loanLogDO.getAfIntrBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, new Money().addTo(loanLogDO.getAfIntrPenalBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, new Money().addTo(loanLogDO.getAfFeeBal()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, new Money().addTo(loanLogDO.getAfFeePenalBal()));
        return debtMoney;
    }
    
    private static DebtMoney createAmt(LoanLogDO loanLogDO) {
        DebtMoney debtMoney = new DebtMoney();
        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN, new Money().addTo(loanLogDO.getPrinAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OVD_PRIN, new Money().addTo(loanLogDO.getOvdPrinAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OVD_INTR, new Money().addTo(loanLogDO.getOvdIntrAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OVD_FEE, new Money().addTo(loanLogDO.getOvdFeeAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.OTHER_FEE,new Money().addTo(loanLogDO.getOtherFeeAmt()));

        debtMoney.setByFieldType(DebtMoneyFieldEnum.PRIN_ODUE_PNY, new Money().addTo(loanLogDO.getPrinPenalAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR, new Money().addTo(loanLogDO.getIntrAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.INTR_ODUE_PNY, new Money().addTo(loanLogDO.getIntrPenalAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE, new Money().addTo(loanLogDO.getFeeAmt()));
        debtMoney.setByFieldType(DebtMoneyFieldEnum.FEE_PNY, new Money().addTo(loanLogDO.getFeePenalAmt()));
        return debtMoney;
    }
    
}
