/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocItemDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAllocItemResultMap;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAllocItem;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocEditParam;
import com.abssqr.plat.common.facade.param.plan.alloc.sub.PlanAllocItemParam;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.convertor: PlanAllocConvert.java, v 0.1 2019-06-03 18:27 bangis.wangdf Exp $
 */
public class PlanAllocConvert {
    public static List<PlanAlloc> convertDomains(List<PlanAllocDO> planAllocDOS) {
        if (CollectionUtils.isEmpty(planAllocDOS)) {
            return null;
        }
        return planAllocDOS.stream().map(planAllocDO -> convertDomain(planAllocDO)).collect(Collectors.toList());
    }

    public static PlanAlloc convertDomain(PlanAllocDO param) {
        if (param==null){
            return null;
        }
        PlanAlloc result = new PlanAlloc();
        //allocAmt 本期分配总额
        result.setAllocAmt(Money.createWithCentNull(param.getAllocAmt()));
        //planNo 计划编号
        result.setPlanNo(param.getPlanNo());
        //allocNo 分配编号
        result.setAllocNo(param.getAllocNo());
        //payDate 兑付日
        result.setPayDate(param.getPayDate());
        //clearDate 结算日
        result.setClearDate(param.getClearDate());
        PlanTypeEnum entityTypeEnum = StringUtils.isNotBlank(param.getPlanType())? PlanTypeEnum.getByCode(param.getPlanType()): PlanTypeEnum.PLAN;
        result.setPlanType(entityTypeEnum);
        //ApproveStatus 审批状态
        result.setApproveStatus(ApproveStatusEnum.getEnum(param.getApproveStatus()));
        //分配状态
        result.setStatus(PlanAllocStatusEnum.getByCode(param.getStatus()));
        return result;
    }
    
    public static PlanAllocItem convertItemDomain(PlanAllocItemResultMap param) {
        PlanAllocItem result = new PlanAllocItem();

        //trialAmt 试算金额
//        result.setTrialAmt(Money.createWithCentNull(param.getTrialAmt()));
        //confirmAmt 确认金额
        result.setConfirmAmt(Money.createWithCentNull(param.getConfirmAmt()));
        //trialTaxAmt 试算增值税
//        result.setTrialTaxAmt(Money.createWithCentNull(param.getTrialTaxAmt()));
        //confirmTaxAmt 确认增值税
        result.setConfirmTaxAmt(Money.createWithCentNull(param.getConfirmTaxAmt()));
        //itemNo 事项编号
        result.setItemNo(param.getItemNo());
        VarChecker.checkNotEmpty(param.getTitleType(),"[{0}]科目未初始化",param.getPlanNo());
        TitleTypeEnum titleTypeEnum = TitleTypeEnum.getByCode(param.getTitleType());
        result.setAllocType(titleTypeEnum.getAllocType());
        //titleNo 科目编号
        result.setTitleNo(param.getTitleNo());
        result.setTitleName(param.getTitleName());
        //tax 是否计税
        result.setTax(param.getTax());
        //是否支付
        result.setPayTrial(param.getPayTrial());
        //sort 排序
        result.setOrder(param.getSort());
        result.setSettleAmt(Money.createWithCent(param.getSettleAmt()));
        return result;
    }

    public static List<PlanAllocDO> convertDos(List<PlanAlloc> allocs) {
        List<PlanAllocDO> result = Lists.newArrayList();
        allocs.forEach(alloc -> {
            result.add(convert2Do(alloc));
        });
        return result;
    }

    public static PlanAllocDO convert2Do(PlanAlloc param) {
        PlanAllocDO result = new PlanAllocDO();
        //allocAmt 本期分配总额
        if (param.getAllocAmt() != null) {
            result.setAllocAmt(param.getAllocAmt().getCent());
        }
        //planNo 计划编号
        result.setPlanNo(param.getPlanNo());
        //allocNo 分配编号
        result.setAllocNo(param.getAllocNo());
        //payDate 兑付日
        result.setPayDate(param.getPayDate());
        if(param.getPlanType()!=null) {
            result.setPlanType(param.getPlanType().getCode());
        }
        //clearDate 结算日
        result.setClearDate(param.getClearDate());
        if (param.getStatus() != null){
            // 分配状态
            result.setStatus(param.getStatus().getCode());
        }
        if (param.getApproveStatus() != null) {
            //批量插入后默认审批状态
            result.setApproveStatus(param.getApproveStatus().getCode());
        }
        return result;
    }
    
    public static List<PlanAllocItemDO> convertItemDos(List<PlanAlloc> allocs) {
        List<PlanAllocItemDO> result = Lists.newArrayList();
        for (PlanAlloc alloc : allocs) {
            for (PlanAllocItem allocItem : alloc.getPlanAllocItemList()) {
                result.add(convertItemDo(allocItem, alloc.getPlanNo(), alloc.getAllocNo()));
            }
        }
        return result;
    }

    public static PlanAllocItemDO convertItemDo(PlanAllocItem param, String planNo, String allocNo) {
        PlanAllocItemDO result = new PlanAllocItemDO();
        //trialAmt 试算金额
//        result.setTrialAmt(param.getTrialAmt().getCent());
        //confirmAmt 确认金额
        result.setConfirmAmt(param.getConfirmAmt().getCent());
        //trialTaxAmt 试算增值税
//        result.setTrialTaxAmt(param.getTrialTaxAmt().getCent());
        //confirmTaxAmt 确认增值税
        result.setConfirmTaxAmt(param.getConfirmTaxAmt().getCent());
        //itemNo 事项编号
        result.setItemNo(param.getItemNo());
        //planNo 计划编号
        result.setPlanNo(planNo);
        //allocNo 分配编号
        result.setAllocNo(allocNo);
        //titleNo 科目编号
        result.setTitleNo(param.getTitleNo());
        //tax 是否计税
        result.setTax(param.getTax());
        //sort 排序
        result.setSort(param.getOrder());
        result.setSettleAmt(param.getSettleAmt().getCent());
        return result;
    }

    public static List<PlanAllocItemDO> convertItemDomain2Dos(String planNo, String allocNo,
                                                              List<PlanAllocItem> planAllocItemList,
                                                              Map<String, Long> titleSettleAmtMap) {
        return planAllocItemList.stream().map(param->convertItemParam2Do(param,planNo,allocNo,titleSettleAmtMap)).collect(Collectors.toList());
    }

    private static PlanAllocItemDO convertItemParam2Do(PlanAllocItem param, String planNo, String allocNo,Map<String, Long> titleSettleAmtMap) {
        PlanAllocItemDO result = new PlanAllocItemDO();
        //trialAmt 试算金额
//        result.setTrialAmt(param.getTrialAmt().getCent());
        //confirmAmt 确认金额
        result.setConfirmAmt(param.getConfirmAmt().getCent());
        //trialTaxAmt 试算增值税
//        result.setTrialTaxAmt(param.getTrialTaxAmt().getCent());
        //confirmTaxAmt 确认增值税
        result.setConfirmTaxAmt(param.getConfirmTaxAmt().getCent());
        //itemNo 事项编号
        result.setItemNo(param.getItemNo());
        //planNo 计划编号
        result.setPlanNo(planNo);
        //allocNo 分配编号
        result.setAllocNo(allocNo);
        //titleNo 科目编号
        result.setTitleNo(param.getTitleNo());
        //tax 是否计税
        result.setTax(param.getTax());
        //sort 排序
        result.setSort(param.getOrder());
        result.setSettleAmt(titleSettleAmtMap.get(param.getTitleNo()));
        return result;
    }
    
    public static PlanAlloc convertParam2Domain(PlanAlloc alloc, PlanAllocEditParam param, List<PlanAcctTitle> planAcctTitles) {
        alloc.setPlanNo(param.getPlanNo());
        alloc.setAllocNo(param.getAllocNo());
        alloc.setStatus(PlanAllocStatusEnum.UNASSIGNE);
        if(param.getClearDate()!=null) {
            alloc.setClearDate(param.getClearDate());
        }
        if(param.getPayDate()!=null) {
            alloc.setPayDate(param.getPayDate());
        }
        if(param.getAllocAmt()!=null) {
            alloc.setAllocAmt(param.getAllocAmt());
        }
        Map<String , PlanAcctTitle> titleMap = planAcctTitles.stream().collect(Collectors.toMap(
                PlanAcctTitle::getTitleNo, item -> item));
        
        alloc.setPlanAllocItemList(convertItem2Domain(param.getPlanAllocItemParamList(), titleMap));
        return alloc;
    }
    
    
    public static List<PlanAllocItem>  convertItem2Domain(List<PlanAllocItemParam> params, Map<String , PlanAcctTitle> titleMap) {
        if (ToolUtil.isEmpty(params)){
            return null;
        }
        List<PlanAllocItem> items = new ArrayList<>(params.size());
        for (PlanAllocItemParam param : params){
            PlanAcctTitle title = titleMap.get(param.getTitleNo());
            PlanAllocItem item = new PlanAllocItem();
            if (StringUtils.isBlank(param.getItemNo())){
                item.setItemNo(SequenceUtil.genId(IdTypeEnum.ALLOCITM));
            }else{
                item.setItemNo(param.getItemNo());
            }
            item.setOrder(param.getOrder());
            item.setTitleNo(param.getTitleNo());
            if (title !=null){
                item.setTitleName(title.getTitleName());
                item.setAllocType(title.getTitleType().getAllocType());
            }
            item.setTax(param.getTax());
//            item.setTrialAmt(new Money(param.getTrialAmt()));
//            item.setTrialTaxAmt(new Money(param.getTrialTaxAmt()));
            item.setSettleAmt(new Money(param.getConfirmAmt()));
            item.setConfirmAmt(new Money(param.getConfirmAmt()));
            item.setConfirmTaxAmt(new Money(param.getConfirmTaxAmt()));
            items.add(item);
        }
        return items;
    }
}
