/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import java.util.List;
import java.util.Set;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.TaskGroupDO;
import com.abssqr.plat.common.facade.enums.TaskGroupTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.TaskGroup;
import com.abssqr.plat.common.model.utils.TextUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.convertor: TaskGroupConvert.java, v 0.1 2019-12-10 11:51 bangis.wangdf Exp $
 */
public class TaskGroupConvert {
    public static TaskGroupDO model2DO(TaskGroup taskGroup) {
        if (taskGroup != null) {
            TaskGroupDO taskGroupDO = new TaskGroupDO();
            taskGroupDO.setTasks(StringUtils.join(taskGroup.getTasks(), ","));
            taskGroupDO.setStatus(taskGroup.getStatus().getCode());
            taskGroupDO.setJobName(taskGroup.getJobName());
            taskGroupDO.setHostName(taskGroup.getHostName());
            taskGroupDO.setTaskType(taskGroup.getTaskType().getCode());
            taskGroupDO.setDisable(taskGroup.getDisable());
            return taskGroupDO;
        }
        return null;
    }

    public static List<TaskGroupDO> model2DOS(List<TaskGroup> taskGroups) {
        List<TaskGroupDO> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(taskGroups)) {
            taskGroups.forEach(taskGroup -> result.add(model2DO(taskGroup)));
        }
        return result;
    }

    public static List<TaskGroup> dos2Models(List<TaskGroupDO> taskGroupDOs) {
        List<TaskGroup> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(taskGroupDOs)) {
            taskGroupDOs.forEach(taskGroupDO -> result.add(do2Model(taskGroupDO)));
        }
        return result;
    }

    public static TaskGroup do2Model(TaskGroupDO taskGroupDO) {
        if (taskGroupDO != null) {
            TaskGroup taskGroup = new TaskGroup();
            taskGroup.setId(taskGroupDO.getId());
            taskGroup.setTaskType(TaskGroupTypeEnum.getByCode(taskGroupDO.getTaskType()));

            List<String> tasks = TextUtil.convertTxt2List(taskGroupDO.getTasks());
            List list = Lists.newArrayList();
            tasks.forEach(task -> list.add(
                taskGroup.getTaskType().getClazz() == Long.class ? Long.valueOf(task) : task
            ));

            taskGroup.setTasks(list);
            taskGroup.setStatus(TaskStatusEnum.getByCode(taskGroupDO.getStatus()));
            taskGroup.setJobName(taskGroupDO.getJobName());
            taskGroup.setHostName(taskGroupDO.getHostName());
            taskGroup.setDisable(taskGroupDO.getDisable());
            return taskGroup;
        }
        return null;

    }

}
