/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.convertor;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderBtn;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderVo;
import com.abssqr.plat.common.model.enums.WorkOrderBtnEnum;
import com.general.system.bpm.domain.param.ProcessStartDefaultRequest;
import com.general.system.bpm.entity.ProcessHistoricEntity;
import com.general.system.bpm.entity.ProcessTaskEntity;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.MessageUtil;
import com.general.system.tool.util.ToolUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * 流程模型转换
 * @author hanfei
 * @version $Id: WorkOrderConvert.java, v 0.1 2019-06-21 7:44 PM hanfei Exp $ 
 */
public class WorkOrderConvert {
    /**
     * 名称key
     */
    public static final String NAME_KEY     = "name";
    /**
     * 关联key
     */
    public static final String RELATION_KEY = "relation";
    
    public static List<WorkOrderVo> convertTask(List<ProcessTaskEntity> entities) {
        if (ToolUtil.isEmpty(entities)) {
            return null;
        }
        List<WorkOrderVo> orders = new ArrayList<>(entities.size());
        entities.forEach(o -> {
            WorkOrderVo workOrderVo = new WorkOrderVo();
            workOrderVo.setApplyUserName(o.getStartUserName());
            workOrderVo.setAssigneeName(o.getAssigneeName());
            workOrderVo.setWorkOrdCode(o.getProcessInstanceId());
            workOrderVo.setWorkOrdName(getWorkOrderName(o.getProcessDefinitionName(), o.getProcessInstanceName()));
            workOrderVo.setBusinessKey(o.getBusinessKey());
            workOrderVo.setBusinessType(ProcessStartDefaultRequest.getType(o.getProcessDefinitionKey()));
            workOrderVo.setGmtCreate(o.getCreateTime());
            workOrderVo.setCurrentApprovalNode(o.getName());
            workOrderVo.setCurrentApprovalStatus(ApproveStatusEnum.PENDING.getDesc());
            workOrderVo.setGmtEnd(null);
            orders.add(workOrderVo);
        });
        return orders;
    }
    
    public static List<WorkOrderVo> convertHis(List<ProcessHistoricEntity> entities) {
        if (ToolUtil.isEmpty(entities)) {
            return null;
        }
        List<WorkOrderVo> orders = new ArrayList<>(entities.size());
        entities.forEach(o -> {
            WorkOrderVo workOrderVo = new WorkOrderVo();
            workOrderVo.setApplyUserName(o.getStartUserName());
            workOrderVo.setAssigneeName(CollectionUtil.join(o.getNowActivityName(), ","));
            workOrderVo.setWorkOrdCode(o.getProcessInstanceId());
            workOrderVo.setWorkOrdName(getWorkOrderName(o.getProcessDefinitionName(), o.getName()));
            workOrderVo.setBusinessKey(o.getBusinessKey());
            workOrderVo.setBusinessType(ProcessStartDefaultRequest.getType(o.getProcessDefinitionKey()));
            workOrderVo.setGmtCreate(o.getStartTime());
            if (CollectionUtil.isEmpty(o.getNowActivityName())){
                workOrderVo.setCurrentApprovalNode(o.getEndActivityName());
            }else{
                workOrderVo.setCurrentApprovalNode(CollectionUtil.join(o.getNowActivityName(), ","));
            }
            workOrderVo.setCurrentApprovalStatus(o.getInstanceStatus());
            workOrderVo.setGmtEnd(o.getEndTime());
            orders.add(workOrderVo);
        });
        return orders;
    }
    
    public static String getWorkOrderName(String DefName, String name) {
        StringBuffer sb = new StringBuffer(DefName);
        sb.append("[");
        if (ToolUtil.isNotEmpty(name)) {
            sb.append(name);
        }else{
            sb.append("-");
        }
        sb.append("]");
        return sb.toString();
    }
    
    /**
     * 组装可操作的按钮
     * @param businessKey
     * @param businessType
     * @return
     */
    public static List<WorkOrderBtn> haveBtn(ProcessHistoricEntity entity, ProcessTaskEntity task,
                                             String businessKey, String businessType, String userCode) {
        List<WorkOrderBtn> btns = new ArrayList<>();
        // 已经结束
        if (entity == null || ToolUtil.isNotEmpty(entity.getEndTime())) {
            return btns;
        }
        // 申请人 具有撤销按钮
        if (StrUtil.equals(entity.getStartUserId(), userCode)) {
            btns.add(installBtn(WorkOrderBtnEnum.revoke, businessKey, businessType));
        }
        
        // 当前办理人 具有 通过、拒绝、-- 退回
        if (task != null) {
            btns.add(installBtn(WorkOrderBtnEnum.complete, businessKey, businessType));
            // 暂时取消 退回操作
            // btns.add(installBtn(WorkOrderBtnEnum.rollback, businessKey,  businessType));
            btns.add(installBtn(WorkOrderBtnEnum.refuse, businessKey, businessType));
        }
        return btns;
    }
    
    public static WorkOrderBtn installBtn(WorkOrderBtnEnum btnEnum, String businessKey, String businessType) {
        WorkOrderBtn btn = new WorkOrderBtn();
        btn.setBtnName(btnEnum.getDesc());
        btn.setBtnUrl(MessageUtil.formatMsg("/workOrder/operate/{0}.json?businessKey={1}&businessType={2}",
                btnEnum.getCode(), businessKey, businessType));
        btn.setBusinessKey(businessKey);
        btn.setBusinessType(businessType);
        return btn;
    }
    
}
