/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.loanTrans;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.base.OrgDistPftMoney;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 资产分析变更明细
 *
 * @author guozhenhua
 * @version LoanInstalTransDetail.java, v 0.1 2017-12-12 下午8:20 guozhenhua
 * Exp $
 */
public class LoanInstalTransDetail implements ITransDtl {
    
    /**
     * 贷款资产分期
     */
    private LoanInstalDomain loanInstal;
    
    /**
     * 贷款资产分期流水
     */
    private LoanInstalLog   activeLog;
    /**
     * 贷款资产交易流水
     */
    private LoanTransDetail loanTransDetail;
    /**
     * 机构资产交易明细
     */
    private List<LoanAstTransDetail> loanAstTransDetails = Lists.newArrayList();
    /**
     * 机构资产收益交易明细
     */
    private List<LoanAstProfitTransDetail> loanAstProfitTransDetails = Lists.newArrayList();
    
    /**
     * 判断是否当天初笔交易
     */
    private boolean isFirstTransOfDay = false;
    /**
     *
     */
    private boolean isUpdate          = false;
    
    private LoanInstalTransDetail() {
    
    }
    
    /**
     * 构建一个贷款资产
     *
     * @param loanInstal
     * @param loanTransDetail
     * @return
     */
    public static LoanInstalTransDetail createLoanInstalTxnDtl(LoanInstalDomain loanInstal,
                                                               LoanTransDetail loanTransDetail) {
        
        // 1. 构造资产交易明细
        LoanInstalTransDetail instalTransDetail = new LoanInstalTransDetail();
        
        // 2. 构造交易流水
        LoanInstalLog log = buildLog(loanTransDetail.getLoan(), loanInstal, loanTransDetail.getActiveLog());
        instalTransDetail.loanInstal = loanInstal;
        instalTransDetail.activeLog = log;
        
        instalTransDetail.loanTransDetail = loanTransDetail;
        
        // 3. 处理初笔交易
        if (DateTimeUtil.isBeforeOnDate(loanInstal.getLastUpdateDate(), log.getAccountDate())) {
            instalTransDetail.isFirstTransOfDay = true;
            loanInstal.setLastUpdateDate(log.getAccountDate());
        }
        
        return instalTransDetail;
    }
    
    private static LoanInstalLog buildLog(Loan loan, LoanInstal loanInstal, LoanLog loanLog) {
        LoanInstalLog log = new LoanInstalLog();
        log.setUserId(loanInstal.getUserId());
        // 2.1. 资产信息
        log.setAstNo(loanInstal.getAstNo());
        log.setOutAstNo(loanInstal.getOutAstNo());
        log.setLenderOrgCode(loanInstal.getAstOrg().getAstOwnership().getLenderOrgCode());
        log.setManagerOrgCode(loanInstal.getAstOrg().getManagerOrgCode());
        log.setSponserOrgCode(loanInstal.getAstOrg().getSponsorOrgCode());
        log.setOwnOrg(loanInstal.getAstOrg().getAstOwnership().getOwnOrg());
        log.setOwnPack(loanInstal.getOwnPack());
        log.setProdNo(loanInstal.getProdNo());
        log.setOutProdNo(loanInstal.getOutProdNo());
        log.setContractNo(loanInstal.getContNo());
        log.setTermNo(loanInstal.getTermNo());
        
        // 2.2. 交易信息
        log.setOutBizNo(loanLog.getOutBizNo());
        log.setBizNo(loanLog.getBizNo());
        log.setEventCode(loanLog.getEventCode());
        log.setSeqNo(SequenceUtil.genSeqByLoan(loan, log.getEventCode().getSceneCode()));
        log.setTransTime(loanLog.getTransTime());
        log.setAccountDate(loanLog.getAccountDate());
        log.setExecTms(loanLog.getExecTms());
        
        // 2.3. 交易前数据备份
        log.setBfStatus(loanInstal.getStatus());
        log.setBfOvdDays(loanInstal.getOvdDays());
        
        return log;
    }
    
    public void clearLoanAst() {
        clearLoanAst(this.activeLog.getAccountDate());
    }
    
    public void clearLoanAst(Date clearDate) {
        Map<String, LoanAstTransDetail> detailMap = Maps.newHashMap();
        for (LoanAstTransDetail loanAstTransDetail : this.loanAstTransDetails) {
            detailMap.put(loanAstTransDetail.getLoanAst().getLoanAstNo(), loanAstTransDetail);
        }
        
        for (LoanAst loanAst : loanInstal.getLoanAsts()) {
            if (loanAst.getAstStatus() != LoanAstStatusEnum.NORMAL) {
                continue;
            }
            LoanAstTransDetail loanAstTransDetail = detailMap.get(loanAst.getLoanAstNo());
            
            if (null == loanAstTransDetail) {
                loanAstTransDetail = LoanAstTransDetail.createTransDtl(loanAst, this, activeLog.getEventCode());
            }
            
            loanAstTransDetail.clearLoanAst(clearDate, this.activeLog.getEventCode(), LoanAstStatusEnum.CLEAR);
        }
    }


    /**
     * 处理发生额
     *
     * @param transAmt
     */
    public void processTransAmt(DebtMoney transAmt) {
        processTransAmt(transAmt, false);
    }

    
    /**
     * 处理发生额
     *
     * @param transAmt
     */
    public void processTransAmt(DebtMoney transAmt, boolean isForce) {
        if (!isForce && transAmt.getSum().isZero()) {
            return;
        }
        isUpdate = true;
        DebtMoney curBal = this.loanInstal.getBal();
        DebtMoney newBal = new DebtMoney();
        
        this.activeLog.getAmt().addAll(transAmt);

        // 1. 根据事件码，设定符号
        int sign = this.activeLog.getEventCode().getAssetFlow().getSign();
        
        // 2. 计算交易后余额
        for (DebtMoneyFieldEnum debtMoneyField : DebtMoneyFieldEnum.values()) {
            Money curBalField = curBal.getByFieldType(debtMoneyField);
            Money transAmtField = transAmt.getByFieldType(debtMoneyField);
            Money newBalField = curBalField.add(transAmtField.multiply(sign));
            newBal.setByFieldType(debtMoneyField, newBalField);
        }
        
        if (this.activeLog.getEventCode().getSceneCode() == TransCodeEnum.REPO) {
            this.loanInstal.getPaidAmt().addAll(transAmt);
        } else if (this.activeLog.getEventCode().getSceneCode() == TransCodeEnum.EXMPT) {
            this.loanInstal.getRmtAmt().addAll(transAmt);
        } else if (this.activeLog.getEventCode() == EventCodeEnum.ADJ_TERM_OUT) {
            this.loanInstal.getCost().subtractFrom(transAmt.getByFieldType(DebtMoneyFieldEnum.PRIN));
        } else if (this.activeLog.getEventCode() == EventCodeEnum.ADJ_TERM_IN) {
            this.loanInstal.getCost().addTo(transAmt.getByFieldType(DebtMoneyFieldEnum.PRIN));
        }
        // 处理发生额
        this.loanTransDetail
                .processTransAmtFromInstal(transAmt, this.activeLog.getBfStatus(), this.activeLog.getEventCode());
        // 3. 回填余额
        this.loanInstal.setBal(newBal);
    }
    
    /**
     * @param amt
     */
    public void processPaidAmtOnly(DebtMoney amt) {
        if (amt.getSum().isZero()) {
            return;
        }
        isUpdate = true;
        
        if (this.activeLog.getEventCode().getSceneCode() != TransCodeEnum.REPO) {
            return;
        }
        
        this.loanInstal.getPaidAmt().addAll(amt);
        this.loanTransDetail.getLoan().getPaid().addAll(amt);
    }
    
    public void processPaidFundAmt(DebtMoney amt) {
        if (amt.getSum().isZero()) {
            return;
        }
        isUpdate = true;
        
        if (this.activeLog.getEventCode().getSceneCode() != TransCodeEnum.REPO) {
            return;
        }
        
        this.loanInstal.getPaidFundAmt().addAll(amt);
        this.getActiveLog().getPaidFundAmt().addAll(amt);
    }
    
    @Override
    public void processBeforeUpdate() {
        // 将交易后信息填入交易流水
        activeLog.setAfBal(loanInstal.getBal().cloneAll());
        activeLog.setAfOvdDays(loanInstal.getOvdDays());
        activeLog.setAfStatus(loanInstal.getStatus());
        
        for (LoanAstTransDetail loanAstTransDetail : this.getLoanAstTransDetails()) {
            loanAstTransDetail.processBeforeUpdate();
        }
        
        for (LoanAstProfitTransDetail loanAstProfitTransDetail : this.getLoanAstProfitTransDetails()) {
            loanAstProfitTransDetail.processBeforeUpdate();
        }
    }
    
    /**
     * 设置逾期天数
     *
     * @param ovdDays
     */
    public void setOvdDays(long ovdDays) {
        isUpdate = true;
        loanInstal.setOvdDays((int) ovdDays);
    }
    
    /**
     * @param amt
     * @param isUpdate
     */
    public void addTransAmtOnly(DebtMoney amt, boolean isUpdate) {
        if (amt.getSum().isZero()) {
            return;
        }
        this.activeLog.getAmt().addAll(amt);
        this.loanTransDetail.addTransAmtOnlyFromInstal(amt, this.activeLog.getBfStatus(), isUpdate);
        this.isUpdate = this.isUpdate || isUpdate;
    }
    
    /**
     * 分期转让设置所属机构
     */
    public void instalTransfer(Money cost, Money prinAmt) {
        this.loanInstal.setCost(this.loanInstal.getCost().add(cost));
        
        this.loanTransDetail.getLoan().getAstOrg().getAstOwnership().setOwnStatus(OwnStatusEnum.PARTIAL_TRANS_OUT);
        DebtMoney amt = new DebtMoney();
        amt.setByFieldType(DebtMoneyFieldEnum.PRIN, prinAmt);
        this.addTransAmtOnly(amt, true);
        this.isUpdate = true;
    }
    
    /**
     * 分期回购
     * @param bal
     */
    public void instalBuyback( DebtMoney bal) {
        // 记状态
        this.loanTransDetail.getLoan().getAstOrg().getAstOwnership().setOwnStatus(OwnStatusEnum.PARTIAL_TRANS_OUT);
        // 记录流水发生额
        this.addTransAmtOnly(bal.cloneAll(), true);
        this.isUpdate = true;
    }
    
    /**
     * 设置清算金额
     *
     * @param clrgAmt
     */
    public void setClrgAmt(Money clrgAmt) {
        this.loanTransDetail.changeClrgAmt(clrgAmt);
        this.activeLog.setClrgAmt(clrgAmt);
    }
    
    public void addHangIntrAmt(Money hangIntrAmt) {
        this.loanInstal.setHangIntrBal(hangIntrAmt);
        this.activeLog.setHangIntrAmt(hangIntrAmt);
        isUpdate = true;
    }
    
    /**
     *
     */
    public void clearHangIntrAmt() {
        this.activeLog.setHangIntrAmt(this.loanInstal.getHangIntrBal());
        this.activeLog.setPaidHangIntrFundAmt(new Money());
        this.loanInstal.setHangIntrBal(new Money());
        this.loanInstal.setPaidHangIntrAmt(new Money());
        isUpdate = true;
    }
    
    public void dealHangIntrAmt(Money hangIntrAmt, Money paidHangIntrFundAmt) {
        this.activeLog.setHangIntrAmt(hangIntrAmt);
        this.activeLog.setPaidHangIntrFundAmt(paidHangIntrFundAmt);
        this.loanInstal.setHangIntrBal(this.loanInstal.getHangIntrBal().subtract(hangIntrAmt));
        this.loanInstal.setPaidHangIntrAmt(this.loanInstal.getPaidHangIntrAmt().add(hangIntrAmt));
        isUpdate = true;
    }
    
    public void dealHangIntrAmt(Money hangIntrAmt) {
        this.activeLog.setHangIntrAmt(hangIntrAmt);
        this.loanInstal.setHangIntrBal(this.loanInstal.getHangIntrBal().subtract(hangIntrAmt));
        isUpdate = true;
    }
    
    public void movePaidHangIntrAmt(Money amt) {
        this.activeLog.getPaidHangIntrFundAmt().subtractFrom(amt);
        isUpdate = true;
    }
    
    /**
     * 结清分期
     *
     * @param clearDate
     */
    public void setClear(Date clearDate) {
        this.isUpdate = true;
        this.loanInstal.getAstPeriod().setClearDate(clearDate);
        this.loanInstal.setOvdDays(0);
        this.loanInstal.setStatus(LoanStatusEnum.CLEAR);
        
        this.clearLoanAst(clearDate);
    }
    
    /**
     * 设置资产状态
     *
     * @param status
     */
    public void changeStatus(LoanStatusEnum status) {
        this.isUpdate = true;
        if (this.loanInstal.getStatus() == LoanStatusEnum.NORMAL && status == LoanStatusEnum.OVERDUE) {
            // 转逾期
            loanTransDetail.changeStatusAmtForTurnOdue(this.loanInstal.getBal());
        }
        this.loanInstal.setStatus(status);
    }
    
    /**
     * 处理机构分润
     *
     * @param hangClrAmt 清算
     * @param fsApporAmt 摊出
     */
    public void calcOrgFeeSplit(Money hangClrAmt, Money fsApporAmt) {
        this.activeLog.setPaidHangIntrFundAmt(hangClrAmt);
        Money intAndFeeAmt = this.getActiveLog().getAmt().getAllIntAndFee();
        Money paidFunIntAndFeedAmt = this.getActiveLog().getPaidFundAmt().getAllIntAndFee();
        //机构分润 = 利息收入-利息实付
        Money fsAmt = intAndFeeAmt.subtract(paidFunIntAndFeedAmt);
        //实付 = 机构分润 - 摊出
        Money fsPaidAmt = fsAmt.subtract(fsApporAmt);
        this.getActiveLog().setOrgDistPftAmt(new OrgDistPftMoney(fsAmt, fsApporAmt, fsPaidAmt));
    }
    
    public boolean isUpdate() {
        return isUpdate;
    }
    
    public void setUpdate() {
        isUpdate = true;
        this.loanTransDetail.setUpdate(true);
    }
    
    public LoanInstalDomain getLoanInstal() {
        return loanInstal;
    }
    
    public LoanInstalLog getActiveLog() {
        return activeLog;
    }
    
    public List<LoanAstTransDetail> getLoanAstTransDetails() {
        return loanAstTransDetails;
    }
    
    public List<LoanAstProfitTransDetail> getLoanAstProfitTransDetails() {
        return loanAstProfitTransDetails;
    }
    
    public LoanTransDetail getLoanTransDetail() {
        return loanTransDetail;
    }
    
    protected void setLoanTransDetail(LoanTransDetail loanTransDetail) {
        this.loanTransDetail = loanTransDetail;
    }
    
}