/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.loanTrans;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;

import java.util.Date;

/**
 * 资产流水
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.domain.loanTrans: LoanLog.java, v 0.1 2017-12-21 上午1:53 guozhenhua Exp $
 */
public class LoanLog extends BaseEntity {
    
    /**
     * execTms 执行时间戳.
     */
    private String           execTms;
    /**
     * id 自增ID.
     */
    private Long             id;
    /**
     * astNo 资产编号.
     */
    private String           astNo;
    /**
     * 用户ID
     */
    private String           userId;
    /**
     * outAstNo 外部资产编号.
     */
    private String           outAstNo;
    /**
     * outBizNo 外部业务流水号.
     */
    private String           outBizNo;
    /**
     * bizNo 内部业务流水号.
     */
    private String           bizNo;
    /**
     * seqNo 日志流水号.
     */
    private String           seqNo;
    /**
     * lenderOrgCode 原始权益机构（计划）.
     */
    private String           lenderOrgCode;
    /**
     * managerOrgCode 资产管理服务机构.
     */
    private String           managerOrgCode;
    /**
     * sponserOrgCode 业务牵头机构.
     */
    private String           sponserOrgCode;
    /**
     * ownOrg 所属机构（计划）.
     */
    private String           ownOrg;
    /**
     * ownPack 所属资产包.
     */
    private String           ownPack;
    /**
     * prodNo 产品码.
     */
    private String           prodNo;
    /**
     * outProdNo 外部产品码.
     */
    private String           outProdNo;
    /**
     * contractNo 合同编号.
     */
    private String           contractNo;
    /**
     * bfRiskCl 客户五级分类.
     */
    private AstRiskClassEnum bfRiskCl;
    /**
     * afRiskCl 交易后五级分类.
     */
    private AstRiskClassEnum afRiskCl;
    /**
     * bfStatus 原状态.
     */
    private LoanStatusEnum   bfStatus;
    /**
     * afStatus 交易后状态.
     */
    private LoanStatusEnum   afStatus;
    /**
     * eventCode 交易码.
     */
    private EventCodeEnum    eventCode;
    /**
     * bfOvdDays 原逾期天数.
     */
    private Integer          bfOvdDays;
    /**
     * afOvdDays 交易后逾期天数.
     */
    private Integer          afOvdDays;
    /**
     * afTermCnt 交易后期数.
     */
    private Integer          afTermCnt;
    /**
     * bfTermCnt 原期数.
     */
    private Integer          bfTermCnt;
    /**
     * 资产持有状态
     */
    private OwnStatusEnum    afOwnStatus;
    /**
     * 资产持有状态
     */
    private OwnStatusEnum    bfOwnStatus;
    /**
     * bfEndDate 原结束日期(YYYYMMDD).
     */
    private Date             bfEndDate;
    /**
     * afEndDate 交易后结束日期(YYYYMMDD).
     */
    private Date             afEndDate;
    /**
     * transTime 交易时间.
     */
    private Date             transTime;
    /**
     * accountDate 会计日期.
     */
    private Date             accountDate;
    /**
     * clrgAmt 清算金额.
     */
    private Money            clrgAmt = new Money();
    /**
     * 交易前余额
     */
    private DebtMoney        bfBal   = new DebtMoney();
    /**
     * 发生额
     */
    private DebtMoney        amt     = new DebtMoney();
    /**
     * 交易后余额
     */
    private DebtMoney        afBal   = new DebtMoney();
    /**
     * 交易前核销状态
     */
    private YNEnum           bfWtfStatus;
    /**
     * 交易后核销状态
     */
    private YNEnum           afWtfStatus;
    /**
     * afCapitalNo 交易后资金编号.
     */
    private String afCapitalNo;
    /**
     * bfCapitalNo 交易前资金编号.
     */
    private String bfCapitalNo;

    /**
     * bfCapitalNo 备注.
     */
    private String memo;
    
    public String getExecTms() {
        return execTms;
    }
    
    public void setExecTms(String execTms) {
        this.execTms = execTms;
    }
    
    public Long getId() {
        return id;
    }
    
    public void setId(Long id) {
        this.id = id;
    }
    
    public String getAstNo() {
        return astNo;
    }
    
    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }
    
    public String getUserId() {
        return userId;
    }
    
    public void setUserId(String userId) {
        this.userId = userId;
    }
    
    public String getOutAstNo() {
        return outAstNo;
    }
    
    public void setOutAstNo(String outAstNo) {
        this.outAstNo = outAstNo;
    }
    
    public String getOutBizNo() {
        return outBizNo;
    }
    
    public void setOutBizNo(String outBizNo) {
        this.outBizNo = outBizNo;
    }
    
    public String getBizNo() {
        return bizNo;
    }
    
    public void setBizNo(String bizNo) {
        this.bizNo = bizNo;
    }
    
    public String getSeqNo() {
        return seqNo;
    }
    
    public void setSeqNo(String seqNo) {
        this.seqNo = seqNo;
    }
    
    public String getLenderOrgCode() {
        return lenderOrgCode;
    }
    
    public void setLenderOrgCode(String lenderOrgCode) {
        this.lenderOrgCode = lenderOrgCode;
    }
    
    public String getManagerOrgCode() {
        return managerOrgCode;
    }
    
    public void setManagerOrgCode(String managerOrgCode) {
        this.managerOrgCode = managerOrgCode;
    }
    
    public String getSponserOrgCode() {
        return sponserOrgCode;
    }
    
    public void setSponserOrgCode(String sponserOrgCode) {
        this.sponserOrgCode = sponserOrgCode;
    }
    
    public String getOwnOrg() {
        return ownOrg;
    }
    
    public void setOwnOrg(String ownOrg) {
        this.ownOrg = ownOrg;
    }
    
    public String getOwnPack() {
        return ownPack;
    }
    
    public void setOwnPack(String ownPack) {
        this.ownPack = ownPack;
    }
    
    public String getProdNo() {
        return prodNo;
    }
    
    public void setProdNo(String prodNo) {
        this.prodNo = prodNo;
    }
    
    public String getOutProdNo() {
        return outProdNo;
    }
    
    public void setOutProdNo(String outProdNo) {
        this.outProdNo = outProdNo;
    }
    
    public String getContractNo() {
        return contractNo;
    }
    
    public void setContractNo(String contractNo) {
        this.contractNo = contractNo;
    }
    
    public AstRiskClassEnum getBfRiskCl() {
        return bfRiskCl;
    }
    
    public void setBfRiskCl(AstRiskClassEnum bfRiskCl) {
        this.bfRiskCl = bfRiskCl;
    }
    
    public AstRiskClassEnum getAfRiskCl() {
        return afRiskCl;
    }
    
    public void setAfRiskCl(AstRiskClassEnum afRiskCl) {
        this.afRiskCl = afRiskCl;
    }
    
    public LoanStatusEnum getBfStatus() {
        return bfStatus;
    }
    
    public void setBfStatus(LoanStatusEnum bfStatus) {
        this.bfStatus = bfStatus;
    }
    
    public LoanStatusEnum getAfStatus() {
        return afStatus;
    }
    
    public void setAfStatus(LoanStatusEnum afStatus) {
        this.afStatus = afStatus;
    }
    
    public EventCodeEnum getEventCode() {
        return eventCode;
    }
    
    public void setEventCode(EventCodeEnum eventCode) {
        this.eventCode = eventCode;
    }
    
    public Integer getBfOvdDays() {
        return bfOvdDays;
    }
    
    public void setBfOvdDays(Integer bfOvdDays) {
        this.bfOvdDays = bfOvdDays;
    }
    
    public Integer getAfOvdDays() {
        return afOvdDays;
    }
    
    public void setAfOvdDays(Integer afOvdDays) {
        this.afOvdDays = afOvdDays;
    }
    
    public Integer getAfTermCnt() {
        return afTermCnt;
    }
    
    public void setAfTermCnt(Integer afTermCnt) {
        this.afTermCnt = afTermCnt;
    }
    
    public Integer getBfTermCnt() {
        return bfTermCnt;
    }
    
    public void setBfTermCnt(Integer bfTermCnt) {
        this.bfTermCnt = bfTermCnt;
    }
    
    public Date getBfEndDate() {
        return bfEndDate;
    }
    
    public void setBfEndDate(Date bfEndDate) {
        this.bfEndDate = bfEndDate;
    }
    
    public Date getAfEndDate() {
        return afEndDate;
    }
    
    public void setAfEndDate(Date afEndDate) {
        this.afEndDate = afEndDate;
    }
    
    public Date getTransTime() {
        return transTime;
    }
    
    public void setTransTime(Date transTime) {
        this.transTime = transTime;
    }
    
    public Date getAccountDate() {
        return accountDate;
    }
    
    public void setAccountDate(Date accountDate) {
        this.accountDate = accountDate;
    }
    
    public Money getClrgAmt() {
        return clrgAmt;
    }
    
    public void setClrgAmt(Money clrgAmt) {
        this.clrgAmt = clrgAmt;
    }
    
    public DebtMoney getAmt() {
        return amt;
    }
    
    public void setAmt(DebtMoney amt) {
        this.amt = amt;
    }
    
    public DebtMoney getAfBal() {
        return afBal;
    }
    
    public void setAfBal(DebtMoney afBal) {
        this.afBal = afBal;
    }
    
    public OwnStatusEnum getAfOwnStatus() {
        return afOwnStatus;
    }
    
    public void setAfOwnStatus(OwnStatusEnum afOwnStatus) {
        this.afOwnStatus = afOwnStatus;
    }
    
    public OwnStatusEnum getBfOwnStatus() {
        return bfOwnStatus;
    }
    
    public void setBfOwnStatus(OwnStatusEnum bfOwnStatus) {
        this.bfOwnStatus = bfOwnStatus;
    }
    
    public DebtMoney getBfBal() {
        return bfBal;
    }
    
    public void setBfBal(DebtMoney bfBal) {
        this.bfBal = bfBal;
    }
    
    public YNEnum getBfWtfStatus() {
        return bfWtfStatus;
    }
    
    public void setBfWtfStatus(YNEnum bfWtfStatus) {
        this.bfWtfStatus = bfWtfStatus;
    }
    
    public YNEnum getAfWtfStatus() {
        return afWtfStatus;
    }
    
    public void setAfWtfStatus(YNEnum afWtfStatus) {
        this.afWtfStatus = afWtfStatus;
    }

    public String getAfCapitalNo() {
        return afCapitalNo;
    }

    public void setAfCapitalNo(String afCapitalNo) {
        this.afCapitalNo = afCapitalNo;
    }

    public String getBfCapitalNo() {
        return bfCapitalNo;
    }

    public void setBfCapitalNo(String bfCapitalNo) {
        this.bfCapitalNo = bfCapitalNo;
    }

    public String getMemo() {
        return memo;
    }

    public void setMemo(String memo) {
        this.memo = memo;
    }

    public String forLog() {
        return "LoanLog{" +
            ", astNo='" + astNo + '\'' +
            ", outAstNo='" + outAstNo + '\'' +
            ", outBizNo='" + outBizNo + '\'' +
            ", bizNo='" + bizNo + '\'' +
            ", seqNo='" + seqNo + '\'' +
            ", lenderOrgCode='" + lenderOrgCode + '\'' +
            ", managerOrgCode='" + managerOrgCode + '\'' +
            ", sponserOrgCode='" + sponserOrgCode + '\'' +
            ", ownOrg='" + ownOrg + '\'' +
            ", ownPack='" + ownPack + '\'' +
            ", eventCode=" + eventCode +
            ", transTime=" + transTime +
            ", accountDate=" + accountDate +
            ", clrgAmt=" + clrgAmt +
            ", bfBal=" + bfBal +
            ", amt=" + amt +
            ", afBal=" + afBal +
            '}';
    }
}
