/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.domain.task;

import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.general.system.common.model.Money;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 转让任务
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.tr: TrTask.java, v 0.1 2018-01-06 00:58 hongwei.wang Exp $
 */
public class TrTask extends DispatcherNode {
    /**
     * 任务ID
     */
    private Long             id;
    /**
     * tfrDt 转让日期.
     */
    private Date             tfrDt;
    /**
     * astDueDt 资产到期日.
     */
    private Date             astDueDt;
    /**
     * tfrAmt 资产转让最终值.
     */
    private Money            tfrAmt         = new Money(0);
    /**
     * tfrPrinBal 转让本金余额.
     */
    private Money      tfrPrinBal     = new Money(0);
    /**
     * seqNo 流水号.
     */
    private String     seqNo;
    /**
     * billId  账单id.
     */
    private String     astNo;
    /**
     * tfrOut 资产转出机构.
     */
    private String     tfrOut;
    /**
     * tfrPln 转让计划.
     */
    private String     tfrPack;
    /**
     * batchNo 批次号-防止异常.
     */
    private String     batchNo;
    /**
     * termNo termNo.
     */
    private Integer    termNo;
    /**
     * tfrIntrBal 利息余额.
     */
    private Money            tfrIntrBal     = new Money();
    /**
     * tfrPenalFeeBal 其他费用余额.
     */
    private Money            tfrPenalFeeBal = new Money();
    /**
     * riskCl 五级分类号.
     */
    private String               riskCl;
    /**
     * intrRate 利息定价比例.
     */
    private BigDecimal           intrRate = BigDecimal.ONE;
    /**
     * prinRate 本金定价比例.
     */
    private BigDecimal           prinRate = BigDecimal.ONE;
    /**
     * penalFeeRate 罚息费用定价比例.
     */
    private BigDecimal           penalFeeRate = BigDecimal.ONE;
    /**
     * packType 资产包类型 (BUY_BACK_BIZ 回购交易 , AST_TRF_BIZ 转让交易) .
     */
    private AstPackTradeTypeEnum packType;
    
    /**
     * 获取原余额总额
     * @return
     */
    public Money getAllBal(){
        return tfrPrinBal.add(tfrIntrBal).addTo(tfrPenalFeeBal);
    }
    
    /**
     * 获取计价总额
     * @return
     */
    public Money getAllPrice(){
        // 计算 定价本金
        Money prinPrice = tfrPrinBal.multiply(prinRate);
        // 计算 定价利息
        Money intrPrice = tfrIntrBal.multiply(intrRate);
        // 计算 定价罚息费用
        Money penalFeePrice = tfrPenalFeeBal.multiply(penalFeeRate);
        return prinPrice.add(intrPrice).addTo(penalFeePrice);
    }
    
    public Date getTfrDt() {
        return tfrDt;
    }

    public void setTfrDt(Date tfrDt) {
        this.tfrDt = tfrDt;
    }

    public Date getAstDueDt() {
        return astDueDt;
    }

    public void setAstDueDt(Date astDueDt) {
        this.astDueDt = astDueDt;
    }

    public Money getTfrAmt() {
        return tfrAmt;
    }

    public void setTfrAmt(Money tfrAmt) {
        this.tfrAmt = tfrAmt;
    }

    public Money getTfrPrinBal() {
        return tfrPrinBal;
    }

    public void setTfrPrinBal(Money tfrPrinBal) {
        this.tfrPrinBal = tfrPrinBal;
    }

    public String getSeqNo() {
        return seqNo;
    }

    public void setSeqNo(String seqNo) {
        this.seqNo = seqNo;
    }

    public String getAstNo() {
        return astNo;
    }

    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }

    public String getTfrOut() {
        return tfrOut;
    }

    public void setTfrOut(String tfrOut) {
        this.tfrOut = tfrOut;
    }

    public String getTfrPack() {
        return tfrPack;
    }

    public void setTfrPack(String tfrPack) {
        this.tfrPack = tfrPack;
    }

    public String getBatchNo() {
        return batchNo;
    }

    public void setBatchNo(String batchNo) {
        this.batchNo = batchNo;
    }

    public Integer getTermNo() {
        return termNo;
    }

    public void setTermNo(Integer termNo) {
        this.termNo = termNo;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Money getTfrIntrBal() {
        return tfrIntrBal;
    }

    public void setTfrIntrBal(Money tfrIntrBal) {
        this.tfrIntrBal = tfrIntrBal;
    }

    public Money getTfrPenalFeeBal() {
        return tfrPenalFeeBal;
    }

    public void setTfrPenalFeeBal(Money tfrPenalFeeBal) {
        this.tfrPenalFeeBal = tfrPenalFeeBal;
    }

    public String getRiskCl() {
        return riskCl;
    }

    public void setRiskCl(String riskCl) {
        this.riskCl = riskCl;
    }

    public BigDecimal getIntrRate() {
        return intrRate;
    }

    public void setIntrRate(BigDecimal intrRate) {
        this.intrRate = intrRate;
    }

    public BigDecimal getPrinRate() {
        return prinRate;
    }

    public void setPrinRate(BigDecimal prinRate) {
        this.prinRate = prinRate;
    }

    public BigDecimal getPenalFeeRate() {
        return penalFeeRate;
    }

    public void setPenalFeeRate(BigDecimal penalFeeRate) {
        this.penalFeeRate = penalFeeRate;
    }

    public AstPackTradeTypeEnum getPackType() {
        return packType;
    }

    public void setPackType(AstPackTradeTypeEnum packType) {
        this.packType = packType;
    }


    public Money getTotalBal(){
        return tfrPrinBal.add(tfrIntrBal).add(tfrPenalFeeBal);
    }
}
