/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.ast.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.AstPackDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPackDO;
import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.model.convertor.PacKConvert;
import com.abssqr.plat.common.model.convertor.LoanAstBuyBackPackConvert;
import com.abssqr.plat.common.model.domain.ast.Pack;
import com.abssqr.plat.common.model.domain.ast.LoanAstBuyBackPack;
import com.abssqr.plat.common.model.repo.ast.PackRepository;
import com.abssqr.plat.common.model.repo.plan.PlanTransferRuleRepo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.repo.pack.impl: PackRepositoryImpl.java, v 0.1 2018-02-26 下午4:20 guozhenhua
 * Exp $
 */
@Component
public class PackRepositoryImpl implements PackRepository {

    @Autowired
    private AstPackDAO           astPackDAO;
    @Autowired
    private PlanTransferRuleRepo planTransferRuleRepo;

    @Override
    public int create(Pack pack) {
        return astPackDAO.insert(PacKConvert.domain2DO(pack));
    }

    @Override
    public int create(LoanAstBuyBackPack pack) {
        return astPackDAO.insert(LoanAstBuyBackPackConvert.domain2DO(pack));
    }

    @Override
    public int createBatch(List<Pack> packs) {
        if (CollectionUtils.isNotEmpty(packs)) {
            List<AstPackDO> collect = packs.stream().map(pack -> PacKConvert.domain2DO(pack)).collect(
                    Collectors.toList());
            return astPackDAO.insertBatch(collect);
        }
        return 0;
    }


    @Override
    public void updatePackStatus(List<Pack> packs, AstPackStatusEnum packStatus, AstPackStatusEnum oldPackStatus) {

        astPackDAO.updatePackStatus(packStatus.getCode(), oldPackStatus.getCode(),
                packs.stream().map(Pack::getPackCode).collect(Collectors.toList()));
    }

    @Override
    public List<AstPackDO> getByPlanNo(String planNo, AstPackTradeTypeEnum tradeTypeEnum) {
        return astPackDAO.getByPlanNo(planNo, tradeTypeEnum.getCode());
    }

    @Override
    public AstPackDO getLastDayPackByPlanNo(String planNo, Date scenDate, AstPackTradeTypeEnum tradeTypeEnum) {
        return astPackDAO.getLastDayPackByPlanNo(scenDate, planNo, tradeTypeEnum.getCode());
    }

    @Override
    public List<AstPackDO> getByManagerOrgCode(String managerOrgCode, Date accountDate, AstPackStatusEnum status, AstPackTradeTypeEnum tradeTypeEnum) {

        return astPackDAO.getByManagerOrg(accountDate, Optional.ofNullable(status).map(AstPackStatusEnum::getCode).orElse(null), tradeTypeEnum.getCode(), managerOrgCode);

    }

    @Override
    public AstPackDO getByPackNo(String packCode) {
        return astPackDAO.getByPackCode(packCode);
    }

    @Override
    public List<AstPackDO> selectPackByStatus(Date scenDate, AstPackStatusEnum status, AstPackTradeTypeEnum tradeTypeEnum) {
        return astPackDAO.selectPackByStatus(scenDate, tradeTypeEnum.getCode(), status.getCode());
    }

    @Override
    public Long countByStatusAndTime(Date time, String status, AstPackTradeTypeEnum tradeTypeEnum) {
        return astPackDAO.countByStatusAndTime(time, status, tradeTypeEnum.getCode());
    }

    @Override
    public AstPackDO lockByPackCode(String packCode) {
        return astPackDAO.lockByPackCode(packCode);
    }
}
