/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.clearing.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanClearingOrderDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanClearingOrderDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanClearingOrderItem;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.convertor.PlanClearingOrderConvert;
import com.abssqr.plat.common.facade.model.trans.PlanClearingOrder;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.general.system.common.model.Money;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 结算单处理
 *
 * @author hanfei
 * @version $Id: PlanClearingOrderRepoImpl.java, v 0.1 2019-07-11 10:37 PM hanfei Exp $
 */
@Repository
public class PlanClearingOrderRepoImpl implements PlanClearingOrderRepo {

    @Autowired
    private PlanClearingOrderDAO planClearingOrderDAO;

    /**
     * 创建结算单
     *
     * @param param
     * @return int
     */
    @Override
    public int createOrder(PlanClearingOrder param) {
        return planClearingOrderDAO.insert(PlanClearingOrderConvert.convert2Do(param));
    }

    /**
     * 批量创建结算单
     *
     * @param list list
     * @return int
     */
    @Override
    public int createOrderList(List<PlanClearingOrder> list) {
        return planClearingOrderDAO.insertBatch(PlanClearingOrderConvert.convert2Dos(list));
    }

    /**
     * 单条更新结算单发起交易的信息
     *
     * @param memo
     * @param seqNo
     * @param transStatus
     * @param transOrderNo
     * @param oldTransStatus
     * @return
     */
    @Override
    public int updateOrderTransBySeqNo(String memo, String seqNo, TaskStatusEnum transStatus, String transOrderNo, TaskStatusEnum oldTransStatus) {
        return planClearingOrderDAO.updateBySeqNo(memo, seqNo, transStatus.getCode(), transOrderNo, oldTransStatus.getCode());
    }

    /**
     * 批量更新结算单发起交易的信息
     *
     * @param memo
     * @param seqNos
     * @param transStatus
     * @param transOrderNo
     * @param oldTransStatus
     * @return
     */
    @Override
    public int updateOrderTransBySeqNos(String memo, List<String> seqNos, TaskStatusEnum transStatus,
                                        String transOrderNo, TaskStatusEnum oldTransStatus) {
        return planClearingOrderDAO.updateBySeqNoList(memo, transStatus.getCode(), transOrderNo, oldTransStatus.getCode(), seqNos);
    }

    /**
     * 获取汇总结算单信息
     *
     * @param planNo      planNo
     * @param transStatus transStatus
     * @return List<PlanClearingOrderStat>
     */
    @Override
    public List<PlanClearingOrderItem> queryByClearingOrderStat(String planNo, TaskStatusEnum transStatus, Date accountDate) {
        return planClearingOrderDAO.queryByClearingOrderStat(accountDate, planNo, transStatus.getCode());
    }

    /**
     * 查看结算单是否都处理完
     *
     * @param planNo
     * @return
     */
    @Override
    public Long cntInitClearingOrderByPlanNo(String planNo) {
        return planClearingOrderDAO.cntInitClearingOrderByPlanNo(planNo);
    }

    @Override
    public Long cntInitClearingOrder(TransCodeEnum transCode) {
        return planClearingOrderDAO.cntInitClearingOrder(transCode.getCode());
    }

    @Override
    public PlanClearingOrder lockBySeqNo(String seqNo) {
        PlanClearingOrderDO order = planClearingOrderDAO.lockBySeqNo(seqNo);
        return PlanClearingOrderConvert.convert2Domain(order);
    }

    @Override
    public Money getAmtByPlanNo(Date accountDate, String planNo, TransCodeEnum transCode) {
        return planClearingOrderDAO.getClrOrderAmtByPlanNoAndActDateAndTransCode(accountDate, planNo, transCode == null ? null : transCode.getCode());
    }
}
