/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.pool.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.AstPoolDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstPoolDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.PactsAstMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstPoolItem;
import com.abssqr.plat.common.model.convertor.AstPoolConvert;
import com.abssqr.plat.common.model.domain.ast.AstPoolEntity;
import com.abssqr.plat.common.model.repo.pool.AstPoolRepository;
import com.abssqr.plat.common.model.utils.TextUtil;
import com.general.system.tool.util.ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.repo.pool: AstPoolRepositoryImpl.java, v 0.1 2018-01-07 下午1:12 guozhenhua Exp $
 */
@Component
public class AstPoolRepositoryImpl implements AstPoolRepository {


    @Autowired
    private AstPoolDAO astPoolDAO;

    @Override
    public  int  create(AstPoolEntity astPoolEntity) {
        AstPoolDO astPoolDO = new AstPoolDO();
        astPoolDO.setAstNo(astPoolEntity.getAstNo());
        astPoolDO.setContext(TextUtil.convertMap2Txt(astPoolEntity.getContext()));
        astPoolDO.setManagerOrgCode(astPoolEntity.getManagerOrgCode());
        astPoolDO.setTermNo(astPoolEntity.getTermNo());
        astPoolDO.setEndDate(astPoolEntity.getEndDate());
        astPoolDO.setStartDate(astPoolEntity.getStartDate());
        astPoolDO.setOwnOrg(astPoolEntity.getOwnOrg());
        return astPoolDAO.insert(astPoolDO);
    }


    @Override
    public PactsAstMatchPage queryForAstMatch(PactsAstMatchPage pactsAstMatchPage) {
        if (ToolUtil.isOneEmpty(pactsAstMatchPage.getOwnOrg(), pactsAstMatchPage.getSysDate())){
            throw new RuntimeException("缺少必要参数");
        }
        return astPoolDAO.queryForAstMatch(pactsAstMatchPage);
    }



    @Override
    public int delete(String astNo, int termNo) {
        return astPoolDAO.deleteByPk(astNo, termNo);
    }

    @Override
    public List<AstPoolItem> getAstPoolItems(Date sysDate, String astNo) {
        return astPoolDAO.getAstPoolItems(sysDate, astNo);
    }

    @Override
    public void updateSlice(String astNo, Integer maxTerm) {
        astPoolDAO.updateSlice(astNo,maxTerm);
    }
    
    /**
     * desc:获取资产池资产.<br/>
     * @param astNo astNo
     * @param ownOrg ownOrg
     * @return AstPoolDO
     */
    @Override
    public AstPoolEntity getAstPoolItem(String astNo, String ownOrg){
        AstPoolDO astPoolDO = astPoolDAO.getAstPoolItem(astNo, ownOrg);
        return AstPoolConvert.convertDomain(astPoolDO);
    }

}
