/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.task.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.CommonTaskDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.CommonTaskDO;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.repo.task.impl: CommonTaskRepositoryImpl.java, v 0.1 2018-03-23 下午5:07 guozhenhua Exp $ 
 */
@Component
public class CommonTaskRepositoryImpl implements CommonTaskRepository {
    
    @Autowired
    private CommonTaskDAO commonTaskDAO;

    private List<String> capticalMatchTypes=Lists.newArrayList(
            CommonTaskTypeEnum.UNMATCH_CAPTITALASSERT_MATCH.getCode(),
            CommonTaskTypeEnum.SYNC_CAPTITALASSERT_MATCH.getCode(),
            CommonTaskTypeEnum.CAPTITALASSERT_UNMATCH.getCode()
    );
    @Override
    public void create(CommonTask commonTask) {
        CommonTaskDO commonTaskDO = domain2do(commonTask);
        commonTaskDO.setGmtCreate(new Date());
        commonTaskDO.setGmtModified(new Date());
        commonTaskDAO.insert(commonTaskDO);
        commonTask.setId(commonTaskDO.getId());
        
    }
    
    @Override
    public CommonTask lock(Long id) {
        CommonTaskDO commonTaskDO = commonTaskDAO.lockById(id);
        return do2domain(commonTaskDO);
    }
    
    @Override
    public List<CommonTask> selectInitTask(List<CommonTaskTypeEnum> taskTypes,Date curTime, int limit) {
        VarChecker.checkNotEmpty(taskTypes);
        List<String> taskList = taskTypes.stream().map(CommonTaskTypeEnum::getCode).collect(Collectors.toList());
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.selectInitTask(curTime, limit,taskList);
        return doList2domainList(commonTaskDOS);
    }
    
    @Override
    public List<CommonTask> selectUnifinishTaskByType(CommonTaskTypeEnum taskType) {
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.selectUnifinishTaskByType(taskType.getCode());
        return doList2domainList(commonTaskDOS);
    }
    
    @Override
    public int updateTaskStatus(Long id, TaskStatusEnum newTaskStatus, TaskStatusEnum oldTaskStatus, String memo) {
        return commonTaskDAO.updateTaskStatus(id, memo, newTaskStatus.getCode(), oldTaskStatus.getCode());
    }
    
    @Override
    public int updateTaskExec(Date lastExecTime, String lastHost, List<CommonTask> tasks) {
        List<CommonTaskDO> commonTaskDOS = Lists.newArrayList();
        for (CommonTask task :
                tasks) {
            commonTaskDOS.add(domain2do(task));
        }
        
        return commonTaskDAO.updateTaskExec(lastExecTime, lastHost, commonTaskDOS);
    }
    
    @Override
    public int updateTaskExep(Date nextExecTime, Long id, String memo) {
        return commonTaskDAO.updateTaskExep(nextExecTime, id, memo);
    }
    
    @Override
    public int updateHangTasksByIp(String ip) {
        return commonTaskDAO.updateHangTasksByIp(ip);
    }

    @Override
    public List<CommonTask> queryInitAndExecUnMatchOrMatchTaskByContext(String planNo) {
        List<String> statuses=new ArrayList<>();
        statuses.add(TaskStatusEnum.INIT.getCode());
        statuses.add(TaskStatusEnum.EXECUTING.getCode());
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.queryUnMatchOrMatchTaskByContextAndStatus(planNo,statuses,capticalMatchTypes);

        return doList2domainList(commonTaskDOS);
    }

    @Override
    public List<CommonTask> queryExecUnMatchOrMatchTaskByContext(String planNo) {
        List<String> statuses=new ArrayList<>();
        statuses.add(TaskStatusEnum.EXECUTING.getCode());
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.queryUnMatchOrMatchTaskByContextAndStatus(planNo,statuses,capticalMatchTypes);

        return doList2domainList(commonTaskDOS);
    }

    @Override
    public List<CommonTask> queryInitUnMatchOrMatchTaskByContext(String planNo) {
        List<String> statuses=new ArrayList<>();
        statuses.add(TaskStatusEnum.INIT.getCode());
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.queryUnMatchOrMatchTaskByContextAndStatus(planNo,statuses,capticalMatchTypes);

        return doList2domainList(commonTaskDOS);
    }

    @Override
    public List<CommonTask> queryIgnoreUnMatchOrMatchTaskByContext(String planNo) {
        List<String> statuses=new ArrayList<>();
        statuses.add(TaskStatusEnum.IGNORE.getCode());
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.queryUnMatchOrMatchTaskByContextAndStatus(planNo,statuses,capticalMatchTypes);
        return doList2domainList(commonTaskDOS);
    }

    @Override
    public List<CommonTask> queryByBizNo(String bizNo) {
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.queryByBizNo(bizNo);
        return doList2domainList(commonTaskDOS);
    }

    @Override
    public int updateIgnoreTaskStatus2Init(Long id) {
        return commonTaskDAO.updateIgnoreTaskStatus2Init(id);
    }

    @Override
    public List<CommonTask> queryByTypesAndStatus(List<String> typeList, List<String> statusList) {
        List<CommonTaskDO> commonTaskDOS = commonTaskDAO.queryByTypesAndStatus(typeList, statusList);
        return doList2domainList(commonTaskDOS);
    }

    @Override
    public Long countByTypesAndStatusAndRange(Long second, List<String> typeList, List<String> statusList) {
        return commonTaskDAO.countByTypesAndStatusAndRange(second,typeList,statusList);
    }

    @Override
    public Long countUnFinishAndMultipleExecByTypes(List<String> typeList) {
        return commonTaskDAO.countUnFinishAndMultipleExecByTypes(typeList);
    }

    private CommonTaskDO domain2do(CommonTask commonTask) {
        
        CommonTaskDO commonTaskDO = new CommonTaskDO();
        commonTaskDO.setId(commonTask.getId());
        commonTaskDO.setBizNo(commonTask.getBizNo());
        commonTaskDO.setTaskType(commonTask.getTaskType().getCode());
        commonTaskDO.setTaskStatus(commonTask.getTaskStatus().getCode());
        commonTaskDO.setLastExecTime(commonTask.getLastExecTime());
        commonTaskDO.setNextExecTime(commonTask.getNextExecTime());
        commonTaskDO.setContext(commonTask.getContext());
        commonTaskDO.setLastHost(commonTask.getLastHost());
        commonTaskDO.setExecTimes(commonTask.getExecTimes());
        commonTaskDO.setMemo(commonTask.getMemo());
        commonTaskDO.setEnv(commonTask.getEnv());
        
        return commonTaskDO;
    }

    private CommonTask do2domain(CommonTaskDO commonTaskDO) {
        if (commonTaskDO == null) {
            return null;
        }
        
        CommonTask commonTask = new CommonTask();
        commonTask.setId(commonTaskDO.getId());
        commonTask.setBizNo(commonTaskDO.getBizNo());
        commonTask.setTaskType(CommonTaskTypeEnum.getByCode(commonTaskDO.getTaskType()));
        commonTask.setTaskStatus(TaskStatusEnum.getByCode(commonTaskDO.getTaskStatus()));
        commonTask.setLastExecTime(commonTaskDO.getLastExecTime());
        commonTask.setNextExecTime(commonTaskDO.getNextExecTime());
        commonTask.setContext(commonTaskDO.getContext());
        commonTask.setLastHost(commonTaskDO.getLastHost());
        commonTask.setExecTimes(commonTaskDO.getExecTimes());
        commonTask.setEnv(commonTaskDO.getEnv());
        commonTask.setMemo(commonTaskDO.getMemo());
        return commonTask;
    }

    private List<CommonTask> doList2domainList(List<CommonTaskDO> param) {
        List<CommonTask> result=new ArrayList<>();
        param.forEach(v->{
            result.add(do2domain(v));
        });
        return result;
    }
    
}
