/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.tr.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.TrTaskDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrTaskDO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.convertor.TrTaskConvert;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.repo.tr.TrTaskRepo;
import com.general.system.common.util.DateTimeUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.tr.impl: TrTaskRepoImpl.java, v 0.1 2018-01-06 01:18 hongwei.wang
 * Exp $
 */
@Component
public class TrTaskRepoImpl implements TrTaskRepo {
    @Autowired
    private TrTaskDAO trTaskDAO;
    
    /**
     * 批量插入转让任务
     *
     * @param trTasks
     * @return
     */
    @Override
    public int insertBatch(List<TrTask> trTasks) {
        if (CollectionUtils.isNotEmpty(trTasks)) {
            List<TrTaskDO> taskDOS = TrTaskConvert.convertDOs(trTasks);
            
            return trTaskDAO.insertBatch(taskDOS);
        }
        return 0;
    }
    
    @Override
    public void updateStatus(String batchNo, TaskStatusEnum status, TaskStatusEnum oldStatus) {
        trTaskDAO.updateBatchTaskStatus(batchNo, status.getCode(), oldStatus.getCode());
    }
    
    @Override
    public TrTask lockByIdNowait(Long id) {
        return TrTaskConvert.convertDTO(trTaskDAO.lockByIdNowait(id));
    }

    @Override
    public void clearHisTrTask(String spvPlanNo, Date acctDate, AstPackTradeTypeEnum astPackTradeTypeEnum) {
        Date daysBefore = DateTimeUtil.addDays(acctDate, -30);
        trTaskDAO.deleteHisTask(daysBefore,spvPlanNo, astPackTradeTypeEnum.getCode());
        trTaskDAO.deleteUnSuccTask(spvPlanNo, astPackTradeTypeEnum.getCode());
    }

    @Override
    public Long cntTrTaskByTfrOut(String tfrOut,List<String> statusList,AstPackTradeTypeEnum astPackTradeTypeEnum){
        return trTaskDAO.cntTrTaskByTfrOut(tfrOut, Optional.ofNullable(astPackTradeTypeEnum).map(AstPackTradeTypeEnum::getCode).orElse(null),statusList);
    }

}
