package com.abssqr.plat.common.util.utils;

import com.abssqr.plat.common.util.base.Day;
import com.general.system.common.util.DateTimeUtil;
import com.google.common.base.Preconditions;

import java.util.Date;

public class DayUtil {
    /**
     * 获得d1,d2之间相差的天数。如果某个日期为null，则返回参数非法异常
     * <p>
     * 注意：如果仅需判断d1是否大于(等于、小于)d2，请使用{@link #compare}方法
     * </p>
     * @param d1
     * @param d2
     * @return
     */
    public static int diffDays(Day d1, Day d2) {
        Preconditions.checkNotNull(d1);
        Preconditions.checkNotNull(d2);
        if (d1.equals(d2)) {
            return 0;
        }
        return (int) DateTimeUtil.calcDayDiff(d1.getDate(), d2.getDate());
    }

    /**
     * 判断两个日期是否相等
     *
     * @param d1
     * @param d2
     * @return
     */
    public static boolean checkEqualWithNull(Day d1, Day d2) {
        if (d1 == null && d2 == null) {
            return true;
        }

        if (d1 == null || d2 == null) {
            return false;
        }

        return diffDays(d1, d2) == 0;
    }

    /**
     * 比较两个8位日期对象d1与d2的大小
     *
     * @param d1
     * @param d2
     * @return 返回值等于0表示两个日期相等，大于0表示d1>d2,小于0表示d1<d2
     * @see String#compareTo(String)
     */
    public static int compare(Day d1, Day d2) {
        Preconditions.checkNotNull(d1);
        Preconditions.checkNotNull(d2);
        return d1.getDayString().compareTo(d2.getDayString());
    }

    /**
     * day 增加指定天数。addDays可以为 0 ，负
     *
     * @param sourceDay
     * @param addDays
     * @return
     */
    public static Day addDays(Day sourceDay, int addDays) {
        Preconditions.checkNotNull(sourceDay);
        return new Day(DateTimeUtil.addDays(sourceDay.getDate(), addDays));
    }

    /**
     *  指定日期增加月数，months可以为0 ， 负数
     * @param sourceMonth
     * @param months
     * @return
     */
    public static Day addMonth(Day sourceMonth, int months) {
        Preconditions.checkNotNull(sourceMonth);
        return new Day(DateTimeUtil.addMonths(sourceMonth.getDate(), months));
    }

    /**
     * 指定日期增加年数，years可以为0 ， 负数
     * @param sourceYear
     * @param years
     * @return
     */
    public static Day addYear(Day sourceYear, int years) {
        Preconditions.checkNotNull(sourceYear);
        return new Day(DateTimeUtil.addYears(sourceYear.getDate(), years));
    }

    /**
     * 指定日期減少年数，days可以为0，负数
     * @param sourceSubDay
     * @param days
     * @return
     */
    public static Day subtractDay(Day sourceSubDay, int days) {
        Preconditions.checkNotNull(sourceSubDay);
        return new Day(DateTimeUtil.addDays(sourceSubDay.getDate(), -days));
    }

    /**
     * 指定日期减少月数，months可以为0，负数
     * @param sourceSubMonth
     * @param months
     * @return
     */
    public static Day subtractMonth(Day sourceSubMonth, int months) {
        Preconditions.checkNotNull(sourceSubMonth);
        return new Day(DateTimeUtil.addMonths(sourceSubMonth.getDate(), -months));
    }

    /**
     * 指定日期减少年数，years可以为0，负数
     * @param sourceSubYear
     * @param years
     * @return
     */
    public static Day subtractYear(Day sourceSubYear, int years) {
        Preconditions.checkNotNull(sourceSubYear);
        return new Day(DateTimeUtil.addYears(sourceSubYear.getDate(), -years));
    }

    /**
     * 求两个时间相差的年数，计算方式为去两个时间的年相减
     *
     * @param startDay 起始时间
     * @param endDay 结束时间
     * @return 相差的年数
     */
    @SuppressWarnings("deprecation")
    public static int diffYears(Day startDay, Day endDay) {
        Preconditions.checkNotNull(startDay);
        Preconditions.checkNotNull(endDay);
        return endDay.getDate().getYear() - startDay.getDate().getYear();
    }

    /**
     * 求两个时间相差的年数，计算方式为去两个时间的年相减
     *
     * @param start 起始时间
     * @param end 结束时间
     * @return 相差的年数
     */
    @SuppressWarnings("deprecation")
    public static int diffYears(Date start, Date end) {
        Preconditions.checkNotNull(start);
        Preconditions.checkNotNull(end);
        return end.getYear() - start.getYear();
    }

    public static Day getMin(Day d1, Day d2) {
        Preconditions.checkNotNull(d1);
        Preconditions.checkNotNull(d2);
        if (diffDays(d1, d2) > 0) {
            return d2.clone();
        } else {
            return d1.clone();
        }
    }
}
