/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.core.service.action.impl;

import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.abssqr.plat.core.service.action.BaseSyncAction;
import com.abssqr.plat.core.service.ast.LoanAstInitManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 处理资产的初始化同步 —— 可针对放款和存量资产同步
 *
 * @author guozhenhua
 * @version com.abssqr.plat.core.service.action.impl: InitSyncAction.java, v 0.1 2017-12-12 下午4:47 guozhenhua Exp $
 */
@Component
public class InitSyncAction extends BaseSyncAction {
    
    private final static Logger LOGGER = CommonLoggers.COMMON_AST;
    
    @Autowired
    private LoanAstInitManager loanAstInitManager;
    
    @Override
    protected Loan doExecute(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans, Loan loan) {
        
        LogUtil.info(LOGGER, "资产生成开始outAstId:{0}", baseTsfRes.getLoanAstSyncBill().getLoanNo());
        
        if (baseTsfRes.getLoanAstSyncBill().getStatus() == LoanStatusEnum.REVERSE) {
            LogUtil.info(LOGGER, "同步的资产[{0}]状态为已冲正，直接跳过", baseTsfRes.getLoanAstSyncBill());
            return loan;
        }
        
        if (baseTsfRes.getLoanAstSyncBill().getStatus() == LoanStatusEnum.CLEAR &&
            DateTimeUtil.calcDayDiff(baseTsfRes.getLoanAstSyncBill().getAccountDate(),
                    baseTsfRes.getLoanAstSyncBill().getDisbDate())
            != 0) {
            LogUtil.info(LOGGER, "同步的资产[{0}]状态为已结清，且非上日放款，直接跳过", baseTsfRes.getLoanAstSyncBill());
            return loan;
        }
        
        loan = loanAstInitManager.initAst(baseTsfRes, loanAstSyncTrans);
        
        LogUtil.info(LOGGER, "资产生成结束,outAstId:{0},AstIds:{1}", loan.getOutAstNo(), loan.getAstNo());
        return loan;
    }
    
    @Override
    public TransCodeEnum getEventSceneCode() {
        return TransCodeEnum.SYNC;
    }
    
}
