/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.AstRiskClassEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.model.prod.ProductRiskCl;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanRisk;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.model.utils.AstUtil;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 资产风险处理manager
 *
 * @author guozhenhua
 * @version LoanAstRiskManager.java, v 0.1 2018-03-28 下午4:55 guozhenhua Exp $
 */
@Component
public class LoanAstRiskManager {

    @Autowired
    private LoanProdRepository loanProdRepository;
    
    /**
     * 五级分类处理
     * @param loan
     * @param loanProduct
     * @return
     */
    @PerformTrace
    public LoanRisk calcRiskInfo(Loan loan, LoanProduct loanProduct) {
        if (loanProduct == null){
            loanProduct = loanProdRepository.getBybssqrProdCode(loan.getProdNo());
        }
        LoanRisk loanRisk = new LoanRisk();
        int ovdDays = 0;
        for (LoanInstal loanInstal : loan.getLoanInstalList()) {
            if (loanInstal.getOvdDays() > ovdDays) {
                ovdDays = loanInstal.getOvdDays();
            }
        }
        
        loanRisk.setOvdDays(ovdDays);
        
        List<ProductRiskCl> productRiskCls = loanProduct.getProductRiskCls();
        AstRiskClassEnum astRiskCl = AstUtil.calcRiskCl(loanRisk.getOvdDays(), productRiskCls);
        loanRisk.setRiskClass(astRiskCl);
        
//        if (ovdDays > 180) {
//            loanRisk.setWtfStatus(YNEnum.Y);
//        }
        
        return loanRisk;
    }
}
