/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.core.service.change;

import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.model.convertor.WorkOrderConvert;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.repo.dataChange.ChangeRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.common.model.utils.SimpleMapUtil;
import com.abssqr.plat.core.service.plan.PlanAllocManager;
import com.general.system.bpm.domain.param.ProcessStartDefaultRequest;
import com.general.system.bpm.domain.result.ProcessAfterResult;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.bpm.facade.ConfigProcessVO;
import com.general.system.bpm.facade.ProcessDefaultStartHandler;
import com.general.system.bpm.facade.selector.AbstractProcessEndHandleNode;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.Map;

/**
 * 业务数据变更记录及 与接入审批流程
 *
 * @author yangcheng
 * @version ChangeManager.java, v0.12019-06-1409:46 yangchengExp$
 */
@Component
public class ChangeManager extends AbstractProcessEndHandleNode {
    private Map<DataChangeBizTypeEnum, ApproveCallBack> callBackMap = Maps.newHashMap();
    @Autowired
    private PlanAllocManager                            planAllocManager;

    public ChangeManager() {}

    @Autowired(required = false)
    public void setApproveCallBacks(List<ApproveCallBack> approveCallBacks) {
        if (CollectionUtils.isNotEmpty(approveCallBacks)) {
            approveCallBacks.forEach(approveCallBack -> {
                List<DataChangeBizTypeEnum> changeTypes = approveCallBack.getChangeTypes();
                if (CollectionUtils.isNotEmpty(changeTypes)) {
                    changeTypes.forEach(
                        dataChangeBizTypeEnum -> callBackMap.put(dataChangeBizTypeEnum, approveCallBack));
                }
            });

        }
    }

    @Autowired
    private ChangeRepo                 changeRepo;
    @Autowired
    private ProcessDefaultStartHandler processDefaultStartHandler;
    @Autowired
    private TransactionTemplate        transactionTemplate;

    /**
     * 保存组装变更记录数据
     *
     * @param bizType
     * @param bizNo
     * @param name
     * @param data
     * @param <T>
     * @return
     */
    public <T> BizDataChange addApprove(DataChangeBizTypeEnum bizType, String bizNo, String name, T data,
                                        String applyUser, String orgCode) {
        BizDataChange<T> dataChang = new BizDataChange();
        dataChang.setBizNo(bizNo);
        dataChang.setBizType(bizType);
        dataChang.setDatas(data);
        dataChang.setName(bizType.getDesc() + name);
        dataChang.setChangeNo(SequenceUtil.genId(IdTypeEnum.APPROVE));

        ConfigProcessVO configProcess = new ConfigProcessVO(bizType.getActiveCode(), bizType.getDesc());
        // 增加扩展参数名称
        SimpleMapUtil simpleMap =
            new SimpleMapUtil().putData(WorkOrderConvert.NAME_KEY, name).putData(WorkOrderConvert.RELATION_KEY, bizNo);
        // 启动工作流参数
        ProcessStartDefaultRequest defaultRequest = new ProcessStartDefaultRequest(
            orgCode, applyUser, name, configProcess, simpleMap);
        processDefaultStartHandler.execute(defaultRequest,
            (result) -> {
                // 流程完成时回调
                this.handle(result);
                return null;
            },
            (status) -> {
                // 业务逻辑
                dataChang.setApproveStatus(status.getCode());
                changeRepo.addChange(dataChang);
                return dataChang.getChangeNo();
            }
        );
        return dataChang;
    }

    /**
     * 查询信息
     */
    public BizDataChange getDataChange(String changeNo) {
        BizDataChange change = changeRepo.getChange(changeNo);
        return change;
    }

    /**
     * 流程完成审批回调业务
     *
     * @param result
     * @return
     */
    @Override
    public Void handle(ProcessAfterResult result) {
        String changeNo = result.getBusinessKey();
        ApproveStatusEnum approveStatus = result.getStatusEnum();
        if (ApproveChangeUtil.callBackPass(approveStatus)) {
            transactionTemplate.execute(s -> {
                BizDataChange change = changeRepo.getChange(changeNo);
                VarChecker.checkNotNull(change, "未找到对应的变更记录数据");
                ApproveStatusEnum oldStatus = ApproveStatusEnum.getEnum(change.getApproveStatus());
                if (oldStatus != approveStatus) {
                    // 保证数据一致性
                    change.setApproveStatus(approveStatus.getCode());
                    boolean status = changeRepo.updateStatus(approveStatus, change.getChangeNo(), oldStatus);
                    VarChecker.checkEquals(true, status, "状态更新失败");
                }
                ApproveCallBack callBack = callBackMap.get(change.getBizType());
                if (callBack != null) {
                    callBack.approveCallBack(change, approveStatus);
                }
                return s;
            });
        }
        return null;
    }

    @Override
    public boolean judge(ProcessAfterResult result) {
        return true;
    }

}
