/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.debugdata.manager;

import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.core.service.debugdata.model.QueryModel;
import com.general.system.common.model.exception.BaseDBException;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.MapListHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.debugdata.manager: QueryManager.java, v 0.1 2019-12-02 12:04 bangis.wangdf
 * Exp $
 */
@Service
public class QueryManager {
    public final static  Logger      LOGGER  = LoggerFactory.getLogger(QueryManager.class);
    private static final String      outPath = "/tmp/logs/wkplat/";
    @Autowired
    private              DataSource  dataSource;
    private              QueryRunner run;

    @PostConstruct
    void init() {
        run = new QueryRunner(dataSource);
    }

    public void exec(List<QueryModel> queryModels, Map<String, String> queryParams) {
        if (CollectionUtil.isNotEmpty(queryModels)) {
            queryModels.forEach(queryModel -> {
                List<Map<String, Object>> queryResults = query(queryModel, queryParams);
                if (CollectionUtil.isNotEmpty(queryResults)) {

                    for (Map<String, Object> queryResultMap : queryResults) {
                        if(queryModel.getParamDelService() != null) {
                            queryModel.getParamDelService().deal(queryResultMap, queryParams);
                        }if(queryModel.getConsumer()!=null) {
                            queryModel.getConsumer().accept(queryParams);
                        }
                    }
                }
                queryModel.setResult(queryResults);
            });
        }
    }

    private List<Map<String, Object>> query(QueryModel queryModel, Map<String, String> params) {
        StringBuilder sqlSb = new StringBuilder("select ");
        sqlSb.append(queryModel.getColums()).append(" from ").append(queryModel.getTableName());
        List<String> paramList = Lists.newArrayList();
        if (CollectionUtil.isNotEmpty(queryModel.getWhereColums())) {
            sqlSb.append(" where 1=1 ");
            for (String cloumn : queryModel.getWhereColums()) {
                sqlSb.append(" and ").append(cloumn).append(" =? ");
                if(params.get(cloumn)==null){
                    return null;
                }
                VarChecker.checkNotNull(params.get(cloumn), "获取{0}数据时{1}参数未赋值", queryModel.getTableName(), cloumn);
                paramList.add(params.get(cloumn));
            }
        }
        ResultSetHandler rsh = new MapListHandler();

        List<Map<String, Object>> resultMap = null;
        try {
            resultMap = (List)run.query(sqlSb.toString(), rsh, paramList.toArray());
        } catch (SQLException e) {
            LogUtil.error(LOGGER, e, e.getMessage());
            throw new BaseDBException("获取数据失败", e);

        }
        return resultMap;
    }
}
