/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.core.service.host;

import com.abssqr.plat.common.dal.mysql.auto.dao.HostInfoDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.HostInfoDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.HostHeartInfo;
import com.abssqr.plat.common.facade.system.HostInfo;
import com.general.system.common.util.SystemDateUtil;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author zhenxuan.luo
 * @version $Id: HostManager.java, v 0.1 2017-06-29 下午15:46 zhenxuan.luo Exp $
 */
@Component
public class HostManager implements InitializingBean {

    @Autowired
    private HostInfoDAO hostInfoDAO;

    private HostInfo hostInfo;

    @Override
    public void afterPropertiesSet() throws Exception {

        InetAddress addr = InetAddress.getLocalHost();

        HostInfoDO hostInfo = hostInfoDAO.selectByName(addr.getHostName());

        Date newTime = SystemDateUtil.getSystemDate();

        if (null == hostInfo) {
            hostInfo = new HostInfoDO();
            hostInfo.setHostIp(addr.getHostAddress());
            hostInfo.setHostName(addr.getHostName());
            hostInfo.setLastBoot(newTime);
            hostInfo.setLastHeart(newTime);
            hostInfoDAO.insert(hostInfo);
        } else {
            hostInfo.setLastBoot(newTime);
            hostInfo.setLastHeart(newTime);
            hostInfoDAO.updateByHostNameUnique(hostInfo);
        }


        this.hostInfo = convert(hostInfo);

    }

    /**
     * 发送心跳
     */
    public void sendHeartbeat() {
        hostInfoDAO.updateHeart(hostInfo.getHostName());
        this.hostInfo.setLastHeart(hostInfoDAO.selectByName(hostInfo.getHostName()).getLastHeart());
    }

    public HostInfo getHostInfo() {
        return hostInfo;
    }

    /**
     * 获取超时主机
     * @param currentTime
     * @param overTime
     * @return
     */
    public List<HostInfo> listOvertimeHost(Date currentTime, Integer overTime) {
        return convert2ModelList(hostInfoDAO.listOvertimeHost(currentTime, overTime));
    }
    /**
     * @return
     */
    public List<HostHeartInfo> getAllHost() {
        return hostInfoDAO.getAllHost();
    }


    private List<HostInfo> convert2ModelList(List<HostInfoDO> param) {
        if (param == null) {
            return null;
        }
        List<HostInfo> result = new ArrayList<>(param.size());
        param.forEach(
                item -> result.add(convert(item))
        );
        return result;
    }

    private HostInfo convert(HostInfoDO param) {
        HostInfo result = new HostInfo();
        //id
        result.setId(param.getId());
        //hostIp
        result.setHostIp(param.getHostIp());
        //hostName
        result.setHostName(param.getHostName());
        //lastBoot
        result.setLastBoot(param.getLastBoot());
        //lastHeart
        result.setLastHeart(param.getLastHeart());
        return result;
    }
}