/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.core.service.plan;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import com.abssqr.plat.common.dal.mysql.auto.paging.PlanListPage;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.CalcTypeEnum;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeTimeTypeEnum;
import com.abssqr.plat.common.facade.enums.FinSerFeeTypeEnum;
import com.abssqr.plat.common.facade.enums.HandleTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.facade.model.org.Organization;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanPageQryParam;
import com.abssqr.plat.common.model.convertor.PlanParamConvert;
import com.abssqr.plat.common.model.convertor.page.PageConvert;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.acct.AcctRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAstPriceRuleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanFeeStructureRepo;
import com.abssqr.plat.common.model.repo.plan.PlanInvestStructureRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.plan.PlanTransferRuleRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.change.ApproveCallBack;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.abssqr.plat.core.service.org.OrgManager;

import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.enums.CommonErrorCodeEnum;
import com.general.system.common.model.exception.BaseCommonException;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.facade.current.IdentityUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 计划核心服务
 *
 * @author bangis.wangdf
 * @version com.abssqr.admin.core.admin.plan: PlanManager.java, v 0.1 2018-01-18 下午12:02 yaphet Exp $
 */
@Component
public class PlanManager implements ApproveCallBack<PlanBase> {
    private static final Logger LOGGER = LoggerFactory.getLogger(PlanManager.class);
    @Autowired
    PlanRepo                   planRepo;
    @Autowired
    PlanTransferRuleManager    planTransferRuleManager;
    @Autowired
    PlanInvestStructureManager planInvestStructureManager;
    @Autowired
    PlanFeeStructureManager    planFeeStructureManager;
    @Autowired
    AcctManager                acctManager;
    @Autowired
    AcctRepo                   acctRepo;
    @Autowired
    PlanAcctTitleManager       planAcctTitleManager;
    @Autowired
    ChangeManager              changeManager;
    @Autowired
    OrgManager                 orgManager;
    @Autowired
    PlanAllocManager           planAllocManager;
    @Autowired
    AstScreenRuleRepo          astScreenRuleRepo;
    @Autowired
    PlanAcctTitleRepo          planAcctTitleRepo;
    @Autowired
    AstSyncTaskCtrRepo         astSyncTaskCtrRepo;
    @Autowired
    PlanAstPriceRuleRepo       planAstPriceRuleRepo;
    @Autowired
    PlanInvestStructureRepo    planInvestStructureRepo;
    @Autowired
    PlanFeeStructureRepo       planFeeStructureRepo;
    @Autowired
    PlanTransferRuleRepo       planTransferRuleRepo;
    @Autowired
    PlanReportRepo             planReportRepo;
    @Autowired
    OrgAcctDayComponent        orgAcctDayComponent;

    @Override
    public DataChangeBizTypeEnum getChangeType() {
        return DataChangeBizTypeEnum.PLAN;
    }

    @Override
    public List<DataChangeBizTypeEnum> getChangeTypes() {
        return Lists.newArrayList(DataChangeBizTypeEnum.PLAN, DataChangeBizTypeEnum.PlanEntity);
    }

    /**
     * 分页查询计划
     *
     * @param param
     * @return
     */
    public PlanListPage getPlanList(PlanPageQryParam param, List<String> statusList, String userCode) {
        PlanListPage planList = PageConvert.toDBBasePate(PlanListPage.class, param, p -> {
            p.setPassed(true);
            // 如果是查询接口则加入身份作为条件
            if (HandleTypeEnum.LIST.getCode().equals(param.getHandleType())) {
                p.setUserCode(IdentityUtil.getUserCode());
            }
            p.setPlanName(param.getPlanName());
            p.setPlanNo(param.getPlanNo());
            p.setPlanStatus(param.getPlanStatus());
            p.setPlanType(param.getPlanType());
            p.setList(statusList);
        });
        return planRepo.getPlanList(planList);
    }

    /**
     * 获取 填充全量信息的计划
     *
     * @param planNo
     * @return
     */
    @PerformTrace
    public <T extends PlanBase> T getPlanFullByNo(String planNo) {
        return getPlanFullByNo(planNo, false);
    }

    /**
     * 获取 填充全量信息的计划
     *
     * @param planNo
     * @return
     */
    public <T extends PlanBase> T getPlanFullByNo(String planNo, boolean balance) {
        PlanBase planBase = planRepo.getPlanByNo(planNo);
        Organization sponsorOrg = orgManager.getOrg(planBase.getSponsorOrgCode());
        planBase.setSponsorOrgName((sponsorOrg == null) ? "" : sponsorOrg.getOrgName());
        Organization managerOrg = orgManager.getOrg(planBase.getPlanMgrOrgCode());
        planBase.setPlanMgrOrgName((managerOrg == null) ? "" : managerOrg.getOrgName());
        return this.fillPlanInfo(planBase, balance);
    }

    /**
     * 填充计划附属信息
     *
     * @param planBase
     * @return
     */
    private <T extends PlanBase> T fillPlanInfo(PlanBase planBase, boolean balance) {
        if (planBase != null) {
            String planNo = planBase.getPlanNo();
            //账号
            planBase.setAccounts(acctManager.getAcctByPlanNoAndType(planNo, AccountTypeEnum.BANK, balance));
            //投资结构
            planBase.setInvestStructureList(planInvestStructureManager.selectByPlanNo(planNo));
            //费用
            planBase.setFeeStructureList(planFeeStructureManager.selectByPlanNo(planNo));

            if (planBase.getPlanType() == PlanTypeEnum.PLAN) {
                Plan plan = (Plan)planBase;
                //转让规则
                plan.setPlanTransferRule(planTransferRuleManager.getByPlanNo(planNo));
                // 定价规则
                plan.setPlanAstPriceRuleList(planAstPriceRuleRepo.queryByPriceRulePlanNo(planNo));
                return (T)plan;
            }
            return (T)planBase;
        }
        return null;
    }

    /**
     * 根据 changeNo 查询数据
     *
     * @param changeNo
     * @return
     */
    public <T extends PlanBase> T getDataChangePlanBy(String changeNo) {
        BizDataChange<T> dataChange = changeManager.getDataChange(changeNo);
        if (dataChange == null) {
            throw new AbssqrBizException("未找到对应待审批的计划数据");
        }
        dataChange.getDatas().setApproveStatus(ApproveStatusEnum.getEnum(dataChange.getApproveStatus()));
        return dataChange.getDatas();
    }

    /**
     * 新增修改计划 ，已经开始转让的计划不允许修改？
     *
     * @param param
     * @return
     */
    public String addOrUpdatePlan(PlanOprParam param, String userCode, String orgCode) {

        PlanBase planBase = PlanParamConvert.convert2PlanBase(param);

        // 判断类型
        if (param.getDraft() == true) {
            planBase.setApproveStatus(ApproveStatusEnum.UNSUBMIT);
            planBase.setStatus(PlanStatusEnum.PREPARING);
            //判断计划是否存在
            if (planRepo.planExist(planBase.getPlanNo(), planBase.getPlanName()) > 0) {
                PlanBase existPlan = planRepo.getPlanByNo(planBase.getPlanNo());
                // 校验状态
                if (planBase != null && (existPlan.getStatus() != PlanStatusEnum.PREPARING
                    || existPlan.getApproveStatus() != ApproveStatusEnum.UNSUBMIT)) {
                    String msg = MessageUtil.formatMsg("计划[{0}]状态为[{1}],审批状态为[{2}]不能保存草稿", existPlan.getPlanNo(),
                        existPlan.getStatus().getDesc(), existPlan.getApproveStatus().getCode());
                    throw new BaseCommonException(msg, CommonErrorCodeEnum.INVALID_PARAM);

                }
            }
            VarChecker.checkEquals(planRepo.getSamePlan(planBase.getPlanNo(), planBase.getPlanName()),
                0, "存在该[{0}]计划编号的计划名称[{1}]",
                planBase.getPlanNo(), planBase.getPlanName());
            LogUtil.info(LOGGER, "保存计划[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
            planBase.setGmtCreator(userCode);
            planBase.setPassed(false);
            if (param.getAdd()) {
                // 添加计划
                planRepo.addPlan(planBase);
            } else {
                // 修改计划
                planRepo.updatePlan(planBase);
            }
            //编辑其他信息
            this.editStructure(planBase);
        } else {
            // 审批状态默认为审批中
            planBase.setApproveStatus(ApproveStatusEnum.PENDING);
            if (param.getAdd()) {
                VarChecker.checkEquals(planRepo.planExist(planBase.getPlanNo(), planBase.getPlanName()),
                    0, "存在该[{0}]计划编号或[{1}]计划名称",
                    planBase.getPlanNo(), planBase.getPlanName());
                LogUtil.info(LOGGER, "新增计划[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
                planBase.setGmtCreator(userCode);

                planBase.setPassed(false);
                // 添加计划
                planRepo.addPlan(planBase);
                //编辑其他信息
                this.editStructure(planBase);
            } else {
                VarChecker.checkEquals(planRepo.getSamePlan(planBase.getPlanNo(), planBase.getPlanName()),
                    0, "存在该[{0}]计划编号的计划名称[{1}]",
                    planBase.getPlanNo(), planBase.getPlanName());
                LogUtil.info(LOGGER, "修改计划[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());

                PlanBase dbPlanBase = planRepo.getPlanByNo(planBase.getPlanNo());
                //判断计划能否修改
                ApproveChangeUtil
                    .bizPassedModification(dbPlanBase.getApproveStatus(), dbPlanBase.getPlanNo(),
                        dbPlanBase.getPlanName());

                planBase.setGmtModifier(userCode);
                planBase.setStatus(dbPlanBase.getStatus());
                planBase.setPassed(dbPlanBase.isPassed());
                //准备中，已撤销可以修改
                if (PlanStatusEnum.PREPARING == dbPlanBase.getStatus()) {
                    // 修改计划
                    planRepo.updatePlan(planBase);
                    //编辑其他信息
                    this.editStructure(planBase);
                } else {
                    planRepo.updateStatusByPlanNo(planBase.getPlanNo(), userCode, ApproveStatusEnum.PENDING);
                }
            }

            //添加审批模型数据
            this.dataChangeElement(planBase);
            // 调用数据变更 提交审批
            DataChangeBizTypeEnum dataChangeBizType = planBase.getPlanType() == PlanTypeEnum.SPV
                ? DataChangeBizTypeEnum.PlanEntity : DataChangeBizTypeEnum.PLAN;
            //提交审批
            BizDataChange dataChange = changeManager
                .addApprove(dataChangeBizType, planBase.getPlanNo(), planBase.getPlanName(), planBase, userCode,
                    orgCode);
        }
        return planBase.getPlanNo();
    }

    /**
     * 修改计划附属配置信息
     *
     * @param planBase
     */
    private void editStructure(PlanBase planBase) {
        //账号
        acctManager.addOrUpdateAccounts(planBase.getPlanNo(), planBase.getAccounts());
        //投资结构
        planInvestStructureManager.addOrUpdate(planBase.getPlanNo(), planBase.getInvestStructureList());
        //费用
        planFeeStructureManager.addOrUpdate(planBase.getPlanNo(), planBase.getFeeStructureList());
        if (planBase.getPlanType() == PlanTypeEnum.PLAN) {
            Plan plan = (Plan)planBase;
            //转让规则
            planTransferRuleManager.addOrUpdate(plan.getPlanNo(), plan.getPlanTransferRule());
        }
    }

    /**
     * 删除草稿
     *
     * @return
     */
    public Void deleteDraftPlan(String planNo) {
        PlanBase existPlan = planRepo.getPlanByNo(planNo);
        if (existPlan != null) {
            if ((existPlan.getStatus() != PlanStatusEnum.PREPARING
                || existPlan.getApproveStatus() != ApproveStatusEnum.UNSUBMIT)) {
                String msg = MessageUtil.formatMsg("计划[{0}]状态为[{1}],审批状态为[{2}]不能删除", existPlan.getPlanNo(),
                    existPlan.getStatus().getDesc(), existPlan.getApproveStatus().getDesc());
                throw new BaseCommonException(msg, CommonErrorCodeEnum.INVALID_PARAM);
            }
            // 删除计划
            planRepo.deleteByPlanNo(planNo);
            // 删除计划相关信息
            //账号
            acctRepo.deleteByEntityCode(planNo);
            //投资结构
            planInvestStructureRepo.cleanByPlanNo(planNo);
            //费用
            planFeeStructureRepo.cleanByPlanNo(planNo);
            //转让规则
            planTransferRuleRepo.deleteByPlanNo(planNo);
            //定价信息
            planAstPriceRuleRepo.deleteByPlanNo(planNo);

        }
        return null;
    }

    /**
     * 审批完成后回调
     *
     * @param approveStatus
     * @param bizDataChange
     */
    @Override
    public void approveCallBack(BizDataChange<PlanBase> bizDataChange, ApproveStatusEnum approveStatus) {
        // 获取转让会计日
        Date accountDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        //审批通过后 创建
        PlanBase planModel = bizDataChange.getDatas();
        PlanBase dbPlan = planRepo.lockPlanByNo(planModel.getPlanNo());

        VarChecker.checkNotNull(dbPlan, "[{0}]计划信息不存在-审批失败", planModel.getPlanNo());

        if (DateTimeUtil.calcDayDiff(accountDate, planModel.getBeginDate()) >= 0L
            && approveStatus == ApproveStatusEnum.PASSED) {
            planModel.setStatus(PlanStatusEnum.ONGOING);
        }
        planModel.setApproveStatus(approveStatus);

        planRepo.updateStatus(planModel.getPlanNo(), planModel.getStatus(), planModel.getApproveStatus());
        if (ApproveChangeUtil.callBackSucceed(approveStatus)) {
            planRepo.updatePassed(planModel.getPlanNo(), true);
            //编辑其他信息
            this.editStructure(planModel);
            //初始化计划科目
            planAcctTitleManager.initPlanAcctTitles(dbPlan, planModel.getEffectDate(),
                planModel.getInvestStructureList(), planModel.getFeeStructureList());

            // 投入资本到账 初始化成立发行日上一日空值报表基础数据
            planReportRepo.defaultRPT001Init(planModel.getPlanNo(), planModel.getBeginDate());


            //初始化分配计划
            planAllocManager.initOrAppendAlloc(planModel, dbPlan.getStatus(), accountDate);

            //编辑是终止购买日 与 上一次有无变化
            if (DateTimeUtil.isBeforeOnDate(accountDate, planModel.getBeginDate()) && DateTimeUtil.isSameDay(
                accountDate,
                planModel.getBeginDate())) {
                planAllocManager.modifyPlanAlloc(planModel);
            }
            // 增加放款池编号到同步任务
            if (planModel.getPlanType() == PlanTypeEnum.SPV && planModel instanceof OrganizationEntity) {
                OrganizationEntity entity = (OrganizationEntity)dbPlan;
                astSyncTaskCtrRepo.addSyncSPV(entity.getManagerOrgCode(), entity.getPlanNo());
            }
        }
    }

    private void dataChangeElement(PlanBase planBase) {
        Organization sponsorOrg = orgManager.getOrg(planBase.getSponsorOrgCode());
        planBase.setSponsorOrgName((sponsorOrg == null) ? "" : sponsorOrg.getOrgName());
        Organization managerOrg = orgManager.getOrg(planBase.getPlanMgrOrgCode());
        planBase.setPlanMgrOrgName((managerOrg == null) ? "" : managerOrg.getOrgName());
        if (planBase instanceof Plan) {
            Plan plan = (Plan)planBase;
            List<AstScreenRule> astScreenRules = plan.getPlanTransferRule().getAstScreenRuleList();
            List<String> screenRuleNos = astScreenRules.stream().map(AstScreenRule::getScreenRuleNo).collect(
                Collectors.toList());
            plan.getPlanTransferRule().setAstScreenRuleList(astScreenRuleRepo.getByNos(screenRuleNos));
        }
    }

    /**
     * 根据spv 查询受让池计划信息
     *
     * @param ownOrgs
     * @param <T>
     * @return
     */
    public <T extends PlanBase> Map<String, List<T>> getPlanBySpvs(List<String> ownOrgs) {
        Map<String, List<T>> result = Maps.newHashMap();
        Map<String, List<String>> planNosMap = planRepo.getPlanBySpvs(ownOrgs);
        if (MapUtils.isNotEmpty(planNosMap)) {
            planNosMap.forEach((ownOrg, planNos) -> {
                List<T> planBases = planRepo.getPlanByPlanNos(planNos, PlanTypeEnum.PLAN,
                    PlanStatusEnum.ONGOING);
                if (CollectionUtils.isNotEmpty(planBases)) {
                    planBases.forEach(planBase -> {
                        fillPlanInfo(planBase, false);
                    });
                    result.put(ownOrg, planBases);
                }
            });
        }
        return result;
    }

    // 兼容代码删除
    //    public void initPlanFor20191030() {
    //        List<PlanBase> spvPlans = planRepo.getPlansByStatus(PlanTypeEnum.SPV, PlanStatusEnum.ONGOING);
    //
    //        if (CollectionUtils.isNotEmpty(spvPlans)) {
    //            spvPlans.forEach(planBase -> {
    //                fillPlanInfo(planBase, false);
    //                LogUtil.info(LOGGER, "补偿放款计划-水位[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
    //                //初始化水位
    //                planAstManager.initPlanAst(planBase.getPlanNo(), planBase.getPlanType());
    //                //金融服务费补偿
    //                LogUtil.info(LOGGER, "补偿放款计划-金融服务费[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
    //                finFeeCompensate(planBase);
    //
    //                //初始化计划科目
    //                LogUtil.info(LOGGER, "补偿放款计划-初始化计划科目[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
    //                planAcctTitleManager.initPlanAcctTitles(planBase, planBase.getEffectDate(),
    //                    planBase.getInvestStructureList(), planBase.getFeeStructureList());
    //
    //                //初始化分配计划
    //                LogUtil.info(LOGGER, "补偿放款计划-初始化分配计划[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
    //                planAllocManager.initOrAppendAlloc(planBase, planBase.getStatus());
    //
    //            });
    //        }
    //
    //        List<PlanBase> plans = planRepo.getPlansByStatus(PlanTypeEnum.PLAN, PlanStatusEnum.ONGOING);
    //
    //        if (CollectionUtils.isNotEmpty(plans)) {
    //            plans.forEach(planBase -> {
    //                fillPlanInfo(planBase, false);
    //                //金融服务费补偿
    //                LogUtil.info(LOGGER, "补偿受让计划-金融服务费补偿[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
    //                FeeStructure feeStructure = finFeeCompensate(planBase);
    //                //初始化计划科目
    //                LogUtil.info(LOGGER, "补偿受让计划-金融服务费科目[{0}][{1}]", planBase.getPlanNo(), planBase.getPlanName());
    //                planAcctTitleManager.initPlanAcctTitles(planBase, planBase.getEffectDate(),
    //                    planBase.getInvestStructureList(), planBase.getFeeStructureList());
    //
    //                //补偿分配方案
    //                if (feeStructure != null) {
    //
    //                    List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(planBase.getPlanNo());
    //                    PlanAcctTitle planAcctTitle = planAcctTitles.stream().filter(
    //                        t -> StringUtils.equals(t.getRefNo(), feeStructure.getFeeNo())).findFirst().get();
    //                    List<PlanAlloc> planAllocs = planAllocManager.queryAllocByEntityNo(planBase.getPlanNo());
    //                    if (CollectionUtils.isNotEmpty(planAllocs)) {
    //                        planAllocs.forEach(planAlloc -> {
    //                            PlanAllocItem allocItem = new PlanAllocItem();
    //                            allocItem.setItemNo(SequenceUtil.genId(IdTypeEnum.ALLOCITM));
    //                            allocItem.setTax(true);
    //                            allocItem.setTitleNo(planAcctTitle.getTitleNo());
    //                            allocItem.setTitleName(planAcctTitle.getTitleName());
    //                            allocItem.setRelNo(feeStructure.getFeeNo());
    //                            allocItem.setLayerOrder(0);
    //                            allocItem.setOrder(0);
    //                            LogUtil.info(LOGGER, "补偿受让计划-分配方案[{0}][{1}][{2}]", planBase.getPlanNo(),
    //                                planBase.getPlanName(), allocItem.getItemNo());
    //                            planAllocRepo.insertPlanAllocItem(allocItem, planBase.getPlanNo(), planAlloc
    //                            .getAllocNo());
    //                        });
    //                    }
    //                }
    //            });
    //
    //        }
    //    }

    private FeeStructure finFeeCompensate(PlanBase planBase) {
        FeeStructure result = null;
        List<FeeStructure> feeStructureList = planBase.getFeeStructureList();
        AtomicBoolean has = new AtomicBoolean(false);
        if (CollectionUtils.isNotEmpty(feeStructureList)) {
            feeStructureList.forEach(feeStructure -> {
                if (feeStructure.getFeeName().startsWith("金融服务费")) {
                    has.set(true);
                }
            });
        }
        if (!has.get()) {
            result = new FeeStructure();
            result.setFeeName("金融服务费");
            result.setFeeNo(SequenceUtil.genId(IdTypeEnum.FEE));
            result.setFinSerFeeType(FinSerFeeTypeEnum.WITHHOLD);
            result.setCalcByDay(false);
            result.setFeeCalcType(CalcTypeEnum.FIN_SER_FEE);
            // result.setFeeScaleType(FeeScaleTypeEnum.START);
            result.setFeeTimeType(FeeTimeTypeEnum.TERM);
            feeStructureList.add(result);
            //费用
            planFeeStructureManager.addOrUpdate(planBase.getPlanNo(), feeStructureList);
        }
        return result;
    }
}
