/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.report.calc.impl;

import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.model.report.ReportItem;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanLog;
import com.abssqr.plat.common.model.enums.OvdDaysTypeEnum;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 报表 逾期天数
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.report.calc.impl: OvdDaysItemCalc.java, v 0.1 2019-06-27 00:59 zhenxuan.luo Exp $
 */
@Component
public class OvdDaysItemCalc extends BaseReportItemCalc {
    
    private static Map<OvdDaysTypeEnum, OvdDaysDesc> ovdDaysDescMap = Maps.newHashMap();
    
    static {
        ovdDaysDescMap.put(OvdDaysTypeEnum.OD01, new OvdDaysDesc(1, 3, OvdDaysTypeEnum.OD01, "逾期1天-3天（含）本金余额"));
        ovdDaysDescMap.put(OvdDaysTypeEnum.OD02, new OvdDaysDesc(1, 30, OvdDaysTypeEnum.OD02, "逾期1天-30天(含)本金余额"));
        ovdDaysDescMap.put(OvdDaysTypeEnum.OD03, new OvdDaysDesc(31, 60, OvdDaysTypeEnum.OD03, "逾期31天-60天(含)本金余额"));
        ovdDaysDescMap.put(OvdDaysTypeEnum.OD04, new OvdDaysDesc(61, 90, OvdDaysTypeEnum.OD04, "逾期61天-90天(含)本金余额"));
        ovdDaysDescMap.put(OvdDaysTypeEnum.OD05, new OvdDaysDesc(91, null, OvdDaysTypeEnum.OD05, "逾期91天及以上本金余额"));
        ovdDaysDescMap.put(OvdDaysTypeEnum.OD06, new OvdDaysDesc(180, null, OvdDaysTypeEnum.OD06, "逾期180天及以上本金余额"));
    }
    
    @Override
    protected ReportAstStatTypeEnum getType() {
        return ReportAstStatTypeEnum.OVD_DAYS;
    }
    
    @Override
    public List<ReportItem> calcByInstal(Loan loan, LoanInstal loanInstal, Date accountDate) {
        return buildCommonItem(loan.getLoanRisk().getOvdDays());
    }
    
    @Override
    public List<ReportItem> calcByLog(LoanLog loanLog, LoanInstalLog log) {
        return buildCommonItem(loanLog.getBfOvdDays());
    }
    
    public String getDesc(OvdDaysTypeEnum type) {
        OvdDaysDesc ovdDaysDesc = ovdDaysDescMap.get(type);
        if (null != ovdDaysDesc) {
            return ovdDaysDesc.desc;
        } else {
            return "-";
        }
    }
    
    private List<ReportItem> buildCommonItem(int ovdDays) {
        List<ReportItem> reportItems = Lists.newArrayList();
        
        for (OvdDaysDesc ovdDaysDesc : ovdDaysDescMap.values()) {
            
            if (ovdDaysDesc.isMatch(ovdDays)) {
                ReportItem reportItem = new ReportItem();
                reportItem.setAstStatType(getType());
                reportItem.setAstStatVal(ovdDaysDesc.code.getCode());
                reportItems.add(reportItem);
            } else {
                continue;
            }
        }
        return reportItems;
    }
    
    private static class OvdDaysDesc {
        
        Integer lowerLimit;
        
        Integer upperLimit;
        
        OvdDaysTypeEnum code;
        
        String desc;
        
        public OvdDaysDesc(Integer lowerLimit, Integer upperLimit, OvdDaysTypeEnum code,
                           String desc) {
            this.lowerLimit = lowerLimit;
            this.upperLimit = upperLimit;
            this.code = code;
            this.desc = desc;
        }
        
        public boolean isMatch(int ovdDays) {
            if (upperLimit == null) {
                return ovdDays >= lowerLimit;
            } else {
                return ovdDays >= lowerLimit && ovdDays <= upperLimit;
            }
        }
    }
}