/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.rule;

import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.model.domain.rule.LoanAstPriceCalcParam;
import com.abssqr.plat.common.model.domain.rule.LoanAstPriceResult;
import com.abssqr.plat.core.service.rule.node.AbstractPriceBaseCalc;
import com.general.system.util.adapter.selector.AbstractSingleActionSelector;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 资产定价计算组件
 * @author hanfei
 * @version $Id: AstPriceRuleComponent.java, v 0.1 2020-02-07 6:01 PM hanfei Exp $ 
 */
@Component
public class AstPriceRuleComponent
        extends AbstractSingleActionSelector<LoanAstPriceCalcParam, LoanAstPriceResult, AstPackTradeTypeEnum> {
    
    @Autowired(required = false)
    private AbstractPriceBaseCalc[] abstractPriceBaseCalc;
    
    /**
     * 初始化所有交易类型的定价计算器
     */
    @Override
    public void initHandlers() {
        super.setHandlers(Lists.newArrayList(abstractPriceBaseCalc));
    }
    
    @Override
    public List<AstPackTradeTypeEnum> getAllAllowTypeValues() {
        return Lists.newArrayList(AstPackTradeTypeEnum.values());
    }
    
    /**
     * 按交易类型匹配计算器
     * @param request
     * @return
     */
    @Override
    public List<AstPackTradeTypeEnum> getRequestAllowTypes(LoanAstPriceCalcParam request) {
        return Lists.newArrayList(request.getAstTradeType());
    }
    
    
}
