/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.core.service.rule;

import com.abssqr.plat.common.dal.mysql.auto.paging.ScreenRulePage;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.enums.HandleTypeEnum;
import com.abssqr.plat.common.facade.enums.ScreenTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.ast.AstScreenRuleEditParam;
import com.abssqr.plat.common.facade.param.ast.AstScreenRulePageQryParam;
import com.abssqr.plat.common.model.convertor.AstScreenRuleConvert;
import com.abssqr.plat.common.model.convertor.page.PageConvert;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.core.service.change.ApproveCallBack;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 资产筛选规则
 *
 * @author yangcheng
 * @version AstScreenRuleManager.java, v0.12019-06-2401:16 yangchengExp$
 */
@Service
public class AstScreenRuleManager implements ApproveCallBack<AstScreenRule> {

    @Autowired
    AstScreenRuleRepo  astScreenRuleRepo;
    @Autowired
    ChangeManager      changeManager;
    @Autowired
    LoanProdRepository loanProdRepository;
    @Autowired
    PlanRepo           planRepo;

    /**
     * 分页查询获取筛选计划
     *
     * @param param
     * @return
     */
    public ScreenRulePage getScreenRuleList(AstScreenRulePageQryParam param, String userCode) {
        ScreenRulePage screenRulePage = PageConvert.toDBBasePate(ScreenRulePage.class, param, p -> {
            //如果是查询接口则加入身份作为条件
            if (HandleTypeEnum.LIST.getCode().equals(param.getHandleType())) {
                p.setUserCode(userCode);
            }
            // 查询审批完成的
            p.setApproveStatus(ApproveStatusEnum.PASSED.getCode());
            p.setLenderOrgCode(param.getLenderOrgCode());
            p.setScreenRuleName(param.getScreenRuleName());
            p.setAbssqrProdCode(param.getAbssqrProdCode());
            p.setCreateStart(param.getCreateStart());
            p.setCreateEnd(param.getCreateEnd());
            p.setScreenType(StringUtils.isNotEmpty(param.getScreenType())?param.getScreenType(): ScreenTypeEnum.AST_TRF.getCode());
        });
        return astScreenRuleRepo.getScreenRuleList(screenRulePage);
    }

    /**
     * 新增Or编辑资产筛选
     *
     * @param param
     */
    public String addOrUpdate(AstScreenRuleEditParam param, String userCode, String orgCode) {
        AstScreenRule astScreenRule = AstScreenRuleConvert.convertParam2Domain(param);

        int cnt = astScreenRuleRepo.getSameRuleCnt(astScreenRule.getScreenRuleNo(), astScreenRule.getScreenRuleName());
        VarChecker.checkEquals(cnt, 0, "存在相同筛选规则[{0}]", astScreenRule.getScreenRuleName());

        AstScreenRule rule = astScreenRuleRepo.getByNo(param.getScreenRuleNo());
        LoanProduct loanProduct = loanProdRepository.getBybssqrProdCode(param.getAbssqrProdCode());

        VarChecker.checkNotNull(loanProduct, "未找到匹配的产品[{0}]", param.getAbssqrProdCode());

        astScreenRule.setProdCode(loanProduct.getProdCode());
        astScreenRule.setProdName(loanProduct.getProdName());
        //转换成domain模型,后续有所改动
        astScreenRule.setManagerOrgCode(loanProduct.getManagerOrgCode());
        if (rule == null) {
            //审批状态默认为审批中
            astScreenRule.setApproveStatus(ApproveStatusEnum.PENDING);
            //如果是添加入参加入创建人
            astScreenRule.setGmtCreator(userCode);
            astScreenRuleRepo.addScreenRule(astScreenRule);
        } else {
            //如果存在判断审批状态，如果是在审批中或者审批完成，则不允许创建新的审批流
            ApproveChangeUtil.bizModification(rule.getApproveStatus(), rule.getScreenRuleNo(),
                rule.getScreenRuleName());
            //先核验在变更审批状态
            astScreenRule.setApproveStatus(ApproveStatusEnum.PENDING);
            //筛选规则如果是撤销状态下允许修改或者其他允许修改   and  加入修改人
            astScreenRule.setGmtModifier(userCode);
            astScreenRuleRepo.update(astScreenRule);
        }

        fillScreenRuleOrgName(astScreenRule);
        changeManager.addApprove(DataChangeBizTypeEnum.AstScreenRule,
            astScreenRule.getScreenRuleNo(),
            astScreenRule.getScreenRuleName(), astScreenRule, userCode, orgCode);
        return astScreenRule.getScreenRuleNo();
    }

    /**
     * 根据筛选规则编号获取筛选规则
     *
     * @param screenRuleNo
     * @return
     */
    public AstScreenRule getScreenRuleByNo(String screenRuleNo) {
        AstScreenRule astScreenRule = astScreenRuleRepo.getByNo(screenRuleNo);
        fillScreenRuleOrgName(astScreenRule);
        return astScreenRule;
    }

    private void fillScreenRuleOrgName(AstScreenRule astScreenRule) {
        if (astScreenRule != null && astScreenRule.getLenderOrgCode() != null) {
            String planName = planRepo.getPlanNames(Lists.newArrayList(astScreenRule.getLenderOrgCode())).get(
                astScreenRule.getLenderOrgCode());
            astScreenRule.setLenderOrgName(planName);
        }
    }

    /**
     * 根据 changeNo 查询数据
     *
     * @param changeNo
     * @return
     */
    public AstScreenRule getDataChangeScreenRuleBy(String changeNo) {
        BizDataChange<AstScreenRule> dataChange = changeManager.getDataChange(changeNo);
        if (dataChange == null) {
            throw new AbssqrBizException("未找到对应待审批的筛选规则数据");
        }
        dataChange.getDatas().setApproveStatus(ApproveStatusEnum.getEnum(dataChange.getApproveStatus()));
        return dataChange.getDatas();
    }

    /**
     * 审批完成后回调
     *
     * @param approveStatus
     * @param bizDataChange
     */
    @Override
    public void approveCallBack(BizDataChange<AstScreenRule> bizDataChange, ApproveStatusEnum approveStatus) {
        //回调更新五级分类
        AstScreenRule astScreenRule = bizDataChange.getDatas();
        astScreenRule.setApproveStatus(approveStatus);
        //update更新业务表数据
        astScreenRuleRepo.update(astScreenRule);
    }

    @Override
    public DataChangeBizTypeEnum getChangeType() {
        return DataChangeBizTypeEnum.AstScreenRule;
    }

}
