/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.util;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanInstalFormationItem;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.result.analyse.FormationAstItemVO;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.enums.TermCntCalcEnum;
import com.abssqr.plat.common.util.base.Day;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 组装数据工具
 * @author hanfei
 * @version $Id: AnalyseReportUtil.java, v 0.1 2019-09-27 10:56 AM hanfei Exp $
 */
public class AnalyseReportUtil {
    
    /**
     * 获取金额便捷方法
     * @param itemMap
     * @param field
     * @return
     */
    public static Money getItemMoney(Map<String, PlanReportItem> itemMap, ReportField... field){
        return getItemMoney(itemMap, CollectionUtil.newArrayList(field));
    }
    /**
     * 获取金额便捷方法
     * @param itemMap
     * @param fields
     * @return
     */
    public static Money getItemMoney(Map<String, PlanReportItem> itemMap, List<ReportField> fields){
        Money amt = new Money();
        if (ToolUtil.isEmpty(itemMap)){
            return amt;
        }
        for(ReportField field : fields){
            PlanReportItem item = itemMap.get(field.getCode());
            if (item == null){
                continue;
            }
            amt.addTo(new Money(item.getAmt()));
        }
        return amt;
    }
    
    /**
     * 获取金额便捷方法
     * @param itemMap
     * @param field
     * @return
     */
    public static BigDecimal getItemBig(Map<String, PlanReportItem> itemMap, ReportField field){
        return getItemBig(itemMap, CollectionUtil.newArrayList(field));
    }
    /**
     * 获取金额便捷方法
     * @param itemMap
     * @param fields
     * @return
     */
    public static BigDecimal getItemBig(Map<String, PlanReportItem> itemMap, List<ReportField> fields){
        BigDecimal amt = BigDecimal.ZERO;
        if (ToolUtil.isEmpty(itemMap)){
            return amt;
        }
        for(ReportField field : fields){
            PlanReportItem item = itemMap.get(field.getCode());
            if (item == null){
                continue;
            }
            amt = NumberUtil.add(amt, item.getAmt());
        }
        return amt;
    }
    
    /**
     * 按字段相同项合并
     * @param planNo
     * @param spvReportItemMap
     * @param planReportItemMap
     * @param field
     */
    public static void summaryByField(String planNo, Map<String, PlanReportItem> spvReportItemMap,
                                      Map<String, PlanReportItem> planReportItemMap, ReportField field){
        PlanReportItem spvItem = spvReportItemMap.get(field.getCode());
        if (spvItem == null){
            spvItem = new PlanReportItem();
            spvItem.setRptItemCode(field.getCode());
            spvItem.setRptItemName(field.getName());
            spvItem.setPlanNo(planNo);
            spvReportItemMap.put(field.getCode(), spvItem);
        }
        PlanReportItem planItem = planReportItemMap.get(field.getCode());
        if (planItem == null){
            return;
        }
        // 合并统计项
        AnalyseReportUtil.summary(spvItem, planItem);
    }
    /**
     * 相同项合并
     * @param total
     * @param item
     */
    public static void summary(PlanReportItem total, PlanReportItem item){
        if (StrUtil.equals(total.getRptItemCode(), item.getRptItemCode())){
            total.setAmt(new Money(total.getAmt()).addTo(new Money(item.getAmt())).getAmount());
        }
    }
    
    /**
     * 获取统计日
     * @param accountDate null 时 默认 D-1日
     * @return
     */
    public static Date getStatDate(Day trfDate, String accountDate, PlanReport lastReport){
        Date lastDate;
        if (lastReport != null){
            lastDate = lastReport.getRptDate().getAccountDate();
        }else{
            // 取转让会计日的上一日
            lastDate = trfDate.getYesterday().getStandardDate();
        }
        // 统计日 默认D-1日
        Date accDate = ToolUtil.isEmpty(accountDate) ? lastDate : DateUtil.parseDate(accountDate);
        VarChecker.checkArgument(DateTimeUtil.calcDayDiff(accDate, lastDate) <= 0,
                "[{0}]统计日期不能超过[{1}]", accDate, DateUtil.formatDate(lastDate));
        return accDate;
    }
    
    /**
     * 获取放款池对应承接池信托计划规模
     * @param planBases
     * @return
     */
    public static Money getPlanTotalAmtBySpvPlanNo(PlanBase planBase, List<PlanBase> planBases){
        // 总规模合计
        Money totalAmt = RptBuilderUtil.getInvestedAmt(planBase);
        if (ToolUtil.isNotEmpty(planBases)){
            planBases.forEach(one -> {
                totalAmt.addTo(RptBuilderUtil.getInvestedAmt(one));
            });
        }
        return totalAmt;
    }
    
    /**
     * 资产结构展示处理
     * @param planNo
     * @param scale
     * @param astFields
     * @param itemMap
     * @return
     */
    public static List<FormationAstItemVO> convertFormation(String planNo, Money scale,
                                                            List<ReportField> astFields, Map<String, PlanReportItem> itemMap) {
        // 初始化一波
        List<FormationAstItemVO> itemVOList = initItemVO( planNo, itemMap, astFields);
        // 排序
        itemVOList.sort(Comparator.comparing(FormationAstItemVO::getAstScale));
        // 取整数100
        BigDecimal scaler = new BigDecimal(100);
        for(int i = 0; i < itemVOList.size(); i++){
            FormationAstItemVO vo = itemVOList.get(i);
            // 排名
            vo.setSequence(itemVOList.size()-i);
            // 占比
            vo.setAstRate(CalcUtil.dividePercRate(vo.getAstScale().getAmount(), scale.getAmount()));
            // 计算避免出头或短账
            scaler = scaler.subtract(vo.getAstRate());
            // 最后补足
            if(i == itemVOList.size()-1){
                vo.setAstRate(vo.getAstRate().add(scaler));
            }
        }
        // 按范围排序
        itemVOList.sort(Comparator.comparing(FormationAstItemVO::getSort));
        return itemVOList;
    }
    
    private static List<FormationAstItemVO> initItemVO(String planNo, Map<String, PlanReportItem> itemMap, List<ReportField> astFields){
        List<FormationAstItemVO> itemVOList = new ArrayList<>(astFields.size());
        for(ReportField field : astFields){
            FormationAstItemVO vo = new FormationAstItemVO();
            vo.setPlanNo(planNo);
            vo.setTermRange(field.getName());
            vo.setAstScale(getItemMoney(itemMap, field));
            vo.setSort(TermCntCalcEnum.matchTermStart(field));
            itemVOList.add(vo);
        }
        return itemVOList;
    }
    /**
     * 资产结构列表转换
     * @param planNo
     * @param datas
     * @return
     */
    public static List<FormationAstItemVO> convertFormation(String planNo, List<LoanInstalFormationItem> datas) {
        FormationAstItemVO item1 = new FormationAstItemVO();
        item1.setPlanNo(planNo);
        item1.setTermRange("1期");
        FormationAstItemVO item1_3 = new FormationAstItemVO();
        item1_3.setPlanNo(planNo);
        item1_3.setTermRange("1-3期");
        FormationAstItemVO item3_6 = new FormationAstItemVO();
        item3_6.setPlanNo(planNo);
        item3_6.setTermRange("3-6期");
        FormationAstItemVO item6_12 = new FormationAstItemVO();
        item6_12.setPlanNo(planNo);
        item6_12.setTermRange("6-12期");
        FormationAstItemVO item12 = new FormationAstItemVO();
        item12.setPlanNo(planNo);
        item12.setTermRange("12期及以上");
        Money astScaleBal = new Money();
        // 按区间分别汇总
        datas.forEach(one -> {
            astScaleBal.addTo(one.getAmtBal());
            switch (one.getTermCount()){
                case 1:
                    item1.getAstScale().addTo(one.getAmtBal());
                    break;
                case 2:
                case 3:
                    item1_3.getAstScale().addTo(one.getAmtBal());
                    break;
                case 4:
                case 5:
                case 6:
                    item3_6.getAstScale().addTo(one.getAmtBal());
                    break;
                case 7:
                case 8:
                case 9:
                case 10:
                case 11:
                case 12:
                    item6_12.getAstScale().addTo(one.getAmtBal());
                    break;
                default:
                    item12.getAstScale().addTo(one.getAmtBal());
            }
        });
        List<FormationAstItemVO> itemVOs = CollectionUtil.newArrayList(item1, item1_3, item3_6, item6_12, item12);
        // 排序
        itemVOs.sort(Comparator.comparing(FormationAstItemVO::getAstScale));
        for(int i=0; i< itemVOs.size(); i++){
            FormationAstItemVO vo = itemVOs.get(i);
            // 排名
            vo.setSequence(itemVOs.size()-i);
            // 占比
            vo.setAstRate(CalcUtil.dividePercRate(vo.getAstScale().getAmount(), astScaleBal.getAmount()));
        }
        return CollectionUtil.newArrayList(item1, item1_3, item3_6, item6_12, item12);
    }
    
    /**
     * 二维数组计算 未来日放款金额的还款金额合计
     * @param grantAmt
     * @param repayRate
     * @param radix
     * @return
     */
    @Deprecated
    private static Money calcRepayArray(Money grantAmt, BigDecimal repayRate, int radix){
        Money[][] array = new Money[radix][radix+1];
        // 第1行初始化基数
        for (int i = 0; i < array.length; i++) {
            array[i][0] = grantAmt.clone();
        }
        int s = 0;
        for (int i = 0; i < array.length; i++) {
            for (int j = 1+s; j < array[i].length; j++) {
                array[i][j] = array[i][0].multiply(repayRate);
                if (array[i][0].isZero()){
                    continue;
                }
                array[i][0].subtractFrom(array[i][j]);
                if (array[i][0].isNegtive()){
                    array[i][0] = new Money();
                }
            }
            s++;
        }
        // 计算还款总金额
        Money repayTotal = new Money();
        for (int i = 0; i < array.length; i++) {
            for (int j = 1; j < array[i].length; j++) {
                if (ToolUtil.isNotEmpty(array[i][j])){
                    repayTotal.addTo(array[i][j]);
                }
            }
        }
        return repayTotal;
    }
    
}
