/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.util;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanBakFormationItem;
import com.abssqr.plat.common.facade.enums.ReportTitleGroupEnum;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.ReporterItemCodeKey;
import com.abssqr.plat.common.facade.model.report.Statistics;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.model.domain.plan.PlanAcctTitleLog;
import com.abssqr.plat.common.model.enums.TermCntCalcEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.core.service.report.ReportItemHolder;
import com.general.system.common.model.Money;
import com.general.system.common.util.MessageUtil;
import com.general.system.tool.util.ToolUtil;
import org.apache.commons.collections.CollectionUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 报表生成基础工具
 * @author hanfei
 * @version $Id: RptBuilderUtil.java, v 0.1 2019-11-19 3:55 PM hanfei Exp $
 */
public class RptBuilderUtil {
    
    /**
     * 获取计划层级结构投入资本本金余额
     * @param plan      计划
     * @return
     */
    public static Money getInvestedAmt(PlanBase plan) {
        if (ToolUtil.isEmpty(plan.getInvestStructureList())){
            throw new AbssqrBizException(MessageUtil.formatMsg("[{0}]未取到项目计划的投资结构", plan.getPlanNo()));
        }
        // 层级结构总额
        Money investAmt = new Money();
        plan.getInvestStructureList().forEach(one -> {
            // 累计层级投入规模
            investAmt.addTo(one.getLayerAmt());
        });
        return investAmt;
    }
    
    /**
     * 构建数据map
     *
     * @param planReportItems
     * @param reportEntityMap
     */
    public static void recordedData(List<PlanReportItem> planReportItems,
                              Map<ReporterItemCodeKey, Statistics> reportEntityMap) {
        if (CollectionUtils.isEmpty(planReportItems)) {
            return;
        }
        planReportItems.stream().forEach(item -> {
            ReporterItemCodeKey reporterKey = new ReporterItemCodeKey(item.getPlanNo(), item.getRptItemCode());
            Statistics statistics = new Statistics(item.getAmt(), item.getAstCnt());
            reportEntityMap.put(reporterKey, statistics);
        });
        
    }
    
    /**
     * 构建差额统计项
     *
     * @param planNo
     * @param accountDate
     * @param field
     * @param cnt
     * @param amt
     * @param lastPlanReport
     * @return
     */
    public static PlanReportItem buildDiffReportItem(String planNo, Date accountDate, ReportField field,
                                                     long cnt, BigDecimal amt, PlanReport lastPlanReport) {
        long diffCnt = 0;
        BigDecimal diffAmt = BigDecimal.ZERO;
        if (lastPlanReport != null) {
            diffCnt = cnt - getReportCnt(lastPlanReport, field.getCode());
            diffAmt = amt.subtract(getReportVal(lastPlanReport, field.getCode()));
        }
        return buildReportItem(planNo, accountDate, field, diffCnt, diffAmt);
    }
    
    /**
     * 构建统计项
     *
     * @param planNo
     * @param accountDate
     * @param field
     * @param cnt
     * @param amt
     * @return
     */
    public static PlanReportItem buildReportItem(String planNo, Date accountDate, ReportField field,
                                                 Long cnt, BigDecimal amt) {
        PlanReportItem reportItem = new PlanReportItem();
        reportItem.setPlanNo(planNo);
        reportItem.setRptItemCode(field.getCode());
        reportItem.setRptItemName(field.getName());
        reportItem.setAstCnt(cnt);
        reportItem.setAmt(amt);
        reportItem.setAccountDate(accountDate);
        return reportItem;
    }
    
    /**
     * 取amt
     * @param reportItemMap
     * @param field
     * @return
     */
    public static BigDecimal getReportVal(Map<String, PlanReportItem> reportItemMap, String field) {
        if (ToolUtil.isOneEmpty(reportItemMap, reportItemMap.get(field))) {
            return BigDecimal.ZERO;
        } else {
            return reportItemMap.get(field).getAmt();
        }
    }
    /**
     * 取数量
     * @param reportItemMap
     * @param field
     * @return
     */
    public static Long getReportCnt(Map<String, PlanReportItem> reportItemMap, String field) {
        if (ToolUtil.isOneEmpty(reportItemMap, reportItemMap.get(field))) {
            return 0L;
        } else {
            return reportItemMap.get(field).getAstCnt();
        }
    }
    
    /**
     * 取amt
     * @param report
     * @param field
     * @return
     */
    public static BigDecimal getReportVal(PlanReport report, String field) {
        if (ToolUtil.isEmpty(report) || ToolUtil.isOneEmpty(report.getReportItemMap(), report.getReportItemMap().get(field))) {
            return BigDecimal.ZERO;
        } else {
            return report.getReportItemMap().get(field).getAmt();
        }
    }
    
    /**
     * 取数量
     * @param report
     * @param field
     * @return
     */
    public static Long getReportCnt(PlanReport report, String field) {
        if (ToolUtil.isEmpty(report) || ToolUtil.isOneEmpty(report.getReportItemMap(), report.getReportItemMap().get(field))) {
            return 0L;
        } else {
            return report.getReportItemMap().get(field).getAstCnt();
        }
    }
    
    /**
     * 获取科目流水发生额
     * @param acctTitleLogMap
     * @param titleGroup
     * @return
     */
    public static Money getAcctTitleMoney(Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> acctTitleLogMap, ReportTitleGroupEnum titleGroup) {
        List<PlanAcctTitleLog> titleLogs = acctTitleLogMap.get(titleGroup);
        Money amt = new Money();
        if (ToolUtil.isNotEmpty(titleLogs)) {
            titleLogs.forEach(one -> {
                amt.addTo(one == null ? new Money() : one.getAcctAmt());
            });
        }
        return amt;
    }
    
    /**
     * 获取计划层级结构投入资本本金余额
     * @param plan      计划
     * @param allocAmt  兑付发生额
     * @return
     */
    public static Money getInvestedBal(PlanBase plan, BigDecimal allocAmt) {
        // 层级结构
        Money investAmt = getInvestedAmt(plan);
        // 减已兑付本金总额
        investAmt.subtractFrom(new Money(allocAmt));
        return investAmt;
    }
    
    /**
     * 获取投资层级结构投入资本本金 与存续期加权总额
     * @param accountDate
     * @param plan
     * @param titleLogs
     * @return
     */
    public static Money getInvestedWeight(Date accountDate, PlanBase plan, List<PlanAcctTitleLog> titleLogs) {
        return getInvestedWeight(accountDate, plan, titleLogs, false);
    }
    
    /**
     * 获取投资层级结构投入资本本金 与存续期加权总额
     * @param accountDate
     * @param plan
     * @param titleLogs
     * @param subInvested  是否扣减保证金
     * @return
     */
    public static Money getInvestedWeight(Date accountDate, PlanBase plan, List<PlanAcctTitleLog> titleLogs, boolean subInvested) {
        // 按照关联编号=层级结构编号 进行分组
        Map<String , List<PlanAcctTitleLog>> titleLogsMap = titleLogs.stream().collect(
                Collectors.groupingBy(PlanAcctTitleLog::getRefNo));
        // 层级结构
        List<InvestStructure> investList = plan.getInvestStructureList();
        // 累计投资结构权重金额
        Money investWeightAmt = new Money();
        // 根据层级结构及兑付流水拆解计算投入本金占用期限资金权重
        for(InvestStructure invest : investList){
            if (ToolUtil.isOneEmpty(invest.getEffectDate(), invest.getLayerEndDate())){
                throw new AbssqrBizException(MessageUtil.formatMsg("计划[{0}]投资结构[{1}]日期错误[S:{2} - E:{3}]",
                        plan.getPlanNo(), invest.getLayerNo(), invest.getEffectDate(), invest.getLayerEndDate()));
            }
            Money investAmt = invest.getLayerAmt().clone();
            // 需要扣减 1%保证金金额
            if (subInvested){
                investAmt.subtractFrom(investAmt.multiply(0.01));
            }
            // 拆分投资结构本金
            if (ToolUtil.isNotEmpty(titleLogsMap)
                && titleLogsMap.containsKey(invest.getLayerNo())
                && ToolUtil.isNotEmpty(titleLogsMap.get(invest.getLayerNo()))){
                List<PlanAcctTitleLog> investTitleLogs = titleLogsMap.get(invest.getLayerNo());
                investTitleLogs.forEach(title -> {
                    // 兑付日|截止日 取小
                    Date endDate = title.getAccountDate().compareTo(invest.getLayerEndDate()) < 0
                                           ? title.getAccountDate() : invest.getLayerEndDate();
                    // 计算已兑付本金占用期期限资金权重  兑付发生额 * (min(兑付日|截止日) - 起息日)
                    investWeightAmt.addTo(title.getAcctAmt().multiply(
                            DateUtil.betweenDay(invest.getEffectDate(), endDate, true) + 1));
                    // 扣减发生额
                    investAmt.subtractFrom(title.getAcctAmt());
                });
            }
            
            if (investAmt.isGreaterThanZero()){
                // 会计日|到期日 取小
                Date endDate = accountDate.compareTo(invest.getLayerEndDate()) < 0
                                       ? accountDate : invest.getLayerEndDate();
                // 累计层级投入规模 * 存续期天数
                investWeightAmt.addTo(investAmt.multiply(
                        DateUtil.betweenDay(invest.getEffectDate(), endDate, true) + 1));
            }
        }
        return investWeightAmt;
    }
    
    /**
     * 资产结构列表转换
     * @param planNo
     * @param items
     * @return
     */
    public static List<PlanReportItem> convertFormation(String planNo, Date accountDate,
                                                        List<LoanBakFormationItem> items, boolean isProd) {
        // 计算项目
        TermCntCalcEnum[] calcs = TermCntCalcEnum.values();
        Map<TermCntCalcEnum, PlanReportItem> reportItemMap = new HashMap<>();
        for (TermCntCalcEnum calc : calcs){
            // 创建初始条目
            PlanReportItem item = buildReportItem(planNo, accountDate,
                    isProd ? calc.getProdField() : calc.getAstField(), 0L, BigDecimal.ZERO);
            reportItemMap.put(calc, item);
        }
        if (ToolUtil.isNotEmpty(items)){
            // 按区间分别汇总
            items.forEach(one -> {
                for (TermCntCalcEnum calc : calcs){
                    PlanReportItem item = reportItemMap.get(calc);
                    // 匹配
                    if (item != null && calc.isMatch(one.getTermCnt())){
                        item.summary(one.getAstCnt(), one.getAmtBal().getAmount());
                    }
                }
            });
        }
        return reportItemMap.values().stream().collect(Collectors.toList());
    }
    
    /**
     * 承接池获取已实现收益额
     * = 闲置资金+保证金+已兑付本金+已兑付利息+兑付税费+总贷款本金余额
     *      +（总贷款利息余额-利息余额的金融服务费）-层级累计规模-逾期91（含）天本金余额-转让当日应计未付利息
     * @return
     */
    public static Money getPlanProfitAmt(PlanBase planBase) {
        Money profitAmt = getProfitAmt(planBase);
        // 转让当日应计未付利息
        profitAmt.subtractFrom(new Money(
                ReportItemHolder.getPlanReportItemHolder(planBase.getPlanNo(), WkAstReportFields.WK132)));
        return profitAmt;
    }
    
    /**
     * 放款池获取已实现收益额
     * = 闲置资金+保证金+已兑付本金+已兑付利息+兑付税费+总贷款本金余额
     *      +（总贷款利息余额-应计提利息余额的金融服务费）-层级累计规模-逾期91（含）天本金余额
     *      + 转让当日应计未收利息和 + 还款服务费
     * @return
     */
    public static Money getSpvProfitAmt(PlanBase planBase) {
        Money profitAmt = getProfitAmt(planBase);
        // 转让当日应计未收利息和
        profitAmt.addTo(new Money(ReportItemHolder.getPlanReportItemHolder(planBase.getPlanNo(), WkAstReportFields.WK132)));
        // 实还还款服务费
        profitAmt.addTo(new Money(ReportItemHolder.getPlanReportItemHolder(planBase.getPlanNo(), WkAstReportFields.WK139)));
        return profitAmt;
    }
    
    /**
     * 闲置资金 + 保证金 + 已兑付金额 + 总贷款本金余额 + 总贷款利息余额 - 应计提利息余额的金融服务费 - 层级累计规模 - 逾期91（含）天本金余额
     * @param planBase
     * @return
     */
    private static Money getProfitAmt(PlanBase planBase) {
        String planNo = planBase.getPlanNo();
        // 计划投资层级规模
        Money planInvested = getInvestedAmt(planBase);
        Money profitAmt = new Money();
        // + 闲置资金
        profitAmt.addTo(new Money(ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK001)));
        // + 保证金 = 层级累计规模 * 1%
        profitAmt.addTo(planInvested.multiply(0.01));
        // + 已兑付本金 + 已兑付利息 + 兑付税费
        profitAmt.addTo(new Money(ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK135)));
        // + 总贷款本金余额
        profitAmt.addTo(new Money(ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK017)));
        // + 总贷款利息余额 - 应计提利息余额的金融服务费
        profitAmt.addTo(new Money(ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK018)
                        .multiply(NumberUtil.sub(1, planBase.getFinSerFeeRate()))));
        // - 层级累计规模
        profitAmt.subtractFrom(planInvested);
        // - 逾期91（含）天本金余额
        profitAmt.subtractFrom(new Money(ReportItemHolder.getPlanReportItemHolder(planNo, WkAstReportFields.WK025)));
        return profitAmt;
    }
    
}
