package com.abssqr.plat.test.unittest.workflow;

import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.BizDataChangeDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.BizDataChangeDO;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderPageParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQprParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderBtn;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderVo;
import com.abssqr.plat.common.facade.service.WorkOrderService;
import com.abssqr.plat.common.facade.service.plan.PlanService;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.enums.WorkOrderBtnEnum;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.bpm.domain.result.ProcessSelStartResult;
import com.general.system.core.common.constant.factory.ConstantFactory;
import com.general.system.tool.util.ToolUtil;
import org.junit.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.util.List;

public class PlanWorkOrderTest extends BaseAutoTests {

    @Autowired
    private WorkOrderService workOrderService;
    @Autowired
    private PlanService      planService;
    @Autowired
    private BizDataChangeDAO bizDataChangeDAO;
    @Autowired
    private AcctDateDAO      acctDateDAO;

    @XTest(relatePath = "unittest/workOrder/plan/add")
    @Test(dataProvider = "YamlDataProvider", description = "发起工单审批", priority = 1)
    public void createWorkOrderPlan(ICase iCase, String acctDate, PlanOprParam param) {
        try {
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
    
            ConstantFactory.me().securityLogin("administrator");
            // 创建产品及工单
            String planNo = planService.edit(param);
            BizDataChangeDO bizData = bizDataChangeDAO.getByBizNoAndStatus(planNo, "PENDING");
    
            // 我的待办
            ConstantFactory.me().securityLogin("042CF5A0-0732-40A6-9499-EA99E6AA5983");
            WorkOrderPageParam pageParam = new WorkOrderPageParam();
            BasePage<WorkOrderVo> my = workOrderService.myWorkOrder(pageParam);
            String checkBizNo = null;
            for (WorkOrderVo vo : my.getDatas()){
                if (ToolUtil.equals(vo.getBusinessKey(), bizData.getChangeNo())){
                    checkBizNo = vo.getBusinessKey();
                    break;
                }
            }
            Assert.assertNotNull(checkBizNo);
    
            // 我的申请
            ConstantFactory.me().securityLogin("administrator");
            BasePage<WorkOrderVo> apply = workOrderService.applyWorkOrder(pageParam);
            Assert.assertNotNull(apply.getDatas());
    
            // 已办事项
            ConstantFactory.me().securityLogin("administrator");
            BasePage<WorkOrderVo> his = workOrderService.hisWorkOrder(pageParam);
            Assert.assertNotNull(his.getDatas());
    
            WorkOrderQryParam qryParam = new WorkOrderQryParam();
            qryParam.setBusinessKey(bizData.getChangeNo());
            // 查看已申请信息 包含 实例信息、历史记录及追踪图
            ProcessSelStartResult detail = workOrderService.detail(qryParam);
            Assert.assertNotNull(detail);
            // 查询办理轨迹
            List<WorkOrderVo> tasks = workOrderService.getWorkOrderTasks(qryParam);
            Assert.assertNotNull(tasks);
            // 详情页对应的按钮数据
            List<WorkOrderBtn> btns = workOrderService.getBtns(qryParam);
            Assert.assertNotNull(btns);
            // 获取业务类型枚举
            DataChangeBizTypeEnum typeEnum = workOrderService.getBizType(qryParam);
            Assert.assertTrue(typeEnum == DataChangeBizTypeEnum.PLAN);
        }finally {
            super.checkDB(iCase);
    
            super.clearDB(iCase);
        }
    }


    @XTest(relatePath = "unittest/workOrder/plan/qpr")
    @Test(dataProvider = "YamlDataProvider", description = "工单操作入口", priority = 1)
    public void operatePlan(ICase iCase, String acctDate, PlanOprParam param, WorkOrderQprParam qprParam) {
        try{
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
    
            ConstantFactory.me().securityLogin("administrator");
            // 创建产品及工单
            String planNo = planService.edit(param);
            BizDataChangeDO bizData = bizDataChangeDAO.getByBizNoAndStatus(planNo, "PENDING");
            
            qprParam.setBusinessKey(bizData.getChangeNo());
            
            // 工单操作  通过 撤销 及 拒绝
            if (WorkOrderBtnEnum.complete == WorkOrderBtnEnum.getByCode(qprParam.getOperate())){
                ConstantFactory.me().securityLogin("042CF5A0-0732-40A6-9499-EA99E6AA5983");
                workOrderService.operate(qprParam);
    
                ConstantFactory.me().securityLogin("B7C6C710-6ADB-4575-A987-A44F30E77A98");
                workOrderService.operate(qprParam);
                
            }else if (WorkOrderBtnEnum.refuse == WorkOrderBtnEnum.getByCode(qprParam.getOperate())){
                
                ConstantFactory.me().securityLogin("042CF5A0-0732-40A6-9499-EA99E6AA5983");
                workOrderService.operate(qprParam);
            }else if (WorkOrderBtnEnum.revoke == WorkOrderBtnEnum.getByCode(qprParam.getOperate())){
    
                ConstantFactory.me().securityLogin("administrator");
                workOrderService.operate(qprParam);
            }
    
            // 办理人员的已完成事项
            WorkOrderPageParam pageParam = new WorkOrderPageParam();
            BasePage<WorkOrderVo> his = workOrderService.hisWorkOrder(pageParam);
            Assert.assertNotNull(his.getDatas());
            
//            putDbCkParam("abssqrProdCode", prod.getAbssqrProdCode());
        }catch (Exception e){
    
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
    
   
}
