/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.web.config;

import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.general.system.bpm.config.ActivitiConfiguration;
import com.general.system.bpm.domain.ProcessBasePage;
import com.general.system.bpm.facade.ConfigProcessVO;
import com.general.system.common.dal.auto.paging.SysRoleListPage;
import com.general.system.common.dal.auto.paging.SysUserListPage;
import com.general.system.core.common.constant.factory.ConstantFactory;
import com.general.system.facade.current.IdentityUtil;
import com.general.system.modular.result.UserResult;
import com.general.system.modular.service.IRoleService;
import com.general.system.modular.service.IUserService;
import com.general.system.tool.util.ToolUtil;
import org.activiti.engine.identity.Group;
import org.activiti.engine.identity.User;
import org.activiti.engine.impl.persistence.entity.GroupEntity;
import org.activiti.engine.impl.persistence.entity.UserEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 注册流程配置及相关必要查询接口
 * @author hanfei
 * @version $Id: EnableActivitiConfiguration.java, v 0.1 2019-06-18 3:46 PM hanfei Exp $
 */
@Component
@Configuration
public class EnableActivitiConfiguration extends ActivitiConfiguration {

    @Override
    public String getUserCode() {
        return IdentityUtil.getUserCode();
    }

    @Override
    public String getUserName(String userCode) {
        return IdentityUtil.getUserByCode(userCode).getUserName();
    }

    @Override
    public String getOrgCode(String userCode) {
        return IdentityUtil.getUserByCode(userCode).getOrgCode();
    }

    @Override
    public String getOrgName(String orgCode) {
        return IdentityUtil.getDeptByCode(orgCode).getSimpleName();
    }
    
    /**
     * 获取角色名称
     * @param roleCode
     * @return
     */
    @Override
    public String getRoleName(String roleCode) {
        return ConstantFactory.me().getSingleRoleName(roleCode);
    }
    
    /**
     * 根据用户编码查询用户
     * @param userCode
     * @return
     */
    @Override
    public User findUserById(String userCode) {
        UserResult userResult = IdentityUtil.getUserByCode(userCode);
        UserEntity userEntity = new UserEntity();
        userEntity.setId(userResult.getUserCode());
        userEntity.setFirstName(userResult.getUserName());
        userEntity.setLastName(userResult.getDeptName());
        userEntity.setEmail(userResult.getEmail());
        return userEntity;
    }
    
    @Autowired
    private IUserService userService;
    @Autowired
    private IRoleService roleService;
    
    /**
     * 查询用户
     * @param basePage
     * @param userName
     * @param groupCode
     * @param groupId
     */
    @Override
    public void findUserList(ProcessBasePage<User> basePage, String userName, String groupCode , String groupId) {
        SysUserListPage page = new SysUserListPage();
        page.setLimit(basePage.getLimit());
        page.setCurrPageNo(basePage.getCurrPageNo());
        page.setUserName(userName);
        page.setRoleCode(groupCode);
        userService.queryUserPage(page);
        basePage.setDatas(convertUser(page.getDatas()));
    }
    
    /**
     * 查询角色
     * @param basePage
     * @param groupName
     * @param userId
     */
    @Override
    public void findGroupsByUser(ProcessBasePage<Group> basePage, String groupName, String userId) {
        SysRoleListPage page = new SysRoleListPage();
        page.setLimit(basePage.getLimit());
        page.setCurrPageNo(basePage.getCurrPageNo());
        page.setRoleName(groupName);
        // 查询用户角色
        if (ToolUtil.isNotEmpty(userId)){
            UserResult result = ConstantFactory.me().getUserByCode(userId);
            if (ToolUtil.isEmpty(result.getRoleCodes())){
                return;
            }
            page.setListRoleCode(result.getRoleCodes());
        }
        roleService.queryRolePage(page);
        basePage.setDatas(convertRole(page.getDatas()));
    }
    
    /**
     * 注册配置流程的业务类型
     * @return
     */
    @Override
    public List<ConfigProcessVO> getBizConfigList() {
        DataChangeBizTypeEnum[] process = DataChangeBizTypeEnum.values();
        List<ConfigProcessVO> configs = new ArrayList<>(process.length);
        List<String> keys = new ArrayList<>(process.length);
        for(DataChangeBizTypeEnum p : process){
            if (keys.contains(p.getActiveCode())){
                continue;
            }
            ConfigProcessVO vo = new ConfigProcessVO();
            vo.setBusinessDesc(p.getDesc());
            vo.setBusinessCode(p.getActiveCode());
            keys.add(p.getActiveCode());
            configs.add(vo);
        }
        return configs;
    }
    
    /**
     * 转换用户
     * @param items
     * @return
     */
    private List<User> convertUser(List items){
        if (ToolUtil.isEmpty(items)){
            return new ArrayList<>();
        }
        List<User> list = new ArrayList<>(items.size());
        for(Map<String, String> one : (List<Map>) items){
            UserEntity user = new UserEntity();
            user.setId(one.get("userCode"));
            user.setFirstName(one.get("userName"));
            user.setLastName(one.get("account"));
            user.setEmail(one.get("email"));
            list.add(user);
        }
        return list;
    }
    
    /**
     * 转换角色
     * @param items
     * @return
     */
    private List<Group> convertRole(List items){
        if (ToolUtil.isEmpty(items)){
            return new ArrayList<>();
        }
        List<Group> list = new ArrayList<>();
        for(Map<String, String> one : (List<Map>) items){
            GroupEntity entity = new GroupEntity();
            entity.setId(one.get("roleCode"));
            entity.setName(one.get("roleName"));
            entity.setType(one.get("roleType"));
            list.add(entity);
        }
        return list;
    }
}
